#!/usr/bin/php
<?php
require_once '/usr/local/esia/plugins/lib/CacheFile.class.php';
include_once '/usr/local/esia/public_html//esia.conf.php';
try
{
	$dbconnection = new PDO('pgsql:host='.DB_HOST.';port='.DB_PORT.';dbname='.DB_NAME, DB_USERNAME, DB_PWD);
	$dbconnection->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
}
catch(Exception $e)
{
	echo 'PDO SQL ERROR: '.$e->getMessage().'<br />';
	echo 'N° : '.$e->getCode();
	exit(3);
}

$STR_STATUS=array("OK","WARNING","CRITICAL","UNKNOWN");
$INT_STATUS=array_flip($STR_STATUS);
$PRIORITIES_STATES=array("CRITICAL","WARNING","UNKNOWN","OK");
$short_opts="h:i:w:c:";
$long_opts=array("help","warning:","critical:");

$options=getopt($short_opts,$long_opts);

function print_usage($bad_usage){
        echo 'USAGE : '.basename($_SERVER['PHP_SELF'])." -w <WARN> -c <CRITIC>\n";
        if($bad_usage) echo 'HELP  : '.basename($_SERVER['PHP_SELF'])." -h\n";
}

function print_help(){
        $col=20;
        print_usage(false);
        echo "\n";
        echo "Check if the CVE list are up to date\n\n";
        echo "\t-w, ".str_pad("--warning=INTEGER",$col)         ."for warning treshold\n";
        echo "\t-c, ".str_pad("--critical=INTEGER",$col)        ."for critical treshold\n";
        echo "\n";
        echo "\t-h, ".str_pad("--help",$col)                    ."Print this help\n";
        echo "\n";
}


function exit_state($state,$msg=null,$usage=false){
        $STR_STATUS=$GLOBALS["STR_STATUS"];
        $INT_STATUS=$GLOBALS["INT_STATUS"];

        $ret=$state;
        if( is_string($state) ) $ret=$INT_STATUS[strtoupper($state)];
        else $state=$STR_STATUS[$ret];

        if( $msg != null ) echo "$state - $msg\n";
        if( $usage ) print_usage(true);
        exit($ret);
}

function getParam($name1,$name2,$default=null){
        global $options;

        if( isset( $options[$name1] ) ) return $options[$name1];
        if( isset( $options[$name2] ) ) return $options[$name2];
        return $default;
}

function calcul_state($a,$b=null){
	$STR_STATUS=$GLOBALS["STR_STATUS"];
	$INT_STATUS=$GLOBALS["INT_STATUS"];
	$PRIORITIES_STATES=$GLOBALS["PRIORITIES_STATES"];

	if(is_string($a) ) $a=$INT_STATUS[strtoupper($a)];
	if(is_string($b) ) $b=$INT_STATUS[strtoupper($b)];

	foreach($PRIORITIES_STATES as $value){
		$value = $INT_STATUS[$value];

		if( $value == $a || $value == $b ) return $value;
	}
	return $INT_STATUS["UNKNOWN"];
}

//traitement

$warning = getParam("w","warning");
$critical = getParam("c","critical");


if ( isset($options["help"]) || isset($options["h"]) ){
        print_help();
        exit($INT_STATUS["UNKNOWN"]);
}

if( $warning == null ) exit_state(3,"You forget to specify warning.",true);
if( $critical == null ) exit_state(3,"You forget to specify critical.",true);
if( $warning < 0 || $critical < $warning ) exit_state(3,"You forget to specify valid treshold warning & critical.",true);


$last_modify_cve = getLastCVEUpdated($dbconnection);

$cmd = 'dpkg -l |grep esia-egide-cve| tr -s " "|cut -d " " -f3';
$output = exec($cmd);
$global_state = 'UNKNOWN';
if( is_numeric($output ) )
{
	$date = DateTime::createFromFormat('ymdH', $output);
	$date_ts = $date->format("U");
	$curr_ts = time();

	$diff_sec = $curr_ts - $date_ts;
	//On retire une heure (afin de laisser une petite marge pour l'update)
	if( $diff_sec > 3600 )
	{
		$diff_sec -= 3600;
	}
	//On arrondi en dessous
	$diff_day = floor($diff_sec / 86400);

	$str = "Package esia-egide-cve version $output last updated ago $diff_day days";
	if( $diff_day >= $critical )
	{
		$global_state = 'CRITICAL';
		$str = "$global_state - $str (> $critical days)";		
	}
	elseif( $diff_day >= $warning )
	{
		$global_state = 'WARNING';
		$str = "$global_state - $str (> $warning days)";
	}
	else
	{
		$global_state = 'OK';
		$str = "$global_state - $str (<= $warning days)";
	}
}
else
{
	$str = "$global_state - Package esia-egide-cve not found";
}


$date = date_parse($last_modify_cve['lastUpdate_cve']);
$date = date_create($last_modify_cve['lastUpdate_cve']);
$date = $date->format("d/m/Y") . " at " . $date->format("H:s");


if( $last_modify_cve['lastUpdate_day'] >= $critical )
{
	$state = "CRITICAL";
	$str = "Last update was ".$last_modify_cve['lastUpdate_day']." days ago (date : $date).\n$str";
}
elseif( $last_modify_cve['lastUpdate_day'] >= $warning )
{
	$state = "WARNING";
	$str = "Last update was ".$last_modify_cve['lastUpdate_day']." days ago (date : $date).\n$str";
}
else
{
	$state = "OK";
	$str = "Last update was ".$last_modify_cve['lastUpdate_day']." days ago (date : $date).\n$str";
}

echo "$state - $str\n";
exit(calcul_state($global_state,$state));


function getLastCVEUpdated($dbconnection)
{
	$sql = "SELECT
            EXTRACT(DAY FROM (now()::date - vul_last_update)) as min_day_cve,
            vul_last_update as update_cve
        FROM
            public.egide_vulnerability
        ORDER BY
            vul_last_update DESC
        LIMIT 1;";
	
	$query = $dbconnection->prepare($sql);
	$query->execute();
	
	$access = $query->fetch(PDO::FETCH_ASSOC);
	
	$lastUpdate_day = $access["min_day_cve"];
	$lastUpdate_cve = $access["update_cve"];
	$ret = array();
	$ret['lastUpdate_day']=$lastUpdate_day;
	$ret['lastUpdate_cve']=$lastUpdate_cve;
	// 		echo $lastUpdate;
	
	return $ret ;
	
}
?>
