#!/usr/bin/php
<?php
define('TMPDIR','/tmp/egide');
$path_file = TMPDIR.'/egide_stats.json';

$STR_STATUS=array("OK","WARNING","CRITICAL","UNKNOWN");
$INT_STATUS=array_flip($STR_STATUS);
$PRIORITIES_STATES=array("CRITICAL","WARNING","UNKNOWN","OK");

$type_list = array('ponderate','max');

$params = new Parameters();
$params->addOpt('','h','help',NULL,'print this help');
$params->addOpt('','D','debug',NULL,'Use to enable debug mode');
$params->addOpt(':','n','node','<nodeid|nodename>','to identify node (with id or name)');
$params->addOpt(':','w','warning','<treshold>','warning treshold');
$params->addOpt(':','c','critical','<treshold>','critical treshold');
$params->addOpt(':','T','type','<test_type>','To specifie test type');

$params->addUsage('-n <nodeid|nodename> -w <warning> -c <critical>');

if( $params->getParam('help') )
{
    $params->print_help();
    exit($INT_STATUS['UNKNOWN']);
}

$o_type = $params->getParam('type');
if(!in_array($o_type,$type_list))
{
    $params_opt_str = $params->getParamOptionsString('type');
    $str = "Incorrect value $params_opt_str=$o_type.";
    $str.= " This value must be : ".implode(", ",$type_list);
    exit_state('UNKNOWN',$str);
}

$required = array('node','warning','critical');
foreach($required as $key)
{
    $auth_params[$key] = $params->getParam($key);
    $params_opt_str = $params->getParamOptionsString($key);
    if( empty($auth_params[$key]))
    {
        echo "Missing required option $params_opt_str\n\n";
        $params->print_usage();
        exit($INT_STATUS['UNKNOWN']);
    }
}

$o_node = $params->getParam('node');

//Configuration et vérifiaction de o_warning et o_critical
$treshold_validate = array('warning','critical');
$tmp = array();
foreach($treshold_validate as $key)
{
    $value = $params->getParam($key);
    if( !is_numeric( $value))
    {
        echo "Treshold values must be numeric for option $params_opt_str\n\n";
        $params->print_usage();
        exit($INT_STATUS['UNKNOWN']);
    }
    $tmp[$key] = $value;
}

$o_warning = $tmp['warning'];
$o_critical = $tmp['critical'];

if( $o_warning < 0 )
{
    exit_state('UNKNOWN',"Treshold must be biggest than 0");
}
elseif( $o_critical < $value )
{
    exit_state('UNKNOWN',"Critical treshold must be bigger than warning treshold");
}


//Lecture du fichier
if( !is_file($path_file))
{
    exit_state('UNKNOWN',"Unable to access to JSON file $path_file");
}

$data_file = @file_get_contents($path_file);
$data_file = @json_decode($data_file,TRUE);

if( !is_array($data_file) )
{
    exit_state('UNKNOWN',"Unable to parse JSON file $path_file");
}

//Peut être que l'utilisateur a spécifier le nodename. On récupére dans ce cas l'ID
$node = $o_node;
if( isset($data_file['node_name'][$o_node]) && !isset($data_file['node_data'][$o_node]))
{
    $node = $data_file['node_name'][$o_node];
}

//Noeud introuvable
if( !isset($data_file['node_data'][$node] ) )
{
    exit_state('UNKNOWN',"Unable found Egide node : $o_node");
}

$node_data = $data_file['node_data'][$node];

$total_vul = $node_data['total_vul'];
$score_sum = $node_data['score_sum'];
$score_max = $node_data['score_max'];
$score_ponderate = $node_data['score_ponderate'];

$rrds = '';
foreach($node_data as $key => $value)
{
    $rrds .= "$key=$value;";
}


if( $o_type == 'ponderate' )
{
    $message = "Node ponderate vulnerability score is $score_ponderate";
    $value = $score_ponderate;
}
elseif( $o_type == 'max')
{
    $message = "Node max vulnerability score is $score_max (Total vul : $total_vul)";
    $value = $score_max;
}

if( $value >= $o_critical )
{
    $ret_state = 'CRITICAL';
}
elseif($value >= $o_warning )
{
    $ret_state = 'WARNING';
}
else
{
    $ret_state = 'OK';
}

exit_state($ret_state,"$message|$rrds");


function calcul_state($a,$b=null){
        $STR_STATUS=$GLOBALS["STR_STATUS"];
        $INT_STATUS=$GLOBALS["INT_STATUS"];
        $PRIORITIES_STATES=$GLOBALS["PRIORITIES_STATES"];

        if(is_string($a) ) $a=$INT_STATUS[strtoupper($a)];
        if(is_string($b) ) $b=$INT_STATUS[strtoupper($b)];

        foreach($PRIORITIES_STATES as $value){
                $value = $INT_STATUS[$value];

                if( $value == $a || $value == $b ) return $value;
        }
        return $INT_STATUS["UNKNOWN"];
}

function exit_state($state,$msg=null,$usage=false){
        $STR_STATUS=$GLOBALS["STR_STATUS"];
        $INT_STATUS=$GLOBALS["INT_STATUS"];

        $ret=$state;
        if( is_string($state) ) $ret=$INT_STATUS[strtoupper($state)];
        else $state=$STR_STATUS[$ret];

        if( $msg != null )
        {
          $msg=trim($msg);
          echo "$state : $msg\n";
        }
        if( $usage ) print_usage(true);
        exit($ret);
}

function format_date($date)
{
    $datetimeFormat = 'Y-m-d H:i:s';
    if( empty($date))
    {
        return '';
    }
    if( preg_match('/\/Date\((\d+)\d{3}\)\//',$date,$matches))
    {
        $dt = new DateTime();
        if( !is_numeric(($date) ))
        {
           return '';
        }
        $dt->setTimestamp(intval($date));
        $date = $dt->format($datetimeFormat);
        if( $dt == FALSE )
        {
            return '';
        }
        return $date;
    }
    if( preg_match('/(\d[0-1]\d{2})(\d{2})(\d{2})/',$date,$matches));
    {
        array_shift($matches);
        return implode('-',$matches);
    }
    return '';
}

class Parameters
{
    private $data = array();
    private $options = NULL;
    private $params = NULL;
    private $usage = array();

    function __construct()
    {
    }

    function addUsage($row)
    {
        $this->usage[] = $row;
    }

    static function getArrayBiggestWordSize($arr){
    	$biggest_size = 0;
            foreach($arr as $word){
                    $cur_size = strlen($word);
                    if( $biggest_size < $cur_size){
                            $biggest_size = $cur_size;
                    }
            }
    	return $biggest_size;
    }

    function print_usage($alone=TRUE)
    {
        $script = basename(__FILE__);
        $left = 'usage : ';
        $size_left = strlen($left);

        foreach($this->usage as $row)
        {
            echo str_pad($left,$size_left) . "$script $row [OPTIONS]\n";
            $left = '';
        }
        echo"\n";
        if($alone)
        {
            $row = "--help for more help";
            echo str_pad($left,$size_left) . "$script $row\n";
        }
    }

    function print_help()
    {
        $this->print_usage(FALSE);
        $opt = array();
        $desc = array();
        foreach($this->data as $key => $row)
        {
            $str = "\t-" . $this->getParamOptionsString($key);
            if( !empty($row['value']) )
            {
                $str.= '=<'.trim($row['value'],'<>').">";
            }
            $opt[] = $str;
            $desc[] = $row['descr'];
        }
        $size_left = self::getArrayBiggestWordSize($opt);
        foreach($opt as $idx => $left)
        {
            echo str_pad($left,$size_left) . "  ".$desc[$idx]."\n";
        }
    }

    public function getParamOptionsString($long_opt)
    {
        $row = $this->data[$long_opt];
        return "-".$row['short'] . ", --" . $row['long'];
    }

    public function addOpt($optional,$short_opt,$long_opt,$value_desc,$descr,$default = NULL)
    {
        $row = array();
        $row['optional'] = $optional;
        $row['short'] = $short_opt;
        $row['long'] = $long_opt;
        $row['value'] = $value_desc;
        $row['descr'] = $descr;
        $row['default'] = $default;
        $this->data[$long_opt] = $row;
    }

    private function getopt()
    {
        if( $this->options != NULL )
        {
            return;
        }

        $short_opts = '';
        $long_opts = array();

        foreach($this->data as $row)
        {
            $short_opts.= $row['short'].$row['optional'];
            $long_opts[] = $row['long'].$row['optional'];
        }
        $this->options = getopt($short_opts, $long_opts);
        $this->params = array();
        foreach($this->data as $row)
        {
            $req_opt = $row['optional'];
            $long_opt = $row['long'];
            $short_opt =  $row['short'];
            $default_opt = $row['default'];
            $key_name = $row['long'];

            if( $req_opt == '' )
            {
                if( isset($this->options[$long_opt]) || isset($this->options[$short_opt] ) )
                {
                    $value = TRUE;
                }
                else
                {
                    $value = FALSE;
                }
            }
            elseif( isset($this->options[$long_opt]))
            {
                $value = $this->options[$long_opt];
            }
            elseif( isset($this->options[$short_opt] ))
            {
                $value = $this->options[$short_opt];
            }
            else
            {
                continue;
                $value = $default_opt;
            }
            $this->params[$key_name] = $value;
        }
    }

    public function getParam($name,$default=NULL)
    {
        $this->getopt();
        if(isset($this->params[$name]) && is_numeric($this->params[$name]) && $this->params[$name] == 0)
        {
            return 0;
        }
        elseif( $default != NULL && empty($this->params[$name]) )
        {
            return $default;
        }
        elseif( !isset($this->params[$name]))
        {
            return NULL;
        }
        elseif(empty($this->params[$name]))
        {
            return FALSE;
        }
        return $this->params[$name];
    }
}
?>
