#!/usr/bin/php
<?php
define('SCRIPT_NAME', 'egide_import_kb_properties');

function Trace($debug, $msg)
{
	if($debug)
	{
		if(is_array($msg))
		{
			print_r($msg);
		}
		else 
		{
			echo "DEBUG: $msg\n";
		}
	}
}

// Get parameters
$options = getopt('c:f:D');
// Web interface document root
if(!empty($options['c']))
{
	$_SERVER['DOCUMENT_ROOT'] = $options['c'];
}
else
{
	$_SERVER['DOCUMENT_ROOT'] = '/usr/local/esia/public_html/';
}
$path = rtrim($_SERVER['DOCUMENT_ROOT'], '/');
// KB properties file path
if(!empty($options['f']))
{
	$kbPropertiesFile = $options['f'];
}
else
{
	$kbPropertiesFile = '/usr/local/esia/egide/data/KB_CVE.properties';
}
// Debug
if(isset($options['D']))
{
	$DEBUG = TRUE;
}
else
{
	$DEBUG = FALSE;
}

$date = new DateTime();
echo SCRIPT_NAME . " - " . $date->format('d-m-Y H:i:s') . "\n\n";

// Check if KB properties file is present (given by package esia-egide-cve-all) 
if(!file_exists($kbPropertiesFile))
{
	echo "KB properties file does not exist ($kbPropertiesFile)\n";
	exit(1);
}
Trace($DEBUG, 'KB properties file is present');

// Esia conf & model
$configFile = $path . '/esia.conf.php';
if(!file_exists($configFile))
{
	echo "Configuration file does not exist ($configFile)\n";
	exit(1);
}
include_once $configFile;
include_once MVCMODEL;

// DB connect
Trace($DEBUG, 'Connect to DB');
try
{
	$dbconnection = new PDO('pgsql:host=' . DB_HOST . ';port=' . DB_PORT . ';dbname=' . DB_NAME, DB_USERNAME, DB_PWD);
	$dbconnection->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
}
catch(Exception $e)
{
	echo 'PDO SQL ERROR: ' . $e->getMessage() . '\n';
	echo 'N° : ' . $e->getCode();
	exit(1);
}
Trace($DEBUG, 'DB connexion OK');

// Check egide licence
Trace($DEBUG, 'Check egide installation');
try
{
	Model_esia::getLicenceInformation($dbconnection, 'LICENCE_EGIDE');
}
catch(EsiaDbException | Exception $e)
{
	echo 'No licence egide found: ' . $e->getMessage() . '\n';
	exit(1);
}
Trace($DEBUG, 'Licence Egide OK');

// Check if module inventory & egidescan are installed
try
{
	$webpEgidescan = Model_esia::getWebPlugin($dbconnection, FALSE, 'WEBP_EGIDESCAN');
	$webpInventory = Model_esia::getWebPlugin($dbconnection, FALSE, 'WEBP_INVENTORY');
	if(empty($webpEgidescan) || empty($webpInventory))
	{
		throw new Exception('Missing egidescan or inventory webp');
	}
}
catch(EsiaDbException | Exception $e)
{
	echo 'Egide modules not found: ' . $e->getMessage() . '\n';
	exit(1);
}
Trace($DEBUG, 'Egide modules are present');

// Include model inventory
$inventoryPath = $webpInventory[0]['apache_path'];
$inventoryPath = $path . rtrim($inventoryPath, '/');

include_once $inventoryPath . '/Model_inventory.php';

// Begin task
try
{
	// Get KB properties
	Trace($DEBUG, 'Parsing KB properties file');
	// File path already tested on script begining. if(!file_exists($kbPropertiesFile))
	$kbFileList = parse_ini_file($kbPropertiesFile);
	if(empty($kbFileList))
	{
		throw new Exception("Cannot parse KB file or file is empty ($kbPropertiesFile)");
	}
	Trace($DEBUG, 'KB properties has been successfully parsed');

	// Get KB with no CVE key in JSON DB
	Trace($DEBUG, 'Get inventory KB components');
	$kbCompList = Model_inventory::getComponentKbWithEmptyCve($dbconnection);
	Trace($DEBUG, 'Inventory KB components fetched');

	if(!empty($kbCompList))
	{
		echo "Add new inventory KB components CVE list:\n";
		foreach($kbCompList as $compId => $compData)
		{
			if(isset($kbFileList[$compData['name']]))
			{
				$cveList = $kbFileList[$compData['name']];
				echo $compData['name'] . " => " . $cveList . "\n";
				
				// Format JSON
				$json = array();
				if(!empty($compData['json']))
				{
					// If component has already a JSON data
					$json = json_decode($compData['json'], TRUE);
				}
				// Add cve list in JSON
				if(empty($cveList))
				{
					// If no CVE list for this KB add empty CVE json array to create the key and no fetch it anymore
					$json['CVE'] = array();
				}
				else
				{
					$json['CVE'] = explode(',', $cveList);
				}
				
				// Update component JSON
				Model_inventory::updateComponentJSON($dbconnection, $compId, $json);
			}
		}
	}
	else
	{
		echo "No new inventory KB component found\n";
	}

	echo "\nInventory KB components CVE list have been saved in DB\n";
}
catch(EsiaDbException | Exception $e)
{
	echo "EXCEPTION: " . $e->getMessage() . "\n";
	exit(1);
}
exit(0);

?>
