#!/usr/bin/php
<?php
define('LOG_LEVEL',LOG_DEBUG);

$_SERVER['DOCUMENT_ROOT'] = '/usr/local/esia/public_html/';

$path = $_SERVER['DOCUMENT_ROOT'];
include $path.'/esia.conf.php';
include_once USR_CLASS;
include_once LIB_LOG;
include_once MVCMODEL;
include_once $path.'/Model_esia.php';
include_once $path.'/plugin/gesa/model_gesa.php';

write_log(LOG_NOTICE,'STARTING cron task script egide_scanner_plugin.php');

$longopts = array(
	'mode:','scan_id:'
);
$shortopts = 'm:i:';

$modes_list = array('start','get');
$options = getopt($shortopts,$longopts);

$o_mode = getParam($options,'mode','m',NULL);
$o_scan = getParam($options,'scan_id','i',NULL);

$modes_list_str = implode('|',$modes_list);
if( empty($o_mode) )
{
    write_log(LOG_EMERG,"You forget to use option -m, --mode=<$modes_list_str>");
}
if( !in_array($o_mode,$modes_list) )
{
    write_log(LOG_EMERG,"Unknow mode $o_mode, option -m, --mode must be ($modes_list_str)");
}

//1 seule operation à la fois
$salt = json_encode($options);
$salt = md5($salt);

$lock_path = "/tmp/egide_plugin_$salt.lock";

$fp = fopen($lock_path, 'w+');
if(!flock($fp,LOCK_EX|LOCK_NB)){
	write_log(LOG_INFO,"Script is already running with this parameters\n");
	exit(0);
}

//connection à la base de donnée
write_log(LOG_INFO,'DB connection');
try
{
	$dbconnection = new PDO('pgsql:host='.DB_HOST.';port='.DB_PORT.';dbname='.DB_NAME, DB_USERNAME, DB_PWD);
	$dbconnection->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
}
catch(Exception $e)
{
	$err='Unable to connect to DB. ';
	$err.='PDO SQL ERROR N° : '.$e->getCode();
	$err.='  MESSAGE : '.$e->getMessage();
	write_log(LOG_EMERG,$err);
}
try
{
	$licegi = Model_esia::getLicenceInformation($dbconnection,'LICENCE_EGIDE');
}
catch (Exception $e)
{
	write_log(LOG_EMERG,"No licence egide found");
	exit(1);
}
$arr = Model_esia::getWebPlugin($dbconnection,false,'WEBP_EGIDESCAN');
$plugin_url = $arr[0]['apache_path'];
include_once $path.$plugin_url.'model_egidescan.php';
include_once $path.$plugin_url.'/inc/ParserNmap.php';

$scanner = new ScanManager();
if( $o_mode == 'start' )
{
    if( empty($o_scan) )
    {
        write_log(LOG_EMERG,"You forget to use option -i, --scan_id=<ID from table vulnerability_scanner>");
        exit(1);
    }
    if( !is_numeric($o_scan) )
    {
        write_log(LOG_EMERG,"Incorrect value for -i, --scan_id=<scan_id> options, must be scan_id column from table vulnerability_scanner");
        exit(1);
    }
    try
    {
        $scan_info = Model_egidescan::getScanById($dbconnection,$o_scan);

        $id			= $scan_info['id'];
        $agent_id	= $scan_info['agent_id'];

        if( empty($scan_info['options']))
        {
            write_log(LOG_EMERG,"Unable to scan scan without database scan_option values (group_list, ...)");
            exit(1);
        }

		if( $scan_info['options']['group_list'] == 'all' )
		{
			   $all_node_list = Model_esia::newgetNodeList($dbconnection,NULL,NULL,Model_esia::NODE_INSTANCE);
		}
		else
		{
			   $all_node_list = Model_egidescan::getNodesOfGroupList($dbconnection,$scan_info['options']['group_list']);
		}

        $addr_list  = array();
        foreach($all_node_list as $node)
        {
            $ip_addr = $node->getIPAddress();
            if (filter_var($ip_addr, FILTER_VALIDATE_IP)) {
                $addr_list[] = $ip_addr;
            }
        }
	if( empty($addr_list))
	{
        	Model_egidescan::notifyScanningError($dbconnection,$id,"Node list empty. Check if group list has nodes with IP address");
        	exit(0);
	}

        $addr_list = implode(',',$addr_list);
        $iface = $scan_info['iface'];

        $box = Model_gesa::getInstanceGESA($dbconnection,$agent_id);
        $sn = $box->getSerialNumber();

        write_log(LOG_DEBUG,"Sending task $id");
	//	$iface = escapeshellcmd($iface);
	//f	$iface = str_replace(' ', '\ ',$iface);
		$iface = '"'.$iface.'"';
        $arguments = "-A -i $id -a $addr_list -e $iface";


        $result = $scanner->execute($arguments,$sn);

        if( $result['return'] != 0 ){ //Une erreur s'est produite.
            notify_error_gesa($result,LOG_WARNING,'Unable to save the task');
        }

        Model_egidescan::updateScanningTaskStatus($dbconnection,$id,Model_egidescan::PROCESSING_TASK);
    }
    catch(Exception $ex){
        $err='Unable to set status processing in DB. ';
        $err.='PDO SQL ERROR N° : '.$e->getCode();
        $err.='  MESSAGE : '.$e->getMessage();
        write_log(LOG_ALERT,$err);
    }
}
elseif( $o_mode == 'get')
{
    $processing_tasks = Model_egidescan::getCurrentScanList($dbconnection,Model_egidescan::PROCESSING_TASK);

    $agent_list = array();
    foreach($processing_tasks as $row)
    {
    	if( empty($agent_list[$row['agent_id']]))
    	{
    		$agent_list[$row['agent_id']] = array();
    	}
    	$agent_list[$row['agent_id']][] = $row['id'];
    }

    write_log(LOG_INFO,'Get terminated tasks list for '.count($agent_list).' agents');
    foreach($agent_list as $agent_id => $db_ids){
    	$box = Model_gesa::getInstanceGESA($dbconnection,$agent_id);
    	$sn = $box->getSerialNumber();

    	write_log(LOG_INFO,"Get terminated tasks status for Agent $sn");
    	$arguments = "-L -s terminated,error";
    	$result = $scanner->execute($arguments,$sn);

    	if( $result['return'] != 0 )
    	{ //Une erreur s'est produite.
            notify_error_gesa($result,LOG_CRIT,'Unable to get id list');
            continue;
    	}
    	else
    	{
    		$for_delete_ids = array();
    		$for_get_ids = array();

    		$dev_ids = trim($result['output'][0]);
    		if(!empty($dev_ids)) $dev_ids = explode(',',$dev_ids);
    		else $dev_ids = array();

    		foreach($dev_ids as $dev_id){
    			$for_delete_ids[]=$dev_id;
    			$for_get_ids[]=$dev_id;
    		}
    	}

    	write_log(LOG_INFO,"Download ".count($for_get_ids)." result");
    	if(!empty($for_get_ids)){
    		$scan_ids = implode(',',$for_get_ids);
    		$arguments = "-G -i $scan_ids";
    		$result = $scanner->execute($arguments,$sn);

    		if( $result['return'] != 0 ){ //Une erreur s'est produite.
                notify_error_gesa($result,LOG_ERR,'Unable to download result');
    			continue;
    		}

    		$parsed = @json_decode(trim(implode("\n",$result['output'])),true);
    		if($parsed == null){
    			$error_message = "Unable to parse received data for $scan_ids";
    			echo $error_message."\n";
    			write_log(LOG_ERR,$error_message);
    			continue;
    		}
			else
			{
				foreach($parsed as $scan_id => $scan_result)
				{
					if(!in_array($scan_id,$db_ids)) //Scan récupéré no présent en base de donnée
					{
						write_log(LOG_INFO,"Scan on Unity not present into DB (probably deleted by user).");
					}
					else if($scan_result['status'] == Model_egidescan::ERROR_TASK)
					{
    					Model_egidescan::notifyScanningError($dbconnection,$scan_id,$scan_result['message']);
    					write_log(LOG_WARNING,$scan_result['message']);
    				}
					else if($scan_result['status'] == Model_egidescan::TERMINATED_TASK)
					{
    					//$data_path = Model_ScanModule::DATA_PATH.'/'.$scan_id;
						try
						{
    						process_data($dbconnection,$scan_id,$scan_result);
    						Model_egidescan::updateScanningTaskStatus($dbconnection,$scan_id,Model_egidescan::TERMINATED_TASK);
    						write_log(LOG_DEBUG,"Task $scan_id setted TERMINATED");
    					}
						catch(Exception $ex)
						{
							try
							{
								Model_egidescan::notifyScanningError($dbconnection,$scan_id,"Unable to update status for $scan_id : ".$ex->getMessage());
							}
							catch(Exception $ex)
							{
								write_log(LOG_ERR,"Unable to notify error for ".$scan_id);
							}
							write_log(LOG_ERR,"Unable save result for ".$scan_id);
    						continue;
    					}
    				}
    			}
    		}
    	}
		write_log(LOG_INFO,"Delete ".count($for_delete_ids)." terminated tasks for agent $sn");
    	if(!empty($for_delete_ids))
    	{
    		$scan_ids = implode(',',$for_delete_ids);
    		$arguments = "-D -i $scan_ids";
   		    $result = $scanner->execute($arguments,$sn);

    		if( $result['return'] != 0 ){ //Une erreur s'est produite.
                notify_error_gesa($result,LOG_WARNING,'Unable to delete scans');
    		}
    	}
    }
	Model_egidescan::refreschScore($dbconnection);
}
write_log(LOG_NOTICE,'CLOSED successfully cron task script egide_scanner_plugin.php');

function notify_error_gesa($result,$log_level,$message)
{
    if( empty($result['output'][0]) )
    {
        $details = 'No result from command';
    }
    else
    {
        if ( preg_match('/ERROR\s*:\s*(.*)/',$result['output'][0],$matches) === FALSE)
		{
			$details = $matches[1];
		}
		else
		{
        	$details= $result['output'][0];
		}
    }
    $error_message="$message => $details.";
    write_log($log_level,$error_message);
}

function write_log($priority,$message)
{
    $level_arr = array('EMERG','ALERT','CRIT.','ERROR','WARN.','NOTI.','INFO.','DEBUG');
    $level_str = $level_arr[$priority];


	if($priority <= LOG_LEVEL){
		$date = new DateTimeImmutable();
		$time = $date->format('Y-m-d H:i:s.u');
		//file_put_contents ( LOG_FILE , "$time|$priority|$message\n" ,  FILE_APPEND  );
		echo "$time|$priority|$level_str|$message\n";
	}
	if( $priority == LOG_EMERG)
	{
		$date = new DateTimeImmutable();
		$time = $date->format('Y-m-d H:i:s.u');
		//file_put_contents ( LOG_FILE , "$time|$priority|$message\n" ,  FILE_APPEND  );
		echo "$time|$priority|$level_str|$message\n";
		exit(1);
	}
}

function process_data($dbconnection,$scan_id,$data)
{
	$t0 = $data['start'];
	$t1 = $data['stop'];

	$tmpfname = sys_get_temp_dir(). '/egide_'.$scan_id.$t1.'.xml';
	file_put_contents($tmpfname,$data['result']);

	$t0 = date("Y-m-d H:i:s",$t0);
	$t1 = date("Y-m-d H:i:s",$t1);

	$scan_info = Model_egidescan::getScanById($dbconnection,$scan_id);
	$agent_id = $scan_info['agent_id'];
	Model_egidescan::addScanHistory($dbconnection, $t0,$t1,$scan_id,$data,Model_egidescan::TERMINATED_TASK);

	$group_list = $scan_info['options']['group_list'];
	if( $scan_info['options']['group_list'] == 'all' )
	{
	   $all_node_list = Model_esia::newgetNodeList($dbconnection,NULL,NULL,Model_esia::NODE_INSTANCE);
	}
	else
	{
	   $all_node_list = Model_egidescan::getNodesOfGroupList($dbconnection,$scan_info['options']['group_list']);
	}

	$list_node = array();
	foreach($all_node_list as $row)
	{
		$list_node[$row->getIPAddress()] = $row;
	}
	sleep(3);
    $data = file_get_contents($tmpfname);
    $data = parseXmlScan($data);

	$inventory_available = Model_egidescan::isWebpInventoryEnabled($dbconnection);
	if($inventory_available)
	{
		$kdb_filter_list = Model_egidescan::getListOfWinKB($dbconnection);
	}

    write_log(LOG_INFO,count($data['hosts']). " devices scanned by scan $scan_id");

    $scan_date = time();
    foreach($data['hosts'] as $host)
    {
        $addr_list = array();
        foreach($host['address'] as $row)
        {
            if($row['addrtype'] != 'mac')
            {
                $addr_list[] = $row['addr'];
            }
        }

        $matched_addr = NULL;
        foreach($addr_list as $row)
        {
            if(array_key_exists($row,$list_node))
            {
                $matched_addr = $row;
            }
        }

        if( $matched_addr != NULL )
        {
            $port_list = array();
            $port_list = $host['ports'];

            $node =  $list_node[$matched_addr];
            $oldEgideService =  model_egidescan::getEgideServicesNode($dbconnection,$node->getId(),$scan_info['iface_id']);

            write_log(LOG_NOTICE,"Host $matched_addr");

            foreach($port_list as $port)
            {
                if( $port['state'] == 'open')
                {
                    $portnum =  $port['portid'];
                    write_log(LOG_DEBUG,"Port $portnum status open");
                    if( empty($port['extrainfo']))
                    {
                        $port['extrainfo'] = '';
                    }
                    if( empty($port['cpe']))
                    {
                        $port['cpe'] = '';
                    }
                    if(array_key_exists($portnum,$oldEgideService))
                    {
                        $egide_service_id = $oldEgideService[$portnum]['id'];
                        model_egidescan::updateEgideServicesNode($dbconnection,$egide_service_id,$port['service_name'],$port['service_product'],$port['service_version'],$port['cpe'],$port['extrainfo'],$scan_info['iface_id']);
                    }
                    else
                    {
                        write_log(LOG_NOTICE,"Port add $portnum");
                        $egide_service_id = model_egidescan::insertEgideServicesNode($dbconnection,$node->getId(),$port['service_name'],$port['service_product'],$port['service_version'],$portnum,$port['protocol'],$port['cpe'],$port['extrainfo'],$scan_info['iface_id']);
                    }

					$cve_list = getCVEListForService($dbconnection,$host,$portnum,$node->getId(),$inventory_available);
					if($inventory_available)
					{
						$cve_list = ignore_win_kdb_cve($node->getId(),$kdb_filter_list,$cve_list);
					}
					$nb = model_egidescan::insertVulnerabilityDetect($dbconnection,$egide_service_id,'NIST',$cve_list,$scan_date);
					$rowCount = model_egidescan::updateNotDetected($dbconnection,$egide_service_id,$scan_date);
					write_log(LOG_NOTICE,"Port $portnum : ".count($cve_list). " vulnerabilites detected, $nb encoded and $rowCount disabled");
                }
            }
        }
    }
	unlink($tmpfname);
}

function ignore_win_kdb_cve($node_id,$kdb_filter_list,$cve_list)
{
	if( isset($kdb_filter_list[$node_id]) && !empty($cve_list) )
	{
		    $tmp = array();
		    $nb_ignored = 0;
		    $nb_keep = 0;
		    $node_kb = $kdb_filter_list[$node_id];
		    write_log(LOG_DEBUG,"KDB Host, searching acquitting CVEs for $portnum");
		    foreach($cve_list as $cve)
		    {
		            if( isset($node_kb[$cve]))
		            {
		                    $nb_ignored++;
		                    write_log(LOG_DEBUG,"Ignoring CVE $cve.  Fix present into ".$node_kb[$cve]);
		            }
		            else
		            {
		                    $nb_keep++;
		                    $tmp[] = $cve;
		            }
		    }
		    write_log(LOG_NOTICE,"KDB Host, $nb_ignored ignored CVE. $nb_keep CVE to INSERT");
		    $cve_list = $tmp;
	}
	return $cve_list;
}

function getParam($options,$name1,$name2,$default=null){
        if( isset( $options[$name1] ) ) return $options[$name1];
        if( isset( $options[$name2] ) ) return $options[$name2];
        return $default;
}

class ScanManager
{
	private $plugin;

	public function __construct(){
		$this->plugin = new ExecPlugin(1,60);
	}

	public function execute($arguments,$gesa)
	{
		$RETRY = 3;
		$SLEEP = 15;

		for($i=0;$i<$RETRY;$i++)
		{
			$result = $this->plugin->execute('egide_scan_adm.php', $arguments,$gesa);
			if( $result['return'] == 0 )
			{
				return $result;
			}
            if( empty($result['output'][0]))
            {
                $result['output'][0] = 'N/A';
            }
            write_log(LOG_WARNING,"Command return ".$result['return'] . ' != 0. Retry to run command. Output message start with : '.$result['output'][0]);
			sleep($SLEEP);
		}
		if( preg_match('/UNKNOWN\s*-\s*(.*)/',$result['output'][0],$matches))
        {
		          $result['output'] = array('ERROR : '.$matches[1]);
        }
		return $result;
	}
}
class ExecPlugin
{
	const CHECK_GESA='check_gesa';

	private $_timeout;
	private $_priority;

	public function __construct($priority=3,$timeout=3)
	{
		$this->_priority = $priority;
		$this->_timeout = $timeout;
	}

	public function execute($command,$arguments,$gesa){
		$timeout = $this->_timeout;
		$pri = $this->_priority;
        write_log(LOG_DEBUG,"Execute command ".PLUGINS_PATH.'/'.self::CHECK_GESA." -G $gesa -C $command -A '$arguments' -t $timeout -p $pri");
		exec(PLUGINS_PATH.'/'.self::CHECK_GESA." -G $gesa -C $command -A '$arguments' -t $timeout -p $pri",$output,$returnexec);
		return array("output"=>$output,"return"=>$returnexec);
	}
}
?>
