#!/usr/bin/php
<?php
define('LOG_LEVEL',LOG_DEBUG);
define('TMPDIR','/tmp/egide');
define('FILE_UPDATE','/tmp/egide/score_update_required.bool');

$_SERVER['DOCUMENT_ROOT'] = '/usr/local/esia/public_html/';

$path = $_SERVER['DOCUMENT_ROOT'];
include $path.'/esia.conf.php';
include_once USR_CLASS;
include_once LIB_LOG;
include_once MVCMODEL;
include_once $path.'/Model_esia.php';
include_once $path.'/Model_adm.php';

//1 seule operation à la fois
$lock_path = "/tmp/passive_egide_plugin.lock";

$fp = fopen($lock_path, 'w+');
if(!flock($fp,LOCK_EX|LOCK_NB)){
	write_log(LOG_INFO,"Script is already running\n");
	exit(0);
}

$path_file = TMPDIR.'/egide_stats.json';

//On vérifie si le script doit être exécuté
$exec_script = TRUE;

//Présence du need_refresh à FALSE ?
if(is_file(FILE_UPDATE))
{
	$file_content = file_get_contents(FILE_UPDATE);
	if( trim($file_content) == 'need_refresh:FALSE' )
	{
		write_log(LOG_DEBUG,'No change detected - Refresh not required');
		$exec_script = FALSE;
	}
	else
	{
		write_log(LOG_DEBUG,'Data change detected - Refresh required');
	}
}
else
{
	write_log(LOG_INFO,'Missing boolean file'.FILE_UPDATE);
}
//Fichier data présent ?
if (!$exec_script && is_file($path_file ))
{
	write_log(LOG_DEBUG,'exit without refresh');
	exit(0);
}
else
{
	write_log(LOG_DEBUG,'Missing data file - Refresh required');
}


//connection à la base de donnée
write_log(LOG_INFO,'DB connection');

try
{
	$dbconnection = new PDO('pgsql:host='.DB_HOST.';port='.DB_PORT.';dbname='.DB_NAME, DB_USERNAME, DB_PWD);
	$dbconnection->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
}
catch(Exception $e)
{
	$err='Unable to connect to DB. ';
	$err.='PDO SQL ERROR N° : '.$e->getCode();
	$err.='  MESSAGE : '.$e->getMessage();
	write_log(LOG_EMERG,$err);
}
try
{
	$licegi = Model_esia::getLicenceInformation($dbconnection,'LICENCE_EGIDE');
}
catch (Exception $e)
{
	write_log(LOG_EMERG,"No licence egide found");
	exit(1);
}

//Récupération des noeuds concernés par Egide
$webpEgide = Model_esia::getWebPlugin($dbconnection,false,'WEBP_EGIDESCAN');
$webpEgide = array_shift($webpEgide);

$plugin_url = $webpEgide['apache_path'];
include_once $path.$plugin_url.'model_egidescan.php';

write_log(LOG_DEBUG,"Récupération des donnée");
$statsList = Model_egidescan::getNodesStats($dbconnection,NULL,NULL,NULL,NULL,NULL);

$nodeList=array();
$nodeIDList = array();
foreach($statsList as $stat)
{
	$retRow = array();
	$retRow['total_vul'] = $stat['total_vul'];
	$retRow['score_sum'] = $stat['sum_score'];
	$retRow['score_max'] = $stat['maxVulnerabilityScoring'];
	$retRow['score_ponderate'] = $stat['averageVulnerabilityScoring'];
	$nodeIDList[$stat['name']] = $stat['id'];
    $nodeList[$stat['id']] = $retRow;
}

write_log(LOG_DEBUG,"Ecriture des résultats");
//Création d'un répertoire temporaire si absent
if( !is_dir(TMPDIR) )
{
	mkdir(TMPDIR);
	chown(TMPDIR,'www-data');
	chgrp(TMPDIR,'www-data');
}
$data['node_name'] = $nodeIDList;
$data['node_data'] = $nodeList;
file_put_contents($path_file,json_encode($data),LOCK_EX);
chown($path_file,'www-data');
chgrp($path_file,'www-data');


function write_log($priority,$message)
{
    $level_arr = array('EMERG','ALERT','CRIT.','ERROR','WARN.','NOTI.','INFO.','DEBUG');
    $level_str = $level_arr[$priority];

	if($priority <= LOG_LEVEL){
		$date = new DateTimeImmutable();
		$time = $date->format('Y-m-d H:i:s.u');
		//file_put_contents ( LOG_FILE , "$time|$priority|$message\n" ,  FILE_APPEND  );
		echo "$time|$priority|$level_str|$message\n";
	}
	if( $priority == LOG_EMERG)
	{
		$date = new DateTimeImmutable();
		$time = $date->format('Y-m-d H:i:s.u');
		//file_put_contents ( LOG_FILE , "$time|$priority|$message\n" ,  FILE_APPEND  );
		echo "$time|$priority|$level_str|$message\n";
		exit(1);
	}
}
