<?php
/**
 * @name  Model_egidescan
 * @description Classe héritant de "Model_esia" gèrent tout les accès à la base de donnée du plugin Server
 * @author Nicolas Biersart <n.biersart@esia-network.com>
 * @contributor
 * @copyright Nicolas Biersart 2012
 * @license
 * @esia 1.00
 * @version 1.0.0
 * @date 08/11/2012
 */

include_once '/usr/local/esia/public_html/Model_esia.php';
include_once SNMP_CLASS;
include_once DEVICE_CLASS;
include_once MVCMODEL;
include_once 'libs/egide_utils.libs.php';

class Model_egidescan extends Model_esia
{
	const NEW_TASK='new';
	const PENDING_TASK='pending';
	const PROCESSING_TASK='processing';
	const TERMINATED_TASK='terminated';
	const ERROR_TASK='error';

	const PLUGIN_CRON_START = 'CRON_EGIDE_SCAN_START';
	//const BOX_FILTER_REGEX = '20.{6}86';            // Pour  Unity x86
    //const BOX_FILTER_REGEX = '.*';                  // Toutes les gesa
	const BOX_FILTER_REGEX = '20.{6}[19]+0';        //10 ou 90, pour box Egide

	const TMPDIR = '/tmp/egide';
	const FILE_UPDATE = Model_egidescan::TMPDIR.'/score_update_required.bool';


	// vulscan_json
	public static function getScoreHtmlColor($score_value)
	{
		$colors = array("#00c400","#00e020","#00f000","#d1ff00","#ffe000","#ffcc00","#ffbc10","#ff9c20","#ff8000","#ff0000","#ff0000"); //9 et 10 doivent être rouge
		$score_value = floor($score_value); //On arrondi en dessous
		return $colors[$score_value];
	}

	public static function existIp($dbconnection, $ip=NULL)
	{
		$sql = "SELECT COUNT(*) as detect_ip_adress
				                        FROM esia_device WHERE device_ip_address = :ip;";

		$params = array(
			':ip' => $ip
		);

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);
		$access = $query->fetch(PDO::FETCH_OBJ);
		$detect_ip_adress = $access->detect_ip_adress;
		if(empty($detect_ip_adress))
		{
			$detect_ip_adress = 'NONE';
		}

		return $detect_ip_adress;
	}

	public static function acquitVulnerabilitiesFromServices($dbconnection,$user_obj,$cve_list=NULL,$include_cve=TRUE)
	{
		//Préparation et validation du parameter
		$in_request = generateInParams('WHERE',$cve_list,'detect_eg_service_id',$include_cve);
		$params = $in_request['param'];
		$where = $in_request['sql'];

		$params[':user']= $user_obj->getName();

		$sql = "
				UPDATE egide_vulnerability_detect
				SET detect_state_id=(SELECT state_id FROM egide_vulnerability_detect_state WHERE state_name='ACQUIT'),
					detect_state = 'ACQUIT',
					detect_username = :user,
					detect_date_update_state = now()
				$where
				RETURNING detect_eg_service_id";

			$query = $dbconnection->prepare($sql);
			$query = Model_esia::executeStatement($query,$params);
			$ret = array();
			while($row=$query->fetch(PDO::FETCH_ASSOC))
			{
				$retRow = array();
				$ret[$row['detect_eg_service_id']] =  $row['detect_eg_service_id'];
			}
			self::fireScoreNeedRefresh($dbconnection,array_keys($ret));
	}

	public static function acquitVulnerabilitiesFromNodes($dbconnection,$user_obj,$node_list,$cve_list=NULL,$include_cve=TRUE)
	{
		//Préparation et validation du parameter
		$params[':user']= $user_obj->getName();

		$in_request = generateInParams('WHERE',$node_list,'eg_device_id',TRUE);
		$param = $in_request['param'];
		$sql_devices = $in_request['sql'];
		$params = array_merge($params,$param);

		$in_request = generateInParams('WHERE',$cve_list,'vul_list_id',$include_cve);
		$param = $in_request['param'];
		$sql_vul = $in_request['sql'];
		$params = array_merge($params,$param);

		$sql_vul = "
				AND detect_vul_id IN(
					SELECT vul_id FROM egide_vulnerability
					$sql_vul
				)
			";

		$sql = "
				UPDATE egide_vulnerability_detect
				SET detect_state_id=(SELECT state_id FROM egide_vulnerability_detect_state WHERE state_name='ACQUIT'),
					detect_state = 'ACQUIT',
					detect_username = :user,detect_date_update_state = now()
				WHERE detect_eg_service_id IN(
						SELECT eg_service_id FROM egide_services
						$sql_devices $sql_vul
				) RETURNING detect_eg_service_id;";

			$query = $dbconnection->prepare($sql);
			$query = Model_esia::executeStatement($query,$params);
			$ret = array();
			while($row=$query->fetch(PDO::FETCH_ASSOC))
			{
				$retRow = array();
				$ret[$row['detect_eg_service_id']] =  $row['detect_eg_service_id'];
			}
			self::fireScoreNeedRefresh($dbconnection,array_keys($ret));
	}

	public static function unacquitVulnerabilitiesDetect($dbconnection,$cve_list=NULL,$include_cve=TRUE)
	{
		//Préparation et validation du parameter
		$in_request = generateInParams('AND',$cve_list,'detect_id',$include_cve);
		$params = $in_request['param'];
		$sql_vul = $in_request['sql'];

		$sql = "
				UPDATE egide_vulnerability_detect
				SET detect_state_id=(SELECT state_id FROM egide_vulnerability_detect_state WHERE state_name='ACTIVE'),
					detect_state = 'ACTIVE',
					detect_username = NULL,detect_date_update_state = NULL
				WHERE detect_eg_service_id IN(
						SELECT eg_service_id FROM egide_services
						WHERE detect_state_id IN (SELECT state_id FROM egide_vulnerability_detect_state WHERE state_name='ACQUIT') $sql_vul
				) RETURNING detect_eg_service_id;";

			$query = $dbconnection->prepare($sql);
			$query = Model_esia::executeStatement($query,$params);
			$ret = array();
			while($row=$query->fetch(PDO::FETCH_ASSOC))
			{
				$retRow = array();
				$ret[$row['detect_eg_service_id']] =  $row['detect_eg_service_id'];
			}
			self::fireScoreNeedRefresh($dbconnection,array_keys($ret));
	}

	public static function getScoreToRefresh($dbconnection,$iface=NULL)
	{
		$sql = "SELECT eg_access_iface_id,eg_access_service_id FROM egide_service_access WHERE need_refresh != FALSE";
		$params = array();
		if( is_array($iface) )
		{
			$in_sql = Model_esia::pdo_array_to_SQL('iface',$iface,$params);
			$sql .=" AND eg_access_iface_id IN ($in_sql) ";
		}
		elseif( $iface != NULL )
		{
			$params[':iface'] = $iface;
			$sql .=" AND eg_access_iface_id = :iface";
		}

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);

		$ret = array();
		while($row=$query->fetch(PDO::FETCH_ASSOC))
		{
			$retRow = array();
			$retRow['iface_id'] = $row['eg_access_iface_id'];
			$retRow['service_id'] = $row['eg_access_service_id'];
			$ret[] = $retRow;
		}
		return $ret;
	}

	public static function fireScoreNeedRefresh($dbconnection,$servicesList)
	{
		$where = '';
		if(empty($servicesList))
		{
			return;
		}
		elseif( is_array($servicesList) )
		{
			$in_sql = Model_esia::pdo_array_to_SQL('service',$servicesList,$params);
			$where .=" WHERE eg_access_service_id IN ($in_sql) ";
		}
		elseif( $servicesList != NULL )
		{
			$params[':service'] = $servicesList;
			$where .=" WHERE eg_access_service_id = :service";
		}

		$sql = "UPDATE egide_service_access SET need_refresh=TRUE $where";
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);

		//Création d'un répertoire temporaire si absent
		if( !is_dir(Model_egidescan::TMPDIR) )
		{
			mkdir(Model_egidescan::TMPDIR);
			chown(Model_egidescan::TMPDIR,'www-data');
			chgrp(Model_egidescan::TMPDIR,'www-data');
		}
		//Set du boolean a TRUE
		file_put_contents(Model_egidescan::FILE_UPDATE,"need_refresh:TRUE\n", LOCK_EX);
		chown(Model_egidescan::FILE_UPDATE,'www-data');
		chgrp(Model_egidescan::FILE_UPDATE,'www-data');
	}

	public static function refreschScore($dbconnection,$ifaceList=NULL,$servicesList=NULL)
	{
		if(is_file(Model_egidescan::FILE_UPDATE))
		{
			$file_content = file_get_contents(Model_egidescan::FILE_UPDATE);
			if( trim($file_content) == 'need_refresh:FALSE' )
			{
				return;
			}
		}

		if( empty($ifaceList) )
		{
			$ifaceList = NULL;
		}

		$where = "WHERE COALESCE(need_refresh,TRUE) != FALSE";
		$params = array();
		if( is_array($ifaceList) )
		{
			$in_sql = Model_esia::pdo_array_to_SQL('iface',$ifaceList,$params);
			$where .=" AND eg_access_iface_id IN ($in_sql) ";
		}
		elseif( $ifaceList != NULL )
		{
			$params[':iface'] = $ifaceList;
			$where .=" AND eg_access_iface_id = :iface";
		}
		if( is_array($servicesList) )
		{
			$in_sql = Model_esia::pdo_array_to_SQL('service',$servicesList,$params);
			$where .=" AND eg_access_service_id IN ($in_sql) ";
		}
		elseif( $servicesList != NULL )
		{
			$params[':service'] = $servicesList;
			$where .=" AND eg_access_service_id = :service";
		}

		$sql = "
					WITH vulscore AS (
						SELECT eg_access_service_id,MAX(vul_scoring) AS max,SUM(vul_scoring) AS sum,count(vul_id) AS total
						FROM egide_service_access
						JOIN egide_services ON(eg_access_service_id=eg_service_id)
						JOIN egide_vulnerability_detect ON(eg_service_id=detect_eg_service_id)
						JOIN egide_vulnerability_detect_state ON state_id=detect_state_id
						JOIN egide_vulnerability ON(detect_vul_id=vul_id)
						$where AND state_name = 'ACTIVE'
						GROUP BY eg_access_service_id
					)
					SELECT * FROM vulscore
					UNION
					SELECT eg_access_service_id,0,0,0
					FROM egide_service_access
					 $where AND eg_access_service_id NOT IN(SELECT eg_access_service_id FROM vulscore);
				";
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);

		$ret = array();
		while($row=$query->fetch(PDO::FETCH_ASSOC))
		{
			$ret[] = $row;
		}

		$service_to_device_sql = "
			SELECT DISTINCT eg_device_id,eg_service_id
			FROM egide_services
			WHERE eg_device_id = (SELECT eg_device_id FROM egide_services WHERE eg_service_id=:service_id);
		";

		$distinct_vul_sql = "
			SELECT DISTINCT eg_device_id,detect_vul_id
			FROM egide_vulnerability_detect
			JOIN egide_vulnerability_detect_state ON state_id=detect_state_id
			JOIN egide_services ON(detect_eg_service_id = eg_service_id)
			WHERE eg_device_id = :device_id AND state_name = 'ACTIVE';
		";

		$update_sql = "
				UPDATE egide_service_access
				SET max_score=:max,sum_score=:sum,total_vul=:total,vul_by_node=:vul_by_node,need_refresh = FALSE
				WHERE eg_access_service_id=:service_id
				";

		$service_to_device_query = $dbconnection->prepare($service_to_device_sql);
		$distinct_vul_query = $dbconnection->prepare($distinct_vul_sql);
		$update_query = $dbconnection->prepare($update_sql);

		$serviceToNodeList = array();
		$nodeVulTotal = array();
		foreach($ret as $row)
		{
			//On récupére tous les services d'un noeud (1 seule fois)
			if( !isset($serviceToNodeList[$row['eg_access_service_id']]) )
			{
				$params = array();
				$params[':service_id'] = $row['eg_access_service_id'];
				Model_esia::executeStatement($service_to_device_query,$params);
				while($tpl=$service_to_device_query->fetch(PDO::FETCH_ASSOC))
				{
					$serviceToNodeList[$tpl['eg_service_id']] = $tpl['eg_device_id'];
				}
			}
			$node_id = $serviceToNodeList[$row['eg_access_service_id']];

			//On calcule le nombre de vulnérabilité pour 1 noeud (1 seule fois)
			if( !isset($nodeVulTotal[$node_id]) )
			{
				$params = array();
				$params[':device_id'] = $node_id;
				$query = Model_esia::executeStatement($distinct_vul_query,$params);
				$data = $query->fetchAll();
				$nodeVulTotal[$node_id] = count($data);
			}

			$params = array();
			$params[':service_id'] = $row['eg_access_service_id'];
			$params[':max'] = $row['max'];
			$params[':sum'] = $row['sum'];
			$params[':total'] = $row['total'];
			$params[':vul_by_node'] = $nodeVulTotal[$node_id];
			$params[':service_id'] = $row['eg_access_service_id'];
			Model_esia::executeStatement($update_query,$params);
		}

		//On vérifie si on peut mettre le fichier refresh a need_refresh:FALSE
		$update_need_refresh_file = ( empty($ifaceList) && empty($$servicesList));
		if( !$update_need_refresh_file )
		{
			$sql = "select 'TRUE' from egide_service_access WHERE need_refresh = TRUE LIMIT 1;";
			Model_esia::executeStatement($update_query,$params);
			//SI on trouve une seule ligne, alors il reste des données qui n'ont pas été updatées.
			//Si on en trouve pas, alors on peut updater le fichier.
			if(!$service_to_device_query->fetch(PDO::FETCH_ASSOC))
			{
				$update_need_refresh_file = TRUE;
			}
		}

		if( $update_need_refresh_file )
		{
			//Création d'un répertoire temporaire si absent
			if( !is_dir(Model_egidescan::TMPDIR) )
			{
				mkdir(Model_egidescan::TMPDIR);
				chown(Model_egidescan::TMPDIR,'www-data');
				chgrp(Model_egidescan::TMPDIR,'www-data');
			}
			//Set du boolean a FALSE
			file_put_contents(Model_egidescan::FILE_UPDATE,"need_refresh:FALSE\n", LOCK_EX);
			chown(Model_egidescan::FILE_UPDATE,'www-data');
			chgrp(Model_egidescan::FILE_UPDATE,'www-data');
		}
	}

	public static function getIFacesList($dbconnection,$user,$treeview_type,$treeview_nodeid=NULL)
	{
		$params = array();
		$where = '';
		//echo '$treeview_type: '.$treeview_type .':'.$treeview_nodeid;
		if( $treeview_type == 'group' )
		{
			$box_list = self::getAgentsOfGroup($dbconnection,$treeview_nodeid,$user);
			$box_id_list = array();
			foreach($box_list as $box)
			{
				$box_id_list[] = $box['id'];
			}
			$in_sql = Model_esia::pdo_array_to_SQL('box_id',$box_id_list,$params);

			$where =" AND device_id IN ($in_sql) ";

		}
		elseif ($treeview_type == 'box')
		{
			$where = ' AND device_id = :box_id';
			$params[':box_id'] = $treeview_nodeid;
		}

		$sql = "
			SELECT vul_iface_id
			FROM vulnerability_iface
			JOIN esia_device ON (device_id = vul_iface_agent_id)
			WHERE device_id IN ( SELECT * FROM getUserAccessNode(:user)) $where;
			";
		$params[':user'] = $user;
		$query = $dbconnection->prepare($sql);
		Model_esia::executeStatement($query,$params);
		$ret = array();
		while($row=$query->fetch(PDO::FETCH_ASSOC))
		{
			$ret[] = $row['vul_iface_id'];
		}
		return $ret;
	}

	public static function getVulnerabilityStats($dbconnection,$ifaceList)
	{
                if( empty($ifaceList) )
                {
                        $ifaceList = NULL;
			return array();
                }

		$params = array();
		if( is_array($ifaceList) )
		{
			$in_sql = Model_esia::pdo_array_to_SQL('iface',$ifaceList,$params);
			$where =" eg_access_iface_id IN ($in_sql) ";
		}
		elseif( $ifaceList != NULL )
		{
			$params[':iface'] = $ifaceList;
			$where =" eg_access_iface_id = :iface";
		}
		$sql = "SELECT * FROM egide_service_access WHERE $where";

		$query = $dbconnection->prepare($sql);
		self::refreschScore($dbconnection,$ifaceList);
		$query = Model_esia::executeStatement($query,$params);

		$ret = array();
		while($row=$query->fetch(PDO::FETCH_ASSOC))
		{
			$retRow = array();
			$retRow['iface_id'] = $row['eg_access_iface_id'];
			$retRow['service_id'] = $row['eg_access_service_id'];
			$retRow['max_score'] = $row['max_score'];
			$retRow['sum_score'] = $row['sum_score'];
			$retRow['total_vul'] = $row['vul_by_node'];
			$retRow['total_match'] = $row['total_vul'];
			$ret[] = $retRow;
		}
		return $ret;
	}

	public static function getNodesStats($dbconnection,$user,$ifaceList,$limit=null,$distinct=FALSE,$nodeid=NULL)
	{
		$where = '';
		$word_where = 'WHERE';
		$params = array();

		if(!empty($user))
		{
			$params[':user'] = $user;
			$where .= " $word_where device_id IN ( SELECT * FROM getUserAccessNode(:user))";
			$word_where = 'AND';
		}

		if( empty($ifaceList) )
		{
				$ifaceList = NULL;
		}

		if( is_array($ifaceList) )
		{
			$in_sql = Model_esia::pdo_array_to_SQL('iface',$ifaceList,$params);
			$where .=" $word_where eg_access_iface_id IN ($in_sql) ";
			$word_where = 'AND';
		}
		elseif( $ifaceList != NULL )
		{
			$params[':iface'] = $ifaceList;
			$where .=" $word_where eg_access_iface_id = :iface";
			$word_where = 'AND';
		}

		if( $distinct )
		{
			$sql_select = 'DISTINCT ON (device_id)';
			$sql_order = 'ORDER BY device_id,avg_score DESC';
		}
		else
		{
			$sql_select = '';
			$sql_order = 'ORDER BY avg_score DESC';
		}
		if( isset($nodeid))
		{
			$where.= " $word_where device_id=:node_id";
			$params[':node_id'] = $nodeid;
			$word_where = 'AND';
		}

		$sql = "
			SELECT $sql_select device_id,vul_iface_id,vul_type_factor,vul_iface_id,vul_iface_label,vul_type_name,vul_type_color,device_type,device_ip_address,device_description,device_name,
			    max(max_score) AS max_score,
				sum(sum_score) AS sum_score,
				sum(total_vul) AS total_vul,
				max(vul_by_node) AS total_by_node,
			    (CASE WHEN sum(total_vul)=0 THEN 0 ELSE (sum(sum_score)/sum(total_vul)*vul_type_factor) END) AS avg_score
			FROM esia_device
			JOIN egide_services ON (device_id=eg_device_id)
			JOIN egide_service_access ON(eg_service_id=eg_access_service_id)
			JOIN vulnerability_iface ON(eg_access_iface_id = vul_iface_id)
			JOIN vulnerability_type ON(vul_iface_type_id = vul_type_id)
			$where
			GROUP BY device_id,vul_iface_id,vul_type_factor,vul_type_name,vul_type_color
			$sql_order
		";

		if(!empty($limit))
		{
			$sql .=" LIMIT :limit";
			$params[':limit'] = $limit;
		}

		$query = $dbconnection->prepare($sql);
		self::refreschScore($dbconnection,$ifaceList);
		$query = Model_esia::executeStatement($query, $params);

		$ret = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$retRow = array(
				'id' => $row->device_id,
                'name' => $row->device_name,
				'iface_id' => $row->vul_iface_id,
				'iface_label' => $row->vul_iface_label,
				'iface_type' => $row->vul_type_name,
				'iface_color' => $row->vul_type_color,
                'description' => $row->device_description,
				'type' => $row->device_type,
				'ip' => $row->device_ip_address,
				'total_vul' => $row->total_by_node,
				'total_match' => $row->total_vul,
				'sum_score' => $row->sum_score,
				'max_score' => $row->max_score,
                'numberVulnerability' => $row->total_by_node,
                'totalVulnerabilityScoring' => $row->sum_score,
                'averageVulnerabilityScoring' => round($row->avg_score,1),
                'maxVulnerabilityScoring' => $row->max_score
			);

			if( $row->total_vul == 0 )
			{
				$retRow['iface_id'] = NULL;
				$retRow['iface_label'] = NULL;
				$retRow['iface_type'] = NULL;
				$retRow['iface_color'] = NULL;
			}
			$ret[] = $retRow;
		}
		return $ret;
	}

	public static function getDeviceEsia($dbconnection, $id=NULL)
	{
		$sql = "SELECT device_name as dv_name, device_type as dv_type, device_ip_address as dv_ip FROM esia_device WHERE device_id=?";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array($id));
		$arr = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$arr=array('dv_name'=>$row->dv_name,
				'dv_type'=>$row->dv_type,
				'dv_ip'=>$row->dv_ip
			);
		}
		return $arr;
	}

	public static function getEgideBoxList($dbconnection,$userid)
	{
		$sql = "SELECT device_id,device_name FROM esia_device";
		$sql.= " WHERE device_id IN(SELECT gesa_device_id FROM gesa_info WHERE gesa_sn::varchar ~ '".Model_egidescan::BOX_FILTER_REGEX."')";
		$sql.= " AND device_id IN ( SELECT * FROM getUserAccessNode(:userid)) ORDER BY device_name ASC";
		$query = $dbconnection->prepare($sql);
		$arr = array(':userid'=>$userid);
		$query = Model_esia::executeStatement($query,$arr);

		$ret = array();
		while($row=$query->fetch(PDO::FETCH_ASSOC))
		{
			$retRow = array();
			$retRow['id'] = $row['device_id'];
			$retRow['name'] = $row['device_name'];
			$ret[] = $retRow;
		}
		return $ret;
	}

	//child peut être une box ou un groupe de noeud
	private static function returnBoxGroups(&$egide_box,$group_id)
	{
		$list=array();
		foreach($egide_box as $key => &$box)
		{
			$ind = array_search($group_id, $box['groups']);
			if($ind !== false)
			{
				$list[] =$box;
				unset($box['groups'][$ind]);
				if(empty($box['groups']))
				{
					unset($egide_box[$key]);
				}
			}
		}
		return $list;
	}
	private static function generateTreeViewBranch(&$group_list,$parent_id,&$egide_box)
	{
		$listgroup = array();
		foreach($group_list as $key => $grp)
		{
			if($grp->device_group_parent == $parent_id )
			{
				$arr = array('id'=>$grp->device_group_id,
						'name'=>$grp->device_group_name,
						'type'=>'group',
						'child'=>0,
						'childs'=>array(),
						);
				unset($group_list[$key]);
				$tmp  = self::generateTreeViewBranch($group_list,$arr['id'],$egide_box);
				$arr['child']=count($tmp);
				$arr['childs']=$tmp;
				$listgroup[$arr['id']] =$arr;
			}
		}
		foreach($egide_box as $key => &$box)
		{
			$ind = array_search($parent_id, $box['groups']);
			if($ind !== false)
			{
				$listgroup[] =$box;
				unset($box['groups'][$ind]);
				if(empty($box['groups']))
				{
					unset($egide_box[$key]);
				}
			}
		}
		return $listgroup;
	}

	private static function addBranchToTreeview($treeview,$branch)
	{
		if(empty($treeview))
		{
			$treeview = array();
		}
		if( empty($treeview[$branch['id']]))
		{
			$treeview[$branch['id']] = $branch;
		}
		else
		{
			foreach($branch['childs'] as $child)
			{
				$treeview[$branch['id']]['childs'] = self::addBranchToTreeview($treeview[$branch['id']]['childs'],$child);
			}
		}
		return $treeview;
	}

	private static function calculChild(&$treeview)
	{
		$treeview = array_values($treeview);
		foreach($treeview as $key => $branch)
		{
			if( !empty($branch['childs']))
			{
				$treeview[$key]['child'] = count($branch['childs']);
				self::calculChild($branch['childs']);
			}
			else
			{
				$treeview[$key]['child'] = 0;
			}
		}
	}

	private static function searchBranch($treeview,$nodeid)
	{
		foreach($treeview as $id => $branch)
		{
			if( $id == $nodeid )
			{
				return $branch['childs'];
			}
			else
			{
				$found = self::searchBranch($branch['childs'],$nodeid);
				if( $found !== FALSE )
				{
					return $found;
				}
			}
		}
		return FALSE;
	}

	public static function getEgideBoxTreeView($dbconnection,$userid,$groupid=NULL)
	{
		//Récupération de tous les groupes de noeuds
		$sql = "SELECT device_id,device_name,group_in_device_group_id
				FROM esia_device LEFT JOIN esia_device_group_in ON (device_id = group_in_device_id)
				WHERE device_id IN(SELECT gesa_device_id FROM gesa_info WHERE gesa_sn::varchar ~ '".Model_egidescan::BOX_FILTER_REGEX."')
					AND device_id IN ( SELECT * FROM getUserAccessNode(:userid));";
		$statement = $dbconnection->prepare($sql);
		$response = Model_esia::executeStatement($statement,array(':userid'=>$userid));
		$egide_box = array();
		$list_group=array();
		while($row = $response->fetch(PDO::FETCH_OBJ))
		{
			if(array_key_exists($row->device_id,$egide_box))
			{
				$egide_box[$row->device_id]['groups'][]=$row->group_in_device_group_id;
			}
			else
			{
				$egide_box[$row->device_id] = array('id'=>$row->device_id,
													 'name'=>$row->device_name,
													 'type'=>'box',
													 'child'=>0,
													 'groups'=>array($row->group_in_device_group_id));
			}
			if(!in_array($row->group_in_device_group_id,$list_group))
			{
				$list_group[]=$row->group_in_device_group_id;
			}
		}
		$groups = implode(',',$list_group);
		$params=array();
		if(!empty($groupid))
		{
			$sql = "WITH RECURSIVE accessgroup AS
					(
							SELECT device_group_id, device_group_name,device_group_parent
						   	FROM esia_device_group
							WHERE device_group_id IN ($groups)
						  	UNION ALL
						 	SELECT m.device_group_id, m.device_group_name,m.device_group_parent
						   	FROM esia_device_group m
						   	JOIN accessgroup ON accessgroup.device_group_parent = m.device_group_id
					)
					SELECT * FROM accessgroup WHERE device_group_parent=:nodegroup;";
				$params[':nodegroup']=$groupid;
		}
		else
		{
			$sql = "WITH RECURSIVE accessgroup AS
					(
							SELECT device_group_id, device_group_name,device_group_parent
						   	FROM esia_device_group
							WHERE device_group_id IN ($groups)
						  	UNION ALL
						 	SELECT m.device_group_id, m.device_group_name,m.device_group_parent
						   	FROM esia_device_group m
						   	JOIN accessgroup ON accessgroup.device_group_parent = m.device_group_id
					)
					SELECT * FROM accessgroup
					ORDER BY device_group_parent NULLS FIRST;";
		}
		//echo $sql;
		$statement = $dbconnection->prepare($sql);
		$response = Model_esia::executeStatement($statement,$params);
		$tree = array();
		$listgroup = $response->fetchAll(PDO::FETCH_CLASS);
		//echo '<pre>';print_r($listgroup);
		foreach($listgroup as $key=> $group)
		{
			if(empty($group->device_group_parent) || !empty($groupid))
			{
				$tree[]=array(
						'id'=>$group->device_group_id,
						'name'=>$group->device_group_name,
						'type'=>'group',
						'child'=>1,
						'childs'=>array(),
						);
				unset($listgroup[$key]);
			}
			else
			{
				//on en a fini avec les tops groups
				break;
			}
		}
		$arr=array();
		if(!empty($groupid))
		{
			$arr  = self::returnBoxGroups($egide_box,$groupid);
			$tree = array_merge($tree,$arr);
		}
	//	print_r($tree);
		return $tree;
	}
	public static function getConfiguratedEgideBoxIfaces($dbconnection,$node_id)
	{
		$sql = "SELECT vul_iface_id,vul_iface_agent_id,vul_iface_local_name,vul_iface_label,vul_type_id,vul_type_name,vul_type_factor,vul_type_color";
		$sql.= " FROM vulnerability_iface INNER JOIN vulnerability_type";
		$sql.= " ON(vul_iface_type_id = vul_type_id)";

		if( $node_id != NULL)
		{
			$sql.= " WHERE vul_iface_agent_id = :node_id";
			$arr = array(':node_id'=>$node_id);
		}

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$arr);
		$ret = array();
		while($row=$query->fetch(PDO::FETCH_ASSOC))
		{
			$retRow = array();
			$retRow['id'] = $row['vul_iface_id'];
			$retRow['label'] = $row['vul_iface_label'];
			$retRow['local_name'] = $row['vul_iface_local_name'];
			$retRow['type_name'] = $row['vul_type_name'];
			$retRow['type_id'] = $row['vul_type_id'];
			$retRow['type_factor'] = $row['vul_type_factor'];
			$retRow['type_color'] = $row['vul_type_color'];
			$retRow['agent_id'] = $row['vul_iface_agent_id'];
			$ret[] = $retRow;
		}
		return $ret;
	}

	public static function getEgideIfacesType($dbconnection)
	{
		$sql = 'SELECT * from vulnerability_type ORDER BY vul_type_factor DESC';
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query);
		$ret = array();
		while($row=$query->fetch(PDO::FETCH_ASSOC))
		{
			$retRow = array();
			$retRow['id'] = $row['vul_type_id'];
			$retRow['name'] = $row['vul_type_name'];
			$retRow['factor'] = $row['vul_type_factor'];
			$retRow['color'] = $row['vul_type_color'];
			$ret[] = $retRow;
		}
		return $ret;
	}
	/**
	*** Modifier une la pondération d'un type d'interface
	*** @param PDO $dbconnection un objet PDO
	*** @param float $factor le nouveau nombre pour la ponderation (0.2, 0.4) max 1.0
	*** @param integer (optionel) $typeId identifiant du type d'interface a modifier (si non spécifier $typeName est requis)
	*** @param string (optionel) $typeName nom du type d'interface a modifier (si non spécifier $typeId est requis)
	*** @exception EsiaDbException
	**/
	public static function updateEgideIfacesTypeFactor($dbconnection, $factor, $typeId=NULL, $typeName=NULL)
	{
		if(!is_numeric($factor))
		{
			throw new EsiaDbException('Iface type factor must be numeric [' . $factor . ']');
		}
		elseif($factor < 0 || $factor > 1.0)
		{
			throw new EsiaDbException('Iface type factor must be between 0 and 1.0 [' . $factor . ']');
		}

		$sql = "UPDATE vulnerability_type SET vul_type_factor = :factor::real";

		$params = array(':factor' => $factor);

		if(isset($typeId) && is_int($typeId))
		{
			$sql .= " WHERE vul_type_id = :type_id";
			$params[':type_id'] = $typeId;
		}
		elseif(isset($typeName) && is_string($typeName))
		{
			$sql .= " WHERE vul_type_name = :type_name";
			$params[':type_name'] = $typeName;
		}
		else
		{
			throw new EsiaDbException('Wrong iface type selector id or name is required');
		}

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);
	}

	public static function addEgideIfaceConfiguration($dbconnection,$box_id,$local_name,$label,$type)
	{
		$sql = 'INSERT INTO vulnerability_iface (vul_iface_agent_id,vul_iface_local_name,vul_iface_label,vul_iface_type_id)';
		$sql.= ' VALUES(:agent,:local,:label,:type) RETURNING vul_iface_id';

		$query = $dbconnection->prepare($sql);
		$arr[':agent'] = $box_id;
		$arr[':local'] = $local_name;
		$arr[':label'] = $label;
		$arr[':type'] = $type;
		$query = Model_esia::executeStatement($query,$arr);
		$row=$query->fetch(PDO::FETCH_OBJ);
		return $row->vul_iface_id;
	}

	public static function updateEgideIfaceConfiguration($dbconnection,$box_id,$local_name,$label,$type)
	{
		$sql = 'UPDATE vulnerability_iface SET vul_iface_label = :label,vul_iface_type_id = :type';
		$sql.= ' WHERE vul_iface_agent_id = :agent AND vul_iface_local_name = :local';

		$query = $dbconnection->prepare($sql);
		$arr[':agent'] = $box_id;
		$arr[':local'] = $local_name;
		$arr[':label'] = $label;
		$arr[':type'] = $type;
		$query = Model_esia::executeStatement($query,$arr);
	}

	public static function getIpDevice($dbconnection, $name=NULL)
	{
		$sql = "SELECT device_ip_address as dv_ip FROM esia_device WHERE device_name=?";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array($name));
		$arr = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$arr=array(
				'dv_ip'=>$row->dv_ip
			);
		}
		return $arr;
	}

	public static function notifyScanningError($dbconnection,$scan_id,$message)
	{
		$scan_info = Model_egidescan::getScanById($dbconnection,$scan_id);
		$start = $scan_info['next_exec'];
		$end = date("Y-m-d H:i:s");
		$result['message'] = $message;
		self::addScanHistory($dbconnection,$start,$end,$scan_id,$result,Model_egidescan::ERROR_TASK);
		self::updateScanningTaskStatus($dbconnection,$scan_id,Model_egidescan::ERROR_TASK);
	}

	public static function addScanHistory($dbconnection, $start,$end,$scan_id,$result,$scan_state=NULL)
	{
		$sql = "INSERT INTO public.vulnerability_scanner_history (scan_date_start, scan_date_end,scan_id,scan_result,scan_state) VALUES (:scan_start, :scan_end, :scan_id, :result,:scan_state);";
		$params[':scan_start'] = $start;
		$params[':scan_end'] = $end;
		$params[':scan_id'] = $scan_id;
		$params[':scan_state'] = $scan_state;
		$params[':result'] = json_encode($result);

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);
	}

	public static function getVulnerability($dbconnection)
	{
		$sql = "SELECT vul_list_id as vul_id_1 FROM public.egide_vulnerability;";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query);
		$row=$query->fetchAll(PDO::FETCH_ASSOC);
		if(empty($row))
		{
			$row = 'NONE';
		}

		return $row;
	}

	public static function getScore($dbconnection, $cve_id=NULL)
	{
		$sql = "SELECT vul_scoring as score FROM public.egide_vulnerability WHERE vul_list_id=:cve_id;";

		$params = array(
			':cve_id' => $cve_id
		);

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);
		$access = $query->fetch(PDO::FETCH_OBJ);
		$score = $access->score;
		if(empty($score))
		{
			$score = 'NONE';
		}

		return $score;
	}

	public static function getDevicesOfAgent($dbconnection,$agent_id,$user_id=NULL)
	{
		$sql = "
		SELECT DISTINCT eg_service_id,eg_device_id
		FROM vulnerability_iface
		INNER JOIN egide_service_access ON (vul_iface_id = eg_access_iface_id)
		INNER JOIN egide_services ON (eg_access_service_id = eg_service_id)
		WHERE vul_iface_agent_id = :agent_id
		";

		$params[':agent_id'] = $agent_id;
		if( !empty($user_id) )
		{
			$params[':userid'] = $user_id;
			$sql.= ' AND vul_iface_agent_id IN ( SELECT * FROM getUserAccessNode(:userid))';
		}
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);
		$row=$query->fetchAll(PDO::FETCH_ASSOC);
		return $row;
	}

	private static function getTreeViewBox($treeview)
	{
		$box_list = array();
		foreach($treeview as $branch)
		{
			if( $branch['type'] == 'box')
			{
				$box_list[] = $branch;
			}
			if( !empty($branch['childs']) )
			{
				$box_list = array_merge($box_list,self::getTreeViewBox($branch['childs']));
			}
		}
		return $box_list;
	}

	public static function getAgentsOfGroup($dbconnection,$group_id,$userid)
	{
		//Récupération de tous les groupes de noeuds
		$sql = "WITH RECURSIVE accessgroup AS
				(
					SELECT device_group_id, device_group_name,device_group_parent
					FROM esia_device_group
					WHERE device_group_parent = :groupid1 OR device_group_id = :groupid2
					UNION ALL
					SELECT m.device_group_id, m.device_group_name,m.device_group_parent
					FROM esia_device_group m
					JOIN accessgroup ON accessgroup.device_group_id = m.device_group_parent
				)
				SELECT DISTINCT device_id,device_name
				FROM esia_device
				LEFT JOIN esia_device_group_in ON (device_id = group_in_device_id)
				WHERE device_id IN(SELECT gesa_device_id FROM gesa_info WHERE gesa_sn::varchar ~ '".Model_egidescan::BOX_FILTER_REGEX."')
					AND device_id IN ( SELECT * FROM getUserAccessNode(:userid)) AND group_in_device_group_id IN (SELECT device_group_id FROM accessgroup);";

		$statement = $dbconnection->prepare($sql);
		$response = Model_esia::executeStatement($statement,array(':userid'=>$userid,':groupid1'=>$group_id,':groupid2'=>$group_id ));
		$egide_boxes = array();
		while($row = $response->fetch(PDO::FETCH_ASSOC))
		{
			$egide_boxes[] = array('id' => $row['device_id'],'name'=>$row['device_name'],'type'=>'box','child'=>0);
		}
		return $egide_boxes;
	}

	public static function getDevicesOfAgentGroup($dbconnection,$group_id,$userid)
	{
		$box_list = self::getAgentsOfGroup($dbconnection,$group_id,$userid);

		$devices_list = array();
		foreach($box_list as $box)
		{
			$devices_list = array_merge($devices_list,self::getDevicesOfAgent($dbconnection,$box['id']));
		}
		return $devices_list;
	}

	public static function getDevices($dbconnection)
	{
		$sql = "SELECT eg_service_id, eg_device_id FROM public.egide_services;";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query);
		$row=$query->fetchAll(PDO::FETCH_ASSOC);
		if(empty($row))
		{
			$row = 'NONE';
		}

		return $row;
	}

	public static function getDevicesGroup_node($dbconnection,$arr)
	{
		$tab_group_node = Model_esia::getAllNodeofGroup($dbconnection,$userid,$arr);
		foreach ($tab_group_node as $node)
		{
			$nodes[] = $node->getId();
		}

		$sql = "SELECT eg_service_id, eg_device_id FROM public.egide_services where";
		$it = 0 ;
		foreach($nodes as $node)
		{
			$sql .= " eg_device_id=$node " ;
			$it++ ;
			if($it<count($nodes))
			{
				$sql .= " OR " ;
			}
		}

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query);
		$row=$query->fetchAll(PDO::FETCH_ASSOC);
		if(empty($row))
		{
			$row = 'NONE';
		}

		return $row;
	}

	public static function getDeviceId($dbconnection,$dvc_name=NULL)
	{
		$sql = "SELECT device_id FROM public.esia_device WHERE device_name = :dvc_name";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array($dvc_name));
		$row=$query->fetch(PDO::FETCH_OBJ);
		$dv_id = $row->device_id ;
		if(empty($dv_id))
		{
			$dv_id = 'NONE';
		}

		return $dv_id ;
	}

	public static function getCVE_Id($dbconnection,$cve_list=NULL)
	{
		$sql = "SELECT vul_id FROM public.egide_vulnerability WHERE vul_list_id = :cve_list";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array($cve_list));
		$row=$query->fetch(PDO::FETCH_OBJ);
		$vl_id = $row->vul_id ;
		if(empty($vl_id))
		{
			$vl_id = 'NONE';
		}

		return $vl_id ;
	}

	private static function executeAndparseScannerHistoryRequest($dbconnection,$sql,$params,$with_devices)
	{
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);

		$arr = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$retRow=array(
				'id'=>$row->scan_history_id,
				'date_start'=>$row->scan_date_start,
				'date_end'=>$row->scan_date_end,
				'scan_state'=>$row->scan_state,
				'scan_id'=>$row->scan_id,
				'result'=>$row->scan_result
			);
			if( $with_devices )
			{
				// recherche du groupe name
				$option = $row->scan_option ;
				$option = json_decode($option);
				$group_node = $option->{'group_list'};
				$group_id = $group_node[0];
				$group = Model_esia::getGroupDeviceIn($dbconnection, $group_id);
				$group = array_shift($group);
				$groupsName = $group['name'];

				$retRow = array_merge($retRow,array(
					'iface_name'=>$row->vul_iface_local_name,
					'device_group_name'=>$row->device_group_name,
					'device_name'=>$row->device_name,
					'group_name' => $groupsName
				));
			}
			$arr[]=$retRow;
		}
		return $arr;
	}

	public static function getScannerHistory($dbconnection,$id)
	{
		$params = array();
		$sql = "SELECT scan_history_id,scan_date_start,scan_date_end,scan_id,scan_state,scan_result
						FROM vulnerability_scanner_history
						WHERE scan_history_id = :id";

		$ret = self::executeAndparseScannerHistoryRequest($dbconnection,$sql,array(':id'=>$id),FALSE);
		return array_shift($ret);
	}

	public static function getVulnerability_scanner_history($dbconnection,$agent_id = NULL,$limit = NULL)
	{
		$params = array();
		$sql = "
		SELECT scan_history_id,scan_date_start,scan_date_end,vulnerability_scanner_history.scan_id,vulnerability_scanner_history.scan_state,vulnerability_scanner.scan_option,scan_result,vul_iface_local_name,device_group_name,device_name
		FROM vulnerability_scanner_history
		JOIN vulnerability_scanner ON(vulnerability_scanner_history.scan_id = vulnerability_scanner.scan_id)
		JOIN vulnerability_iface ON(scan_iface_id = vul_iface_id)
		JOIN esia_device ON(esia_device.device_id = vulnerability_iface.vul_iface_agent_id)
		";

		if( $agent_id != NULL )
		{
			$in_request = generateInParams('WHERE',$agent_id,'vul_iface_agent_id',TRUE);
			$params = $in_request['param'];
			$where = $in_request['sql'];

			$sql = "$sql $where";
		}
		if( $limit != NULL )
		{
			$sql = "$sql ORDER BY scan_history_id DESC LIMIT :limit";
			$params[':limit'] = $limit;
		}
		return self::executeAndparseScannerHistoryRequest($dbconnection,$sql,$params,TRUE);
	}

	public static function getEgideServicesNode($dbconnection,$node_id,$iface_id = NULL)
	{
		$sql =  "SELECT eg_service_id, eg_service_name, eg_service_product, eg_service_version, eg_service_port, eg_service_cpe,
						eg_service_extrainfo,eg_service_port_type
				 FROM public.egide_services
				 JOIN egide_service_access ON (eg_access_service_id = eg_service_id)
				 WHERE eg_device_id = ?";

		$params = array($node_id);
		if( $iface_id  != NULL )
		{
			$sql.= " AND eg_access_iface_id = ?";
			$params[] = $iface_id;
		}
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);
		$arr = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$arr[$row->eg_service_port]=array('id'=>$row->eg_service_id,
				'name'=>$row->eg_service_name,
				'product'=>$row->eg_service_product,
				'version'=>$row->eg_service_version,
				'port'=>$row->eg_service_port,
				'port_type'=>$row->eg_service_port_type,
				'cpe'=>$row->eg_service_cpe,
				'extrainfo'=>$row->eg_service_extrainfo);
		}
		return $arr;
	}

// 	public static function getServicesNode($dbconnection)
// 	{
// 		$sql = "SELECT eg_service_id, eg_service_name, eg_service_product, eg_service_version, eg_service_port, eg_service_cpe,
// 					eg_service_extrainfo,eg_service_port_type, detect_vul_id,vul_scoring, device_ip_address, device_name
// 				FROM public.egide_services
// 				JOIN egide_vulnerability_detect ON eg_service_id = detect_eg_service_id
// 				JOIN esia_device ON device_id = eg_device_id
// 				JOIN egide_vulnerability ON vul_id = detect_vul_id";
// 		$query = $dbconnection->prepare($sql);
// 		$query = Model_esia::executeStatement($query);
// 		$arr = array();
// 		while($row=$query->fetch(PDO::FETCH_OBJ))
// 		{
// 			$arr[]=array('id'=>$row->eg_service_id,
// 				'name'=>$row->eg_service_name,
// 				'product'=>$row->eg_service_product,
// 				'version'=>$row->eg_service_version,
// 				'port'=>$row->eg_service_port,
// 				'port_type'=>$row->eg_service_port_type,
// 				'cpe'=>$row->eg_service_cpe,
// 				'vul_id'=>$row->detect_vul_id,
// 				'devc_ip'=>$row->device_ip_address,
// 				'devc_name'=>$row->device_name,
// 				'vul_score'=>$row->vul_scoring,
// 				'extrainfo'=>$row->eg_service_extrainfo);
// 		}
// 		return $arr;
// 	}

	public static function getServicesByCVE($dbconnection)
	{
		$sql = "SELECT eg_service_name,vul_list_id
				FROM public.egide_services
				JOIN egide_vulnerability_detect ON eg_service_id = detect_eg_service_id
				JOIN egide_vulnerability ON vul_id = detect_vul_id";
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query);
		$arr = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$arr[]=array(
				'cve'=>$row->vul_list_id,
				'name'=>$row->eg_service_name
			);
		}
		return $arr;
	}

	public static function getCVEByNodes($dbconnection,$nodes)
	{
		$sql = "SELECT distinct vul_list_id, count(vul_list_id) as match, vul_description, vul_scoring, vul_severity, vul_attack_complexity
				FROM public.egide_vulnerability
				JOIN egide_vulnerability_detect ON vul_id = detect_vul_id
				JOIN egide_services ON eg_service_id = detect_eg_service_id
				where";
		foreach ($nodes as $node)
		{
			$sql=$sql." eg_device_id = ".$node." OR " ;
		}
		$sql = substr($sql,0,-3);
		$sql=$sql."group by vul_list_id,vul_description,vul_scoring,vul_severity,vul_attack_complexity" ;
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query);
		$arr = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$arr[$row->vul_list_id]=array(
				'cve'=>$row->vul_list_id,
				'cve_descr'=>$row->vul_description,
				'cve_score'=>$row->vul_scoring,
				'cve_severt'=>$row->vul_severity,
				'cve_cmplx'=>$row->vul_attack_complexity,
				'match'=>$row->match,
			);
		}
		return $arr;
	}

	public static function getCVEByServices($dbconnection,$service_id)
	{
		$sql = "SELECT distinct vul_list_id, vul_date_publishing, vul_description, vul_scoring, vul_severity, vul_attack_complexity
				FROM public.egide_vulnerability
				JOIN egide_vulnerability_detect ON vul_id = detect_vul_id
				JOIN egide_services ON eg_service_id = detect_eg_service_id
				where eg_service_id = ".$service_id;

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query);
		$arr = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$arr[$row->vul_list_id]=array(
				'id'=>$row->vul_list_id,
				'cve_pub'=>$row->vul_date_publishing,
				'description'=>$row->vul_description,
				'scoring'=>$row->vul_scoring,
				'severity'=>$row->vul_severity,
				'complexity'=>$row->vul_attack_complexity,
				'css'=> getCSSClassByscore($row->vul_scoring)
			);
		}
		return $arr;
	}

	public static function insertEgideServicesNode($dbconnection,$node,$name,$product,$version,$port,$type,$cpe,$extra,$iface_id)
	{
		$sql =  "INSERT INTO public.egide_services(eg_service_id,eg_service_name, eg_service_product, eg_service_version,
						eg_service_port,eg_service_port_type, eg_service_cpe, eg_service_extrainfo, eg_device_id)
				VALUES (DEFAULT,:name,:product,:version,:port,:type,:cpe,:extra,:node) RETURNING eg_service_id ;";
		$query = $dbconnection->prepare($sql);
		$params = array(':node'=>$node,
			':name'=>$name,
			':product'=>$product,
			':version'=>$version,
			':port'=>$port,
			':type'=>$type,
			':cpe'=>$cpe,
			':extra'=>$extra
		);
		$query = Model_esia::executeStatement($query,$params);
		$row=$query->fetch(PDO::FETCH_OBJ);

		self::linkServiceToIface($dbconnection,$iface_id,$row->eg_service_id);
		Model_esia::addNodeTabs($dbconnection, $node, 'TAB_EGIDESCAN');
		return $row->eg_service_id;
	}

	public static function linkServiceToIface($dbconnection,$iface_id,$service_id)
	{
		$sql = "INSERT INTO egide_service_access (eg_access_iface_id,eg_access_service_id)";
		$sql.= " VALUES(:iface_id,:service_id) ON CONFLICT (eg_access_iface_id,eg_access_service_id) DO NOTHING";
		$query = $dbconnection->prepare($sql);
		$params[':iface_id'] = $iface_id;
		$params[':service_id'] = $service_id;
		$query = Model_esia::executeStatement($query,$params);
	}

	public static function updateEgideServicesNode($dbconnection,$id,$name,$product,$version,$cpe,$extra,$iface_id)
	{
		$sql =  "UPDATE public.egide_services
				SET eg_service_name=:name,
					eg_service_product=:product,
					eg_service_version=:version,
					eg_service_cpe=:cpe,
					eg_service_extrainfo=:extra
				WHERE  eg_service_id=:id;";
		$query = $dbconnection->prepare($sql);
		$params = array(':id'=>$id,
			':name'=>$name,
			':product'=>$product,
			':version'=>$version,
			':cpe'=>$cpe,
			':extra'=>$extra
		);
		$query = Model_esia::executeStatement($query,$params);
		$row=$query->fetch(PDO::FETCH_OBJ);
		self::linkServiceToIface($dbconnection,$iface_id,$id);
		return $id;
	}
	public static function insertVulnerabilityDetect($dbconnection,$serviceid,$provider, $vulid, $scan_date)
	{
		//récupération du provider
		$sql= "SELECT vul_id, vul_provider_id, vul_list_id	FROM egide_vulnerability";
		if(is_string($provider))
		{
			$sql.=" WHERE vul_provider_id IN (SELECT provider_id FROM egide_provider WHERE provider_name = :provider)";

		}
		else
		{
			$sql.=" WHERE vul_provider_id = :provider";
		}
		$params[":provider"]=$provider;
		//récupération des CVEID
		if(is_array($vulid))
		{
			$arrids = implode("','", $vulid);
			$sql.= " AND vul_list_id IN ('$arrids')";
		}
		else
		{
			$sql.= " AND vul_list_id = :vulid";
			$params[":vulid"]=$vulid;
		}

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);

		//préparation d'une reqûete d'insertion
		$queryinsert = $dbconnection->prepare("INSERT INTO egide_vulnerability_detect(detect_eg_service_id, detect_vul_id, detect_date,detect_date_update_state , detect_state,detect_state_id)
				 VALUES (:serviceid,:vulid,to_timestamp(:detect_date),to_timestamp(:detect_date),'ACTIVE',(SELECT state_id FROM egide_vulnerability_detect_state WHERE state_name='ACTIVE'))
				 ON CONFLICT ON CONSTRAINT \"detect_fk_uniq\"
				 	DO UPDATE SET detect_date_update_state=to_timestamp(:detect_date),detect_state = 'ACTIVE', detect_state_id=(SELECT state_id FROM egide_vulnerability_detect_state WHERE state_name='ACTIVE');");
		$inserparams[':serviceid']=$serviceid;
		$inserparams[':detect_date'] = $scan_date;

		$i=0;
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$inserparams[':vulid']=$row->vul_id;
			$queryinsert = Model_esia::executeStatement($queryinsert,$inserparams);
			$i++;
		}

		self::fireScoreNeedRefresh($dbconnection,$serviceid);
		return $i;
	}

	public static function updateNotDetected($dbconnection,$serviceid, $scan_date)
	{
		$params[':detect_date'] = $scan_date;
		$params[':service_id'] = $serviceid;

		$sql = "UPDATE egide_vulnerability_detect
				SET detect_state = 'CLOSED',
					detect_state_id = (SELECT state_id FROM egide_vulnerability_detect_state WHERE state_name='CLOSED'),
					detect_date_update_state = to_timestamp(:detect_date)
				WHERE detect_state_id IN (SELECT state_id FROM egide_vulnerability_detect_state WHERE state_name<>'CLOSED')
				AND detect_date != to_timestamp(:detect_date)
				AND detect_date_update_state != to_timestamp(:detect_date)
				AND detect_eg_service_id = :service_id";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);

		self::fireScoreNeedRefresh($dbconnection,$serviceid);
		return $query->rowCount();
	}

	//mise à jour :

	public static function getEgideVulDetect($dbconnection)
	{
		$sql =  "SELECT detect_eg_service_id as detect_eg_service_id1, detect_vul_id as detect_vul_id1
	FROM public.egide_vulnerability_detect;";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query);
		$row=$query->fetchAll(PDO::FETCH_ASSOC);
		if(empty($row))
		{
			$row = 'NONE';
		}

		return $row;
	}

	public static function getCVEVulDetect($dbconnection,$agent_id=NULL)
	{
		$sql = "SELECT count(detect_vul_id) as match,vul_list_id,vul_description, vul_scoring, vul_severity, vul_attack_complexity";
		$sql.= " FROM egide_vulnerability";
		$sql.= " JOIN egide_vulnerability_detect ON vul_id = detect_vul_id";
		$sql.= " JOIN egide_vulnerability_detect_state ON state_id=detect_state_id";
		$sql.= " WHERE state_name = 'ACTIVE'";

		$params = array();
		if( $agent_id != NULL )
		{
			$sub_req = "
				SELECT eg_service_id
				FROM egide_services
				INNER JOIN egide_service_access ON (eg_access_service_id = eg_service_id)
				INNER JOIN vulnerability_iface ON (vul_iface_id = eg_access_iface_id)
				WHERE vul_iface_agent_id = :agent_id
			";
			$sql.= "AND detect_eg_service_id IN($sub_req)";
			$params[':agent_id'] = $agent_id;
		}
		$sql.= " GROUP BY vul_list_id,vul_description, vul_scoring, vul_severity, vul_attack_complexity";
		$sql.= " ORDER BY vul_list_id DESC";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$arr[]=array(
				'cve'=>$row->vul_list_id,
				'cve_descr'=>$row->vul_description,
				'cve_score'=>$row->vul_scoring,
				'cve_severt'=>$row->vul_severity,
				'cve_cmplx'=>$row->vul_attack_complexity,
				'match_vul'=> $row->match
			);
		}
		return $arr;
	}

	public static function getCVEVulDetectByIface($dbconnection,$iface_id)
	{
		$sql = "
			SELECT count(detect_vul_id) as match,vul_list_id, vul_scoring, vul_severity, vul_attack_complexity
			FROM vulnerability_iface
			JOIN egide_service_access ON (vul_iface_id = eg_access_iface_id)
			JOIN egide_services ON (eg_access_service_id = eg_service_id)
			JOIN egide_vulnerability_detect ON (eg_service_id = detect_eg_service_id)
			JOIN egide_vulnerability_detect_state ON state_id=detect_state_id
			JOIN egide_vulnerability ON (vul_id = detect_vul_id)
			WHERE vul_iface_id = :iface
				AND state_name = 'ACTIVE'
			GROUP BY vul_list_id,vul_description, vul_scoring, vul_severity, vul_attack_complexity
			ORDER BY vul_list_id DESC;
		";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array(':iface'=>$iface_id));
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$arr[]=array(
				'cve'=>$row->vul_list_id,
				'cve_score'=>$row->vul_scoring,
				'cve_severt'=>$row->vul_severity,
				'cve_cmplx'=>$row->vul_attack_complexity,
				'match_vul'=> $row->match
			);
		}
		return $arr;
	}

	public static function getLastCVEUpdated($dbconnection)
	{
		$sql = "SELECT MAX(EXTRACT(DAY FROM (now()::date - vul_last_update))) as lastUpdate
	FROM public.egide_vulnerability ;";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query);

		$access = $query->fetch(PDO::FETCH_ASSOC);

		$lastUpdate = $access["lastupdate"];
// 		echo $lastUpdate;

		return $lastUpdate ;

	}

	public static function getCVE($dbconnection,$id_vul)
	{
		$sql =  "SELECT vul_list_id, vul_description, vul_scoring, vul_severity, vul_attack_complexity
				 FROM public.egide_vulnerability
				 WHERE vul_id = ?";
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array($id_vul));
		//$arr = array();

		$row=$query->fetch(PDO::FETCH_OBJ);
		$arr=array('cve'=>$row->vul_list_id,
			'cve_descr'=>$row->vul_description,
			'cve_score'=>$row->vul_scoring,
			'cve_severt'=>$row->vul_severity,
			'cve_cmplx'=>$row->vul_attack_complexity
		);
		return $arr;
	}

	public static function getServices_infos($dbconnection,$srvcId)
	{
		$sql =  "SELECT eg_service_name, eg_service_port, eg_service_product, eg_service_port_type
				 FROM public.egide_services
				 WHERE eg_service_id = ?";
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array($srvcId));
		//$arr = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$arr=array('srvc_name'=>$row->eg_service_name,
				'srvc_port'=>$row->eg_service_port,
				'srvc_product'=>$row->eg_service_product,
				'srvc_port_type'=>$row->eg_service_port_type
			);
		}
		return $arr;
	}

	public static function getDetectCVEid($dbconnection,$detect_eg_serv_id=NULL)
	{
		$sql =  "SELECT detect_vul_id as detect_vul_id1
				 FROM public.egide_vulnerability_detect
				 WHERE detect_eg_service_id = :detect_eg_serv_id";
		$params = array(
			':detect_eg_serv_id' => $detect_eg_serv_id
		);
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);
		$row=$query->fetchAll(PDO::FETCH_ASSOC) ;
		//$detect_vul_id = $row->detect_vul_id1;
		if(empty($detect_vul_id))
		{
			$detect_vul_id = 'NONE';
		}

		return $row;
	}

	public static function getDetectCVEservice($dbconnection,$detect_vl_id=NULL)
	{
		$sql =  "SELECT detect_eg_service_id
				 FROM public.egide_vulnerability_detect
				 WHERE detect_vul_id = :detect_vl_id AND detect_state = 'ACTIVE'";
		$params = array(
			':detect_vl_id' => $detect_vl_id
		);
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);
		$row=$query->fetchAll(PDO::FETCH_ASSOC) ;
		//$detect_vul_id = $row->detect_vul_id1;
		if(empty($row))
		{
			$row = 'NONE';
		}

		return $row;
	}

	public static function getDevice_services($dbconnection,$srvc=NULL)
	{
		$sql =  "SELECT device_name
				 FROM public.esia_device
				 WHERE device_id=(SELECT eg_device_id FROM public.egide_services WHERE eg_service_id = :srvc) ";
		$params = array(
			':srvc' => $srvc
		);
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);
		$row=$query->fetch(PDO::FETCH_OBJ) ;
		$dvc_name = $row->device_name;
		if(empty($dvc_name))
		{
			$dvc_name = 'NONE';
		}

		return $dvc_name;
	}

	public static function updateVulnerabilityInfo($dbconnection,$vulUrl=NULL,$vulScore=NULL,$vulSeverity=NULL,$vulComplexity=NULL,$idvulnerability=NULL)
	{
		//$sql = "INSERT INTO public.egide_vulnerability(vul_url, vul_scoring, vul_severity, vul_attack_complexity) VALUES (:vulUrl , :vulScore, :vulSeverity, :vulComplexity);";
		$sql = "UPDATE public.egide_vulnerability
						SET vul_url= :vulUrl,
							vul_scoring= :vulScore,
							vul_severity= :vulSeverity,
							vul_attack_complexity= :vulComplexity
					WHERE vul_id = :idvulnerability;";
		$params = array(
			':vulUrl' =>$vulUrl,
			':vulScore' =>$vulScore,
			':vulSeverity' =>$vulSeverity,
			':vulComplexity' =>$vulComplexity,
			':idvulnerability' =>$idvulnerability
		);

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);
	}

	// scan config
	public static function Lastscan_id($dbconnection)
	{
		$sql0 = "SELECT max(scan_id) as lastscan FROM public.vulnerability_scanner;";
		$query0 = $dbconnection->prepare($sql0);
		$query0 = Model_esia::executeStatement($query0);
		$access = $query0->fetch(PDO::FETCH_OBJ);
		$id_lastScan = $access->lastscan;
		if(empty($id_lastScan) && ($id_lastScan!=0))
		{
			$id_lastScan = 0;
			return $id_lastScan ;
		}
		else
		{
			//$id_lastScan++;
			return $id_lastScan ;
		}
	}

	public static function addScan($dbconnection,$start_date,$start_time,$interval_value,$interval_unit,$iface_id,$list_scan,$retries,$retries_interval)
	{
		$cron_plugin = Model_esia::getCronPlugin($dbconnection,Model_egidescan::PLUGIN_CRON_START);
		$cron_plugin = array_shift($cron_plugin);

		$sql = 'INSERT INTO vulnerability_scanner';
		$sql.= ' (scan_iface_id,scan_option)';
		$sql.= " VALUES (:iface,:options)";
		$sql.= ' RETURNING scan_id';

		//Calcul de date
		$dt = DateTime::createFromFormat('d/m/Y H:i:s',"$start_date 00:00:00");
		$dt->add(new DateInterval("PT$start_time".'S'));

		//Création des données pour le scan_option json
		$options['group_list'] = $list_scan;
		$options['interval_value'] = $interval_value;
		$options['interval_unit'] = $interval_unit;

		//Configuration des paramètres
		$params[':iface'] = $iface_id;
		$params[':options'] = json_encode($options);

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);
		$row=$query->fetch(PDO::FETCH_OBJ);

		//Configuration de la tâche cron
		$scan_id = $row->scan_id;

		switch($interval_unit)
		{
			case 'hour' :
				$interval_seconds = $interval_value * 3600;
				break;
			case 'day' :
				$interval_seconds = $interval_value * 86400;
				break;
			case 'week' :
				$interval_seconds = $interval_value * 86400;
				break;
			case 'month' :
				$startdate = new DateTime();
				$enddate = new DateTime();
				$startdate->sub(new DateInterval('P'.$interval_value.'M'));
				$startdate = $startdate->format('U');
				$enddate = $enddate->format('U');
				$interval_seconds = $enddate - $startdate;
				break;
		}

		$cron_id = Model_esia::addCronTask($dbconnection,$cron_plugin['id'],"-m start -i $scan_id",$interval_seconds,$dt->format('U'),"start_scan_$scan_id_".'$DATE.log',$retries,$retries_interval);

		$params = array();
		$sql = "UPDATE vulnerability_scanner
				SET scan_cron_exec = :cron_id
				WHERE scan_id = :scan_id";


		$params[':cron_id'] = $cron_id;
		$params[':scan_id'] = $scan_id;
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);
		return $scan_id;
	}

	public static function startScan($dbconnection,$scan_id)
	{
		$cron_plugin = Model_esia::getCronPlugin($dbconnection,Model_egidescan::PLUGIN_CRON_START);
		$cron_plugin = array_shift($cron_plugin);
		Model_esia::addCronTask($dbconnection,$cron_plugin['id'],"-m start -i $scan_id",NULL,NULL,"man_start_scan_$scan_id_".'$DATE.log',0,0);
	}

	public static function removeIface($dbconnection,$iface_id)
	{
		$params[':iface_id'] = $iface_id;
		$sql = '
			DELETE FROM vulnerability_iface WHERE vul_iface_id = :iface_id
		';
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);
	}

	public static function removeScan($dbconnection,$scan_id)
	{
		//TODO : Remplacer la recherche par paramètre par une recherche en ajoutant une clé étrangere dans egide_scan
		$cron_plugin = Model_esia::getCronPlugin($dbconnection,Model_egidescan::PLUGIN_CRON_START);
		$cron_plugin = array_shift($cron_plugin);
		$cron_list = Model_esia::getCronActiveTasks($dbconnection,NULL,$cron_plugin['id']);

		$found_cron_task = NULL;
		foreach($cron_list as $cron)
		{
			if( $cron['parameter'] == "-m start -i $scan_id")
			{
				$found_cron_task = $cron;
			}
		}

		if( $found_cron_task != NULL )
		{
			Model_esia::deleteCronTask($dbconnection,$found_cron_task['id']);
		}

		$params =  array();
		$params[':scan_id'] = $scan_id;
		$sql_history = 'DELETE FROM vulnerability_scanner_history WHERE scan_id = :scan_id';
		$sql_scanner = 'DELETE FROM vulnerability_scanner WHERE scan_id = :scan_id';

		$query = $dbconnection->prepare($sql_history);
		$query = Model_esia::executeStatement($query,$params);

		$query = $dbconnection->prepare($sql_scanner);
		$query = Model_esia::executeStatement($query,$params);
	}

	public static function updateScan($dbconnection,$scan_id,$start_date,$start_time,$interval_value,$interval_unit,$iface_id,$list_scan,$retries,$retries_interval)
	{
		$params =  array();
		$params[':scan_id'] = $scan_id;
		$sql = "
			UPDATE vulnerability_scanner
			SET scan_iface_id=:iface,
				scan_option=:options
			WHERE scan_id = :scan_id
		";

		//Calcul de date
		$dt = DateTime::createFromFormat('d/m/Y H:i:s',"$start_date 00:00:00");
		$dt->add(new DateInterval("PT$start_time".'S'));

		//Création des données pour le scan_option json
		$options['group_list'] = $list_scan;
		$options['interval_value'] = $interval_value;
		$options['interval_unit'] = $interval_unit;

		//Configuration des paramètres
		$params[':iface'] = $iface_id;
		$params[':options'] = json_encode($options);

		//TODO : Remplacer la recherche par paramètre par une recherche en ajoutant une clé étrangere dans egide_scan
		$cron_plugin = Model_esia::getCronPlugin($dbconnection,Model_egidescan::PLUGIN_CRON_START);
		$cron_plugin = array_shift($cron_plugin);
		$cron_list = Model_esia::getCronActiveTasks($dbconnection,NULL,$cron_plugin['id']);

		$found_cron_task = NULL;
		foreach($cron_list as $cron)
		{
			if( $cron['parameter'] == "-m start -i $scan_id")
			{
				$found_cron_task = $cron;
			}
		}

		if( $found_cron_task != NULL )
		{
			switch($interval_unit)
			{
				case 'hour' :
					$interval_seconds = $interval_value * 3600;
					break;
				case 'day' :
					$interval_seconds = $interval_value * 86400;
					break;
				case 'week' :
					$interval_seconds = $interval_value * 86400;
					break;
				case 'month' :
					$startdate = new DateTime();
					$enddate = new DateTime();
					$startdate->sub(new DateInterval('P'.$interval_value.'M'));
					$startdate = $startdate->format('U');
					$enddate = $enddate->format('U');
					$interval_seconds = $enddate - $startdate;
					break;
			}
			Model_esia::updateCronTask($dbconnection,$found_cron_task['id'],"-m start -i $scan_id",$interval_seconds,$dt->format('U'),"start_scan_$scan_id_".'$DATE.log',$retries,$retries_interval);
		}

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);
		return $query->rowCount();
	}

	public static function getScanList($dbconnection,$id=NULL,$agent_id=NULL)
	{
		$sql = "SELECT
			scan_id,
			cron_last_exec,
			cron_next_exec,
			scan_state,
			scan_option,
			vul_iface_id,
			vul_iface_local_name,
			vul_iface_label,
			device_id,
			device_name,
			device_ip_address,
			vul_type_name,
			vul_type_color,
			vul_type_factor
			FROM vulnerability_scanner
			INNER JOIN vulnerability_iface ON(scan_iface_id = vul_iface_id)
			INNER JOIN esia_device ON(vul_iface_agent_id = device_id)
			INNER JOIN vulnerability_type ON(vul_type_id = vul_iface_type_id)
			INNER JOIN esia_cron_plugin ON (scan_cron_exec = cron_id)
			";

		if( $id != NULL )
		{
			$sql.= " WHERE scan_id = :scan_id";
			$params[':scan_id'] = $id;
		}
		elseif( $agent_id != NULL )
		{
			$in_request = generateInParams('WHERE',$agent_id,'vul_iface_agent_id',TRUE);
			$params = $in_request['param'];
			$where = $in_request['sql'];

			$sql = "$sql $where";
		}

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);

		$ret = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$retRow = array(
				'id' => $row->scan_id,
				'state' => $row->scan_state,
				'last_exec' => $row->cron_last_exec,
				'next_exec' => $row->cron_next_exec,
				'options' => json_decode($row->scan_option,TRUE),
				'iface_id' => $row->vul_iface_id,
				'iface' => $row->vul_iface_local_name,
				'label' => $row->vul_iface_label,
				'agent_id'=> $row->device_id,
				'agent_name'=> $row->device_name,
				'agent_addr'=> $row->device_ip_address,
				'type_name' => $row->vul_type_name,
				'type_color' => $row->vul_type_color,
				'type_factor' => $row->vul_type_factor,
			);

			if( empty($retRow['state']))
			{
				$retRow['state'] = self::NEW_TASK;
			}
			$ret[] = $retRow;
		}
		return $ret;
	}

	public static function getScanById($dbconnection,$id)
	{
		$list = self::getScanList($dbconnection,$id);
		return array_shift($list);
	}

	public static function getAgentScanList($dbconnection,$agent)
	{
		return self::getScanList($dbconnection,NULL,$agent);
	}

	public static function getNodesOfGroupList($dbconnection,$group_list)
	{
		$all_node_list = array();
		foreach($group_list as $group_id)
		{
			$nodes_group = Model_esia::newgetNodeList($dbconnection,NULL,$group_id,Model_esia::NODE_INSTANCE);
			//On ne souhaite qu'un seul exemplaire à la fois du même noeud
			foreach($nodes_group as $node)
			{
				$all_node_list[$node->getID()] = $node;
			}
		}
		return array_values($all_node_list);
	}

	public static function getCurrentScanList($dbconnection,$status=NULL)
	{
		$sql = "SELECT scan_id,scan_state,scan_option,vul_iface_local_name,device_id,device_ip_address";
		$sql.= " FROM vulnerability_scanner INNER JOIN vulnerability_iface";
		$sql.= " ON(scan_iface_id = vul_iface_id)";
		$sql.= " INNER JOIN esia_device";
		$sql.= " ON(vul_iface_agent_id = device_id)";
		$sql.= " INNER JOIN esia_cron_plugin ON (scan_cron_exec = cron_id)";
		$params = array();

		if( $status != null )
		{
			if( $status == self::PENDING_TASK )
			{
				$sql.= ' AND (scan_state = :status OR scan_state IS NULL)';
			}
			else
			{
				$sql.= ' AND scan_state = :status';
			}
			$params[':status'] = $status;
		}
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);

		$arr = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$arr[] = array(
				'id' => $row->scan_id,
				'state' => $row->scan_state,
				'options' => json_decode($row->scan_option,TRUE),
				'iface' => $row->vul_iface_local_name,
				'agent_id'=> $row->device_id,
				'agent_addr'=> $row->device_ip_address
			);
		}
		return $arr;
	}

	public static function updateScanningTaskStatus($dbconnection,$scan_id,$status)
	{
		$sql = "UPDATE vulnerability_scanner SET scan_state=:state";

		$params[':id'] = $scan_id;
		$params[':state'] = $status;

		$sql.= " WHERE scan_id=:id";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);
	}

// 	public static function newScan($dbconnection, $id_lastScan=NULL, $scan_date=NULL,$dv_scanned=NULL,$interval_exec=NULL,$interface=NULL)
// 	{
// 		$sql = "SELECT vul_iface_id FROM vulnerability_iface WHERE vul_iface_agent_id = :dv_scanned AND $interface = :interface";
// 		$query = $dbconnection->prepare($sql);
// 		$query = Model_esia::executeStatement($query, array(':dv_scanned'=>$dv_scanned,':interface'=>$interface));
// 		$row = $query->fetch(PDO::FETCH_ASSOC);

// 		$sql = "INSERT INTO public.vulnerability_scanner(scan_id, scan_next_execution, scan_agent_id,scan_interval_execution,scan_src_interface)
//                                         VALUES (:scan_id, :scan_date, :interval_exec, :scan_iface_id);";
// 		$params = array(
// 			':scan_id' =>$id_lastScan,
// 			':scan_date' =>$scan_date,
// 			':interval_exec' =>$interval_exec,
// 			':scan_iface_id' =>$row['vul_iface_id']
// 		);

// 		$query = $dbconnection->prepare($sql);
// 		$query = Model_esia::executeStatement($query, $params);
// 	}

	public static function getConfigScan($dbconnection,$id_lastScan=NULL)
	{
		$sql = "SELECT cron_next_exec,vul_iface_agent_id";
		$sql.= "\nFROM vulnerability_scanner INNER JOIN vulnerability_iface";
		$sql.= "\nON(scan_iface_id = vul_iface_type_id) INNER JOIN esia_cron_plugin ON (scan_cron_exec = cron_id)";
		$sql.= "\nWHERE scan_id= :id_lastScan;";

		$ret = array('NONE','NONE');
		if( $id_lastScan != NULL )
		{
			$params = array(
				':id_lastScan' => $id_lastScan
			);

			$query = $dbconnection->prepare($sql);
			$query = Model_esia::executeStatement($query, $params);
			$row = $query->fetch(PDO::FETCH_ASSOC);

			if(!empty($row['cron_next_exec']))
			{
				$ret[0] = $row['cron_next_exec'];
			}
			if(!empty($row['cron_next_exec']))
			{
				$ret[1] = $row['vul_iface_agent_id'];
			}
		}
		return $ret;
	}

	public static function getInterfaceScan($dbconnection,$id_lastScan=NULL)
	{
		$sql = "SELECT vul_iface_local_name";
		$sql.= "\nFROM vulnerability_scanner INNER JOIN vulnerability_iface";
		$sql.= "\nON(scan_iface_id = vul_iface_type_id)";
		$sql.= "\nWHERE scan_id= :id_lastScan;";

		$ret = 'NONE';
		if( $id_lastScan != NULL )
		{
			$params = array(
				':id_lastScan' => $id_lastScan
			);

			$query = $dbconnection->prepare($sql);
			$query = Model_esia::executeStatement($query, $params);
			$row = $query->fetch(PDO::FETCH_ASSOC);

			if(!empty($row['vul_iface_local_name']))
			{
				$ret = $row['vul_iface_local_name'];
			}
		}
		return $ret;
	}

	public static function getNumberGlobalByScoring($dbconnection,$user, $node=null)
	{
		$params = array();
		$sql = "SELECT FLOOR(vul_scoring) as domain,count(*) as number
                FROM egide_vulnerability
				JOIN egide_vulnerability_detect  ON vul_id = detect_vul_id
				JOIN egide_services ON eg_service_id=detect_eg_service_id
				JOIN egide_vulnerability_detect_state ON state_id=detect_state_id
                WHERE vul_scoring IS NOT NULL
                	AND eg_device_id IN ( SELECT * FROM getUserAccessNode(:user))
                	AND state_name = 'ACTIVE'";
        $params[':user'] = $user;

		$in_request = generateInParams('AND',$node,'eg_device_id');
		$param = $in_request['param'];
		$where = $in_request['sql'];

		$sql.= ' ' . $where;
		$params = array_merge($params,$param);

        $sql .=" GROUP BY FLOOR(vul_scoring)";
        $query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);
		$arr = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$arr[] = array('domain' => $row->domain,
				'number' => $row->number);
		}
		return $arr;
	}

	public static function getServicesByIDList($dbconnection,$user,$cve_list=NULL,$include_cve=TRUE)
	{
		$in_request = generateInParams('AND',$cve_list,'eg_service_id',$include_cve);
		$params = $in_request['param'];
		$where = $in_request['sql'];

		$sql = "SELECT eg_service_id, eg_service_name, eg_service_product, eg_service_version, eg_service_port, eg_service_cpe,
					eg_service_extrainfo,eg_service_port_type, device_ip_address, device_name, device_type,
					count(detect_vul_id) as matchnum,AVG(vul_scoring) AS avg_scoring,MAX(vul_scoring) AS max_scoring
				FROM public.egide_services
				JOIN egide_vulnerability_detect ON eg_service_id = detect_eg_service_id
				JOIN esia_device ON device_id = eg_device_id
				JOIN egide_vulnerability ON vul_id = detect_vul_id
				JOIN egide_vulnerability_detect_state ON state_id=detect_state_id
				WHERE eg_device_id IN ( SELECT * FROM getUserAccessNode(:user))
					AND state_name = 'ACTIVE' $where
				GROUP BY eg_service_id, eg_service_name, eg_service_product, eg_service_version, eg_service_port, eg_service_cpe,
					eg_service_extrainfo,eg_service_port_type, device_ip_address, device_name,device_type";

		$params[':user'] = $user;

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		$arr = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$arr[] = array(
							'id' => $row->eg_service_id,
							'name' => $row->device_name . ' '.$row->eg_service_product . ' port '.$row->eg_service_name . '('.$row->eg_service_port .')',
							'number' => $row->matchnum
							);
		}
		return $arr;
	}


	public static function getAcqCVE($dbconnection,$user,$cve_list=NULL,$include_cve=TRUE)
	{
		$sql = "
			SELECT vul_list_id, vul_description,vul_scoring, vul_severity, vul_attack_complexity, device_name,eg_service_name,detect_id
			FROM egide_vulnerability
			JOIN egide_vulnerability_detect  ON vul_id = detect_vul_id
			JOIN egide_vulnerability_detect_state ON state_id=detect_state_id
			JOIN egide_services ON eg_service_id=detect_eg_service_id
			JOIN esia_device ON eg_device_id=device_id
			WHERE eg_device_id IN ( SELECT * FROM getUserAccessNode(:user)) AND state_name = 'ACQUIT'
		";
		$params[':user'] = $user;

		$in_request = generateInParams('AND',$cve_list,'detect_id',$include_cve);
		$param = $in_request['param'];
		$where = $in_request['sql'];

		$sql.= ' ' . $where;
		$params = array_merge($params,$param);

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		$arr = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$arr[] = array(
							'id' => $row->detect_id,
							'id_vul' => $row->vul_list_id,
                            'description' => $row->vul_description,
                            'scoring' => $row->vul_scoring,
                            'severity' => $row->vul_severity,
                            'complexity' => $row->vul_attack_complexity,
							'device_name' => $row->device_name,
							'service_name' => $row->eg_service_name
                            );
        }
        return $arr;
	}


	public static function getCVEByNode($dbconnection,$user,$node_id,$ifaceList=NULL)
	{
		$params = array();
		$params[':user'] = $user;
		$params[':nodeid'] = $node_id;

		$in_request = generateInParams('AND',$ifaceList,'eg_access_iface_id');
		$param = $in_request['param'];
		$where = $in_request['sql'];
		$params = array_merge($params,$param);

		$sql = "
			SELECT DISTINCT ON (vul_list_id) eg_device_id,eg_service_name,vul_list_id,vul_iface_label,vul_description,vul_severity,vul_attack_complexity,vul_scoring,vul_type_color,
			(vul_scoring*vul_type_factor) AS calculated_score
			FROM egide_vulnerability
			JOIN egide_vulnerability_detect  ON vul_id = detect_vul_id
			JOIN egide_services ON eg_service_id=detect_eg_service_id
			JOIN egide_service_access ON(eg_service_id=eg_access_service_id)
			JOIN vulnerability_iface ON(eg_access_iface_id = vul_iface_id)
			JOIN vulnerability_type ON(vul_iface_type_id = vul_type_id)
			JOIN egide_vulnerability_detect_state ON state_id=detect_state_id
			WHERE state_name ='ACTIVE'
				AND eg_device_id = :nodeid AND eg_device_id IN ( SELECT * FROM getUserAccessNode(:user)) $where
			GROUP BY vul_list_id,eg_device_id,vul_iface_label,vul_type_factor,eg_service_name,vul_type_color,vul_scoring,vul_severity,vul_attack_complexity,vul_description
			ORDER BY vul_list_id,calculated_score DESC
		";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		$arr = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$arr[] = array('id' => $row->vul_list_id,
							'description' => $row->vul_description,
							'scoring' => $row->vul_scoring,
							'severity' => $row->vul_severity,
							'complexity' => $row->vul_attack_complexity,
							'iface_label' => $row->vul_iface_label,
							'iface_color' => $row->vul_type_color,
							'service_name'=> $row->eg_service_name,
							'calculated_score' => $row->calculated_score
							);
		}
		return $arr;
	}

    public static function getCVEByScoring($dbconnection,$user,$node=null,$limit=null,$cve_list=NULL,$include_cve=TRUE)
    {
		//Préparation de la requête
		$params = array();
		$sql = "
			SELECT vul_id, vul_list_id, vul_description,vul_scoring, vul_severity, vul_attack_complexity,count(detect_vul_id) as matchnum,
			    (CASE
					WHEN state_name = 'ACTIVE' THEN 1
					WHEN state_name = 'ACQUIT' THEN 2
					WHEN state_name = 'CLOSED' THEN 3
				END) AS priority_detect_state,
				state_name
			FROM egide_vulnerability
			JOIN egide_vulnerability_detect  ON vul_id = detect_vul_id
			JOIN egide_services ON eg_service_id=detect_eg_service_id
			JOIN egide_vulnerability_detect_state ON state_id=detect_state_id
			WHERE eg_device_id IN ( SELECT * FROM getUserAccessNode(:user))
		";
		$params[':user'] = $user;

		$in_request = generateInParams('AND',$node,'eg_device_id');
		$param = $in_request['param'];
		$where = $in_request['sql'];

		$sql.= ' ' . $where;
		$params = array_merge($params,$param);

		$in_request = generateInParams('AND',$cve_list,'vul_list_id',$include_cve);
		$param = $in_request['param'];
		$where = $in_request['sql'];

		$sql.= ' ' . $where;
		$params = array_merge($params,$param);

		$sql .= " GROUP BY vul_id,vul_list_id,vul_description, vul_scoring, vul_severity, vul_attack_complexity, state_name,vul_date_publishing
					ORDER BY priority_detect_state ASC,vul_scoring DESC,matchnum DESC,vul_date_publishing DESC";

		//Récupération des CVE en cours
		if(!empty($limit))
        {
            $sql .=" LIMIT :limit";
            $params[':limit'] = $limit;
        }

        $query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		$arr = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$arr[] = array('id' => $row->vul_list_id,
							'dbid'=>$row->vul_id,
                            'description' => $row->vul_description,
                            'scoring' => $row->vul_scoring,
                            'severity' => $row->vul_severity,
                            'complexity' => $row->vul_attack_complexity,
                            'number' => $row->matchnum,
							'state' => $row->state_name
                            );
        }
        return $arr;
    }
		public static function getNodesByScoring($dbconnection,$user, $node=null,$limit=null)
    	{
        	$params = array();
			if( !empty($node))
			{
				if( is_array($node) || is_numeric($node))
				{
					$in_request = generateInParams('AND',$node,'device_id');
					$param = $in_request['param'];
					$where = $in_request['sql'];
					$params = array_merge($params,$param);
				}
				else
				{
					$where = "AND device_name = :name";
					$params[':name'] = $node;
				}
			}

				$sql = "
					(
						With active_vul AS (
							SELECT device_id,device_name,device_description,device_type,device_ip_address,
								COUNT(detect_vul_id) as number,
								MAX(vul_scoring) as maximun,
								SUM(vul_scoring) as total
							FROM esia_device
							JOIN egide_services ON device_id=eg_device_id
							JOIN egide_vulnerability_detect ON eg_service_id = detect_eg_service_id
							JOIN egide_vulnerability ON vul_id = detect_vul_id
							JOIN egide_vulnerability_detect_state ON state_id=detect_state_id
							WHERE device_id IN ( SELECT * FROM getUserAccessNode(:user))
								AND state_name = 'ACTIVE' $where
							GROUP BY device_id,device_name,device_description,device_type,device_ip_address
						)
						SELECT device_id,device_name,device_description,device_type,device_ip_address,number,maximun,total
						FROM active_vul
						UNION
						SELECT device_id,device_name,device_description,device_type,device_ip_address,0 as number,0 as maximun,0 as total
						FROM esia_device
						WHERE device_id IN ( SELECT * FROM getUserAccessNode(:user)) AND device_id NOT IN(SELECT device_id from active_vul) $where
					)
					ORDER BY maximun DESC,total DESC, number DESC, device_name ASC
				";
				$params[':user'] = $user;

        if(!empty($limit))
        {
            $sql .=" LIMIT :limit";
            $params[':limit'] = $limit;
        }
        $query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);
		$arr = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$average_score = 0;
			if( $row->number > 0 )
			{
				$average_score = round($row->total/$row->number,1);
			}

			$arr[] = array('id' => $row->device_id,
                            'name' => $row->device_name,
                            'description' => $row->device_description,
                            'type' => $row->device_type,
                            'ip' => $row->device_ip_address,
                            'numberVulnerability' => $row->number,
                            'totalVulnerabilityScoring' => $row->total,
                            'averageVulnerabilityScoring' => $average_score,
                            'maxVulnerabilityScoring' => $row->maximun);
        }
        return $arr;
    }

	/******************************** SECTION POUR LE SCANNER INTO CLASS ***************************/
	public static function getListOfWinKB($dbconnection)
	{
		    $sql = "
		            SELECT built_device_id,component_name,component_json
		            FROM inventory_component
		            JOIN inventory_type ON(component_type_id = type_id)
		            JOIN inventory_is_built ON(built_hardware_id = component_id)
		            WHERE type_name = 'KB'
		            AND component_json IS NOT NULL
		            AND component_json->'CVE' IS NOT NULL
		            ORDER BY component_id ASC
		    ";

		    $stmt = $dbconnection->prepare($sql);
		    $stmt->execute();
		    $ret = array();
		    while( $row = $stmt->fetch(PDO::FETCH_ASSOC) )
		    {
		            $row['component_json'] = json_decode($row['component_json'],TRUE);
		            if( !empty($row['component_json']['CVE']))
		            {
		                    foreach($row['component_json']['CVE'] as $cve )
		                    {
		                            $ret[$row['built_device_id']][$cve] = $row['component_name'];
		                    }
		            }
		    }
		    return $ret;
	}

	public static function isWebpInventoryEnabled($dbconnection)
	{
		$table_exist = "SELECT EXISTS (SELECT 1 FROM information_schema.tables WHERE table_name = 'inventory_is_built')";
		$stmt = $dbconnection->prepare($table_exist);
		$stmt->execute();
		$table_exists = $stmt->fetch(PDO::FETCH_COLUMN);
		if( !$table_exist )
		{
			return FALSE;
		}
		return TRUE;
	}

	/**
	*** Get basic informations about vulnerability
	*** @param PDO $dbconnection un objet PDO
	*** @param integer (optional) $vulId vulnerability DB ID (optional if $vulListId is given)
	*** @param string (optional) $vulListId vulnerability name (vul_list_id) (optional if $vulId is given)
	*** @throws EsiaDbException
	**/
	public static function getVulInfosById($dbconnection, $vulId=NULL, $vulListId=NULL)
	{
		$sql = "SELECT vul_id, vul_list_id, vul_description, vul_url, vul_date_publishing, vul_scoring, vul_severity, vul_attack_complexity
				FROM egide_vulnerability";

		if(!empty($vulId) && is_numeric($vulId))
		{
			$sql .= " WHERE vul_id = :vul_id";

			$params = array(':vul_id' => $vulId);
		}
		elseif(!empty($vulListId))
		{
			$sql .= " WHERE vul_list_id = :vul_list_id";

			$params = array(':vul_list_id' => $vulListId);
		}
		else
		{
			throw new EsiaDbException('Missing vulnerability identifier (DB ID or name)');
		}

        $query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		if($query->rowCount() <= 0)
		{
			throw new EsiaDbException('Vulnerability not found');
		}

		while($row = $query->fetch(PDO::FETCH_OBJ))
        {
			$vul = array(
				'id' => $row->vul_id,
				'id_cve' => $row->vul_list_id,
				'desc' => $row->vul_description,
				'url' => json_decode($row->vul_url, TRUE),
				'date_pub' => $row->vul_date_publishing,
				'scoring' => $row->vul_scoring,
				'severity' => $row->vul_severity,
				'complexity' => $row->vul_attack_complexity,
			);
        }

		return $vul;
	}

	/**
	*** Get data for a vulnerability found by vulnerability DB ID. (vulnerability, node, egide box) box interface)
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $userId l'id de l'utilisateur (pour l'acccess aux noeuds)
	*** @param integer $vulId vulnerability DB ID
	*** @throws EsiaDbException
	**/
	public static function getVulDetectByVulId($dbconnection, $userId, $vulId)
	{
		$sql = "SELECT node.device_id, node.device_name, node.device_ip_address, device_group_path,
					eg_service_name, eg_service_product, eg_service_version, eg_service_port, eg_service_port_type,
					vul_iface_local_name, vul_iface_label,
					vul_type_name, vul_type_factor, vul_type_color,
					box.device_id AS box_id, box.device_name AS box_name, box.device_ip_address AS box_ip
				FROM esia_device AS node
				JOIN egide_services ON device_id = eg_device_id
				JOIN egide_vulnerability_detect ON detect_eg_service_id = eg_service_id
				JOIN egide_vulnerability_detect_state ON state_id=detect_state_id
				JOIN egide_service_access ON eg_access_service_id = eg_service_id
				JOIN vulnerability_iface ON eg_access_iface_id = vul_iface_id
				JOIN vulnerability_type ON vul_type_id = vul_iface_type_id
				JOIN esia_device AS box ON vul_iface_agent_id = box.device_id
				LEFT JOIN esia_device_group_in ON group_in_device_id = node.device_id
				LEFT JOIN esia_device_group ON device_group_id = group_in_device_group_id
				WHERE node.device_id IN (SELECT * FROM getUserAccessNode(:user_id))
				AND node.device_delete = FALSE
				AND state_name = 'ACTIVE'
				AND detect_vul_id = :vul_id
				ORDER BY node.device_name;";

		$params = array(
			':user_id' => $userId,
			':vul_id' => $vulId,
		);

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		$vulList = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
        {
			$vulList[] = array(
				'device_id' => $row->device_id,
				'device_name' => $row->device_name,
				'device_ip' => $row->device_ip_address,
				'device_group' => $row->device_group_path,
				'eg_srv_name' => $row->eg_service_name,
				'eg_srv_product' => $row->eg_service_product,
				'eg_srv_version' => $row->eg_service_version,
				'eg_srv_port' => $row->eg_service_port,
				'eg_srv_port_type' => $row->eg_service_port_type,
				'if_local_name' => $row->vul_iface_local_name,
				'if_label' => $row->vul_iface_label,
				'if_type_name' => $row->vul_type_name,
				'if_type_factor' => $row->vul_type_factor,
				'if_type_color' => $row->vul_type_color,
				'box_id' => $row->box_id,
				'box_name' => $row->box_name,
				'box_ip' => $row->box_ip,
			);
        }

		return $vulList;
    }
}
/******************************** SECTION POUR LE SCANNER ***************************/

/**
	@brief Analyse un tableau host d'un scan nmap et retourne la liste des CPEs permettant d'identifier ce device
	@param array $host: Un array provenant d'un scan nmap (fonction parseXmlScan(...)) et contenant le tableau suivant
		$host['osmatch'][][list][]['cpe'][] = $cpe
	@todo : Améliorer l'identification des CPEs ?
				- Ne retourner que le numéro de version le plus élevé ?
				- Tenter de mieux identifier le device avec l'aide des CPEs des services ? (voir script d'essai/test)
				- Utiliser/SNMP/Powershell/Module inventaire pour les Microsoft Windows ?
	@retval array<string>	La liste des CPEs conçernant un device.
*/
function getCpeListFromOs($host)
{
    $ret = array();
    foreach($host['osmatch'] as $osmatch)
    {
        $retRow = array();
        foreach($osmatch['list'] as $osclass)
        {
                $retRow = array_merge($retRow,$osclass['cpe']);
        }
        $ret = array_merge($retRow,$ret);
    }
    return $ret;
}
/**
@brief Analyse un tableau host d'un scan nmap et retourne la liste des CPEs permettant d'identifier les services
@param array $host: Un array provenant d'un scan nmap (fonction parseXmlScan(...)) et contenant le tableau suivant
	$host['ports'][] = array(
		'state' => 'open',
		'portid' => $port, //(80,443,22,...)
		'service_cpe' => array<string> $service_cpe
 	)
@todo : Améliorer l'identification des CPEs ?
			- Ne retourner que le numéro de version le plus élevé ?
			- Tenter de mieux identifier le device avec l'aide des CPEs des services ? (voir script d'essai/test)
			- Utiliser/SNMP/Powershell/Module inventaire pour les Microsoft Windows ?
@retval array[<portid>]<array>	La liste des CPEs conçernant un service. La valeur à forme array('portid'=><integer>,'cpe'=>array<string>)
*/
function getCpeListFromServices($host)
{
    $ret = array();
    foreach($host['ports'] as $port)
    {
        if( $port['state'] == 'open' )
        {
            $ret[$port['portid']] = array(
                'portid' => $port['portid'],
                'cpe' => $port['service_cpe']
            );
        }
    }
    return $ret;
}

/**
    @brief Récupére en base de donnée une liste de CVEs potentiels conçernant les CPEs renseignés
	@param PDO $dbconnection La base de donnée pour récupérer les infos CVEs
	@param array<string> $cpe_list Un array contenant une liste de cpe sous forme de string
	@param <'<'|'<='|'='|'>='|'>'> Optionnel, utilie uniquemet si les CPEs contiennent des versions. Pour matcher sur les version +petites ou +grande
	@retval array La liste des CVEs conçernant les CPEs spécifiés dans le format suivant :
		$ret['cpe_vul_id'] = L'identifiant en base de donnée
		$ret['vul_list_id'] = Le numéro de la CVE (CVE-2023-14544)
		$ret['vul_cpe'] = Le json indiquant comment vérifier si la CVEs match bien des CPEs renseignés.
*/
function getCVECandidatesFromCPEList($dbconnection,$cpe_list,$operator='=')
{
    $product_type['h'] = 'hardware';
    $product_type['o'] = 'operating system';
    $product_type['a'] = 'application';

    $sql['without_version'] = "
        SELECT cpe_vul_id,vul_list_id,vul_cpe
        FROM egide_cpe_vulnerability
        JOIN egide_vulnerability ON (cpe_vul_id=vul_id)
        WHERE cpe_product_id = (SELECT product_id FROM egide_cpe_product WHERE product_name = :product AND product_type  = :type
            AND product_vendor_id = (SELECT vendor_id FROM egide_cpe_vendor WHERE vendor_name = :vendor)
        )
    ";

    //Equal version
    $sql['='] = $sql['without_version'];
    $sql['='].= "
        AND
        (
            cpe_equal_version = :version OR cpe_equal_version like :version2
            OR (
        	cpe_equal_version IN('*','-')
        	AND(
                    (cpe_start_version_exclude IS NULL AND cpe_start_version_include IS NULL)
                    OR (cpe_start_version_exclude IS NOT NULL AND compare_versions(cpe_start_version_exclude,:version) < 0)
                    OR (cpe_start_version_include IS NOT NULL AND compare_versions(cpe_start_version_include,:version) <= 0)
                )
                AND
                (
                    (cpe_end_version_include IS NULL AND cpe_end_version_exclude IS NULL)
                    OR (cpe_end_version_include IS NOT NULL AND compare_versions(cpe_end_version_include,:version) >= 0)
                    OR (cpe_end_version_exclude IS NOT NULL AND compare_versions(cpe_end_version_exclude,:version) > 0)
                )
            )
        )
    ";

    //Smaller version
    foreach(array('<','<=') as $op)
    {
        $sql[$op] = $sql['without_version'];
        $sql[$op].= "
            AND
            (
                ((LENGTH(cpe_equal_version) > 1  OR cpe_equal_version ~ '\d' ) AND compare_versions(cpe_equal_version,:version) $op 0)
                OR (
                        (cpe_start_version_exclude IS NOT NULL AND compare_versions(cpe_start_version_exclude,:version) < 0)
                        OR (cpe_start_version_include IS NOT NULL AND compare_versions(cpe_start_version_include,:version) <= 0)
                )
            )
        ";
    }

    //Bigger version
    foreach(array('>','>=') as $op)
    {
        $sql[$op] = $sql['without_version'];
        $sql[$op].= "
            AND
            (
                (cpe_equal_version like :version2 OR ( (LENGTH(cpe_equal_version) > 1  OR cpe_equal_version ~ '\d' ) AND compare_versions(cpe_equal_version,:version) $op 0))
                OR (
                        (cpe_end_version_include IS NOT NULL AND compare_versions(cpe_end_version_include,:version) >= 0)
                        OR (cpe_end_version_exclude IS NOT NULL AND compare_versions(cpe_end_version_exclude,:version) > 0)
                )
            )
        ";
    }

    $stmt_list['without_version'] = $dbconnection->prepare($sql['without_version']);
    $stmt_list['with_version'] = $dbconnection->prepare($sql[$operator]);

    $ret = array();
    foreach($cpe_list as $cpe)
    {
	$params = array();

        $parsed_cpe = parse_nmap_cpe($cpe);
        $params[':type'] = $product_type[$parsed_cpe['type']];
        $params[':vendor'] = $parsed_cpe['vendor'];
        $params[':product'] = $parsed_cpe['product'];

        if(empty($parsed_cpe['version']))
        {
            $stmt = $stmt_list['without_version'];
        }
        else
        {
            $stmt = $stmt_list['with_version'];
            $params[':version'] = $parsed_cpe['version'];
	    if( $operator == '=' )
	    {
		$params[':version2'] = $params[':version'].'.%';
	    }
            if( $operator == '>' )
            {
                $params[':version2'] = $params[':version'].'_%';
            }
            elseif( $operator == '>=' )
            {
                $params[':version2'] = $params[':version'].'%';
            }
        }
	$stmt->execute($params);
	while($row=$stmt->fetch(PDO::FETCH_ASSOC))
	{
		$ret[$row['vul_list_id']]=$row;
	}
    }
    return array_values($ret);
}

/**
    @brief Retourne la liste des CVEs identifiés pour un service donnée
	@param PDO $dbconnection La base de donnée pour récupérer les infos CVEs
	@param array $cpe_list: Un array contenant des listes de CPEs
		$cpe_list['device'] = array<string> => Tableau de string correspondant au CPE du device
		$cpe_list['services'][$portid] = array<string> => Tableau de string correspondant au CPE du service identifié par son port
		Une cpe ressemble à ceci : cpe:o:microsoft:windows_server_2022:-:*:*:*:*:*:*:*
	@param integer $portid Est le numéro du port conçerné
	@retval array<string>	La liste des CVEs conçernant un service d'un noeud donné.
	@details
	Le but de cette fonction est de retourné une liste de CVEs pour un service donné d'un noeud.
	Il fonctionne de la manière suivante :
	- Filtrage de la liste $cpe_list afin de récupérer les CPEs pour identifier le service a|application ou, si absent o|operating system
	- Filtrage de la liste $cpe_list afin de récupérer les CPEs pour identifier le device conçerné o|operating system si absent du service et/ou h|hardware
	- Avec la liste des CPEs de service, on récupére en base de donnée la liste des CVEs probablement conçernés.
	- On analyse la liste des CVEs recupérés afin d'éliminter les CVEs qui nous intéressent pas.
	- On retourne le numéro de chacun des CVES identifiés.

	Les fonctions autres employées sont les suivantes :
	- cpe_type_filter pour filtrer les CPEs par type (a|application, o|operating system, h|hardware)
	- getCVECandidatesFromCPEList pour récupérer un ensemble de CVEs pour les services matchant des CPEs et dont la version correspont (si la version est spécifiée via $cpe_list).
	- cve_list_filter est utilisé pour le filtre, on lui passe la liste des CVEs à filtrer, les CPEs pour le service et les CPEs pour les devices.

	- Les CPEs de type devices sont nécessaires car parfois une faille de sécurité sur un service ne concerne que certains devices.
*/

///OPTIM
function prepareCPEList($dbconnection,$hostlist)
{

}

function getCVEListForNode($dbconnection,$host,$port_list,$node_id,$inventory_available)
{
	$cpe_list = array();
	$cpe_list['device'] = getCpeListFromOs($host);
	$cpe_list['services'] = getCpeListFromServices($host);

	$device = identifyDevice($dbconnection,$cpe_list,$node_id,$inventory_available);

	$cve_list = array();
	foreach($port_list as $port_tab)
	{
		$port_id = $port_tab['portid'];
		//Identify service
		$service = $cpe_list['services'][$port_id];
		if( $inventory_available )
		{
			$service = identifyService($dbconnection,$service,$node_id);
		}
		$cve_list[$port_id] = getCVEListForService($dbconnection,$port_tab,$device,$service);
	}
	return $cve_list;
}

function getCVEListForService($dbconnection,$host,$port_id,$node_id,$inventory_available)
{
	$cpe_list = array();
	$cpe_list['device'] = getCpeListFromOs($host);
	$cpe_list['services'] = getCpeListFromServices($host);

    $service = $cpe_list['services'][$port_id];
	$device = identifyDevice($dbconnection,$cpe_list,$node_id,$inventory_available);
	$service = identifyService($dbconnection,$service,$node_id);

	$hw_cpe = cpe_type_filter($device,array('h'));
	$os_cpe = cpe_type_filter($device,array('o'));
	$ap_cpe = cpe_type_filter($service['cpe'],array('a'));

	$services_cpe = $ap_cpe;
	$devices_cpe = $hw_cpe;
	if( empty($services_cpe) && !empty($hw_cpe))
	{
		$services_cpe = $os_cpe;
	}
	else
	{
		$devices_cpe = array_merge($os_cpe,$hw_cpe);
	}
	if( empty($devices_cpe))
	{
		$devices_cpe = $os_cpe;
	}

	$cve_list = array();
    if( !empty($ap_cpe))
    {
            $tmp = $services_cpe;
            $op = '=';
            if( count($services_cpe) == 1 )
            {
                    $service_cpe = $services_cpe[0];
                    $parsed_service = parse_nmap_cpe($service_cpe);
                    if(empty( $parsed_service['version']))
                    {
                            $service_version = NULL;
                            foreach($host['ports'] as $service)
                            {
                                    if( $service['portid'] == $port_id )
                                    {
                                            $service_version = $service['service_version'];
                                            break;
                                    }
                            }
                            $list_versions = explode(' ',$service_version);
                            $services_cpe = array( $service_cpe.':'.$list_versions[0]);
                            if( !isset($list_versions[1] )) //1 seule version
                            {
                                    $op = '=';
                            }
                            else
                            {
                                    $op = '>=';
                            }
                    }
            }
            $cve_list = getCVECandidatesFromCPEList($dbconnection,$services_cpe,$op);
            $services_cpe = $tmp;
    }

	if( empty($ap_cpe) || empty($cve_list))
	{
		foreach($host['ports'] as $port)
		{
				if( $port['portid'] == $port_id )
				{
					if( !empty($devices_cpe) )
					{
						$vendor_list = array();
						foreach($devices_cpe as $cpe)
						{
							$parsed_device = parse_nmap_cpe($cpe);
							$vendor_list[$parsed_device['vendor']] = $parsed_device['vendor'];
						}
						foreach($vendor_list as $vendor)
						{
							$retRow = getAppCVEsByProductInfo($dbconnection,$port,$vendor);
							$cve_list = array_merge($cve_list,$retRow);
							$ap_cpe = array();
							$services_cpe = $os_cpe;
						}
					}
					if(  empty($cve_list) || empty($devices_cpe) )
					{
						$retRow = getAppCVEsByProductInfo($dbconnection,$port,NULL);
						$cve_list = array_merge($cve_list,$retRow);
						$ap_cpe = array();
						$services_cpe = $os_cpe;
					}
					break;
				}
		}
	}
	$real_cve = array();
	foreach($cve_list as $cve)
	{
		$real_cve[$cve['vul_list_id']] = $cve;
	}
	$cve_list = $real_cve;
	if( !empty($services_cpe))
	{
		$cve_list = cve_list_filter($cve_list,$services_cpe,$devices_cpe);
	}
	$ret = array();
	foreach($cve_list as $row)
	{
		$ret[] = $row['vul_list_id'];
	}
    return $ret;
}

function identifyService($dbconnection,$service,$node_id)
{
	$cpe_list = $service['cpe'];
	$ret = array();
	foreach($cpe_list as $cpe)
	{
		$retRow = $cpe;
		$parse = parse_nmap_cpe($cpe);
		if( empty($parse['version']) )
		{
			$version = getServiceVersionFromInventory($dbconnection,$parse['product'],$node_id);
			if( !empty($version))
			{
				$retRow = "$cpe:$version";
			}
		}
		$ret[] = $retRow;
	}
	$service['cpe'] = $ret;
	return $service;
}

function getServiceVersionFromInventory($dbconnection,$name,$node_id)
{
	$table_exist = "SELECT EXISTS (SELECT 1 FROM information_schema.tables WHERE table_name = 'inventory_is_built')";
	$stmt = $dbconnection->prepare($table_exist);
	$stmt->execute();
	$table_exists = $stmt->fetch(PDO::FETCH_COLUMN);
	if( !$table_exist )
	{
		return NULL;
	}

	$params[':node_id'] = $node_id;
	$params[':service'] = "%$node_id%";
	$sql = "
		SELECT manufacturer_name,built_version,count(*)
	    FROM inventory_is_built
	    JOIN inventory_component ON(built_hardware_id = component_id)
	    JOIN inventory_manufacturer ON (component_manufacturer_id = manufacturer_id)
	    WHERE component_type_id IN (SELECT type_id FROM inventory_type WHERE type_type = 'SOFTWARE')
	    AND built_device_id = :node_id AND built_name ilike :service
	    GROUP BY manufacturer_name,built_version ORDER BY count(*) DESC LIMIT 1;
	";
	$stmt = $dbconnection->prepare($sql);
	$stmt->execute($params);
	while($row=$stmt->fetch(PDO::FETCH_ASSOC))
	{
		return $row['built_version'];
	}
	return NULL;
}

function getOSInfoFromInventory($dbconnection,$node_id,$inventory_available)
{
	if( $inventory_available == FALSE)
	{
		return NULL;
	}
	$params[':node_id'] = $node_id;
	$sql = "
		SELECT manufacturer_name,built_name,built_version,built_json
		FROM inventory_is_built
		JOIN inventory_component ON(built_hardware_id = component_id)
		JOIN inventory_manufacturer ON (component_manufacturer_id = manufacturer_id)
		WHERE component_type_id = (SELECT type_id FROM inventory_type WHERE type_name = 'OS')
		AND built_device_id = :node_id
		ORDER BY LENGTH(built_version) DESC,LENGTH(built_name) DESC;
	";
	$stmt = $dbconnection->prepare($sql);
	$stmt->execute($params);
	while($row=$stmt->fetch(PDO::FETCH_ASSOC))
	{
		$ret = array();
		$ret['vendor'] = $row['manufacturer_name'];
		$ret['name'] = $row['built_name'];
		$ret['version'] = $row['built_version'];
		$decode = json_decode($row['built_json'],TRUE);
		if( is_array($decode))
		{
			$ret['data'] = $decode;
		}
		return $ret;
	}
	return NULL;
}

function identifyDevice($dbconnection,$cpe_list,$node_id,$inventory_available)
{
	$fromInventoryOs = getOSInfoFromInventory($dbconnection,$node_id,$inventory_available);
	if( isset($fromInventoryOs['vendor']) && stripos($fromInventoryOs['vendor'],'Microsoft') !== FALSE)
	{
		$product_identity = array();
		$type = 'workstation';

		//Build dans le 3ieme chiffre Ex: 10.0.19041
		$exp = explode('.',$fromInventoryOs['version']);
		$build = $exp[2];
		$product_num = $exp[0];

		if( empty($fromInventoryOs['data']['version_os']) )
		{
			if( $build >= 9600) //9600 => 6.3
			{
				$product_identity = microsoft_os_identitify($build,6.3);
			}
		}
		if( !empty($fromInventoryOs['data']['type']) && strcasecmp($fromInventoryOs['data']['type'],'server'))
		{
			$type == 'server';
		}
		if( $type != 'server'  && stripos($fromInventoryOs['name'],'server'))
		{
			$type = 'server';
		}

		$product_identity['type'] = $type;
		$product_identity['build'] = $build;
		$product_identity['product_num'] = $product_num;

		if( !empty($fromInventoryOs['data']['version_os']) )
		{
			$product_identity['product_version'] = $fromInventoryOs['data']['version_os'];
		}

		if( $product_identity['type'] == 'server' )
		{
			if( preg_match('/\d{4}/',$fromInventoryOs['data']['name'],$matches))
			{
				$product_identity['server_num'] = $matches[0];
			}
		}
	//	print_r($product_identity);
		$microsoft_cve = get_microsoft_cpe_from_product_identity($product_identity,$fromInventoryOs['version']);
		return $microsoft_cve;
	}

	$product = identify_product_by_services($cpe_list['services']);
	if( empty($product))
	{
		$product = identify_product_by_devices($cpe_list['device']);
	}
	if( empty($product))
	{
		$product = identify_partial_product_by_devices($cpe_list['device']);
	}
	$ret = array();
	foreach($product as $row)
	{
		$retRow = explode(':',$row,2)[1];
		if( $retRow == 'o:microsoft:windows' )
		{
			$ret[] = $row;
		}
		else
		{
			//On prend recherche la version la plus elevée
			$all_cpes = $cpe_list['device'];
			foreach($cpe_list['services'] as $service)
			{
				$all_cpes = array_merge($all_cpes,$service['cpe']);
			}

			$parsed_identified = parse_nmap_cpe($retRow);
			$identified_version = explode(':',$parsed_identified['version'])[0];//On ne prends que le premier chiffre
			$identified_version = trim($identified_version,'-*');
			foreach($all_cpes as $cpe)
			{
				$parsed_current = parse_nmap_cpe($cpe);
				if( is_same_cpe($parsed_identified,$parsed_current) )
				{
					$current_version = explode(':',$parsed_current['version'])[0];
					$current_version = trim($current_version,'-*');
					if(!empty($current_version))
					{
						if( empty($identified_version))
						{
							$identified_version = $current_version;
						}
						else
						{
							if(version_compare($identified_version, $current_version,"<"))
							{
								$identified_version = $current_version;
							}
						}
					}
				}
			}
			if( empty($identified_version) )
			{
				$identified_version = '';
			}
			$parsed_identified['version'] = $identified_version;

			if(
				$parsed_identified['vendor'] == 'cisco'
				&& $parsed_identified['product'] == 'ios'
				&& !empty( $parsed_identified['version'])
			);
			{
				//Pour cisco, on ne prends que le premier chiffre de numversion
				$exp = explode('.',$parsed_identified['version']);
				$parsed_identified['version'] = $exp[0];
			}


			$ret[] = implode(':',$parsed_identified);
		}
	}
	return $ret;
}

/***************Fonction utilisées directement par getCVECandidatesFromCPEList et/ou getCVEListForService ****************/

/**
    @brief Transforme un string de cpe en array pour simplifier sa comparaison
	@param string $cpe Une cpe au format nmap ou au format proventant de la base de donnée
		- nmap : 'cpe:\o:microsoft:windows:2320'
		- db : 'cpe:2.3:o:microsof:windows:*:*:**:*:*:*'
	@retval array<string> Le cpe parsé au format suivant :
		$ret['type'] = <a|o|h>
		$ret['vendor'] = $vendor
		$ret['product'] = $product
		$ret['version'] = $version|NULL
*/
function parse_nmap_cpe($cpe)
{
    $ret = array();
	$exp = explode(':',$cpe);
	if($exp[0]  == 'cpe' )
	{
		array_shift($exp);
	}
	elseif(is_numeric($exp[0]))
	{
		array_shift($exp);
	}

    $ret['type'] = trim(array_shift($exp),'/');
    if( strlen($ret['type']) > 1)
    {
		//Si on entre ici, alors le exp[1] vaut 2.3 et le type est en exp[2]
		$ret['type'] = trim(array_shift($exp),'/');
    }
    $ret['vendor'] = array_shift($exp);
    $ret['product'] = array_shift($exp);


    if( empty($exp))
    {
        $ret['version'] = NULL;
    }
    else
    {
        $ret['version'] = implode(':',$exp);
    }
    return $ret;
}

/**
    @brief Filtre une liste de CPEs par type (a=>hardware,o=>operating system,a=>application)
	@param array<string> $cpe_list Un array contenant une liste de cpe sous forme de string
	@param array<a|o|h>  $included_list est une liste de type à include. Les valeurs d	@param integer $portid Est le numéro du port conçerné
	@retval array<string>	La liste des CPEs de type conçernés
*/
function cpe_type_filter($cpe_list,$included_list)
{
    $ret = array();
    foreach($cpe_list as $cpe)
    {
        $parsed = parse_nmap_cpe($cpe);
        foreach($included_list as $included)
        {
            if( $included == $parsed['type'] )
            {
                $ret[] = $cpe;
            }
        }
    }
    return $ret;
}

/**
    @brief Filtre les CVEs d'un service en se basant sur une liste de CPEs pour ce service et une liste de CPEs pour le device
	@param array<array> $cve Un array contenant une liste de cve de type tableau, le tableau d'une cve doit avoir le format suivant :
		$cve['vul_cpe'] = json_encode($cve_rules);
	@param array<string> $services_cpe Un array contenant une liste de cpe sous forme de string pour les services
	@param array<string> $devices_cpe Un array contenant une liste de cpe sous forme de string pour indiquer le device conçerné
	@retval array<string>	La liste des CVEs qui sont matchés vulnérable pour les CPEs renseignés
*/
function cve_list_filter($cve_list,$services_cpe,$devices_cpe)
{
    $ret = array();
    foreach($cve_list as $cve)
    {
        if( is_vulnerable($cve,$services_cpe,$devices_cpe))
        {
            $ret[] = $cve;
        }
    }
    return $ret;
}

// ****************Fonctions utilisées pour filtrer les CVEs via les régles ******************

/**
    @brief Utilise la régle d'une CVE afin de vérifier si elle matche le(s) CPEs spécifiés
	@param array $cve Une CVE à checker, le tableau doit avoir le format suivant :
		$cve['vul_cpe'] = json_encode($cve_rules);
	@param array<string> $services_cpe Un array contenant une liste de cpe sous forme de string pour les services
	@param array<string> $cpe_hosts Un array contenant une liste de cpe sous forme de string pour indiquer le device conçerné
	@retval TRUE|FALSE	Si les CPEs concernent la CVE ou pas.
	@details
		Cette function récupére l'opérateur (AND/OR) et boucle sur tous les 'nodes' pour checker si ils sont vulnérable afin de calculer TRUE ou FALSE.
		La fonction utilsée pour vérifier si un noeud est vulnérable est is_CVE_node_vulnerable().
*/
function is_vulnerable($cve,$cpe_services,$cpe_hosts=NULL)
{
    if( $cpe_hosts == NULL )
    {
        $cpe_hosts = array();
    }

    $decoded = json_decode($cve['vul_cpe'],TRUE);
    $ret = FALSE;

	if(!is_array($decoded))
	{
		return FALSE;
	}
    foreach($decoded as $row)
    {
        if( empty($row['operator']))
        {
            $row['operator'] = 'AND';
        }
        foreach($row['nodes'] as $node)
        {
            $retRow = is_CVE_node_vulnerable($node,$cpe_services,$cpe_hosts);
            if( $row['operator'] == 'OR' && $retRow )
            {
                return TRUE;
            }
            elseif( $row['operator'] == 'AND' && !$retRow )
            {
                return FALSE;
            }
            else
            {
                $ret = ( $row['operator'] == 'AND' && !$ret );
            }
        }
    }
    return $ret;
}

/**
    @brief Utilise la partie nodes d'une régle CVE afin de déterminer si les nodes spécifiés matchent.
	@param array $node Partie nodes d'une régle CVE
		$node = json_decode($cve['vul_cpe'][$i]['nodes']
	@param array<string> $services_cpe Un array contenant une liste de cpe sous forme de string pour les services
	@param array<string> $cpe_hosts Un array contenant une liste de cpe sous forme de string pour indiquer le device conçerné
	@retval TRUE|FALSE	Si les CPEs concernent la CVE ou pas.
	@details
		Cette function récupére l'opérateur (AND/OR) et boucle sur tous les 'cpeMatch' pour checker si ils sont vulnérable afin de calculer TRUE ou FALSE.
		La fonction utilsée pour vérifier si un noeud est vulnérable est is_CVE_match_vulnerable().
*/
function is_CVE_node_vulnerable($node,$cpe_services,$cpe_hosts=NULL)
{
	if( empty($node['cpeMatch']))
	{
		return is_CVE_match_vulnerable($node,$cpe_services,$cpe_hosts);
	}

    if( $cpe_hosts == NULL )
    {
        $cpe_hosts = array();
    }
    $negate = ((isset($node['negate'])) && $node['negate']);

	$ret = FALSE;
    if( empty($node['operator']))
    {
        $node['operator'] = 'OR';
    }
    foreach($node['cpeMatch'] as $cpeMatch)
    {
        $retRow = is_CVE_node_vulnerable($cpeMatch,$cpe_services,$cpe_hosts);
        if( $node['operator'] == 'OR' && $retRow )
        {
            return ( TRUE && !$negate );
        }
        elseif( $node['operator'] == 'AND' && !$retRow )
        {
            return ( FALSE || $negate );
        }
        else
        {
            $ret = ( $node['operator'] == 'AND' && !$ret );
        }
    }
    if( $negate )
    {
        return !$ret;
    }
    return $ret;
}

/**
    @brief Utilise la partie cpeMatch d'une régle CVE afin de déterminer si les cpeMatch spécifiés matchent.
	@param array $cpeMatch Partie cpeMatch d'une régle CVE
		$cpeMatch = json_decode($cve['vul_cpe'][$i]['nodes'][$j]['cpeMatch']
		$cpeMatch contient au minimum les clés suivante :
		 - criteria : une CPE
		 - vulnerable : TRUE|FALSE si cette CPE est vulnérable
	@param array<string> $services_cpe Un array contenant une liste de cpe sous forme de string pour les services
	@param array<string> $cpe_hosts Un array contenant une liste de cpe sous forme de string pour indiquer le device conçerné
	@retval TRUE|FALSE	Si les CPEs concernent la CVE ou pas.
	@details
		Match la CPE et retourne TRUE si :
		- Une des CPE service $cpe_services est matché et que le match est taggé vulnerable = TRUE
		- Une seule des CPE device $cpe_hosts est matché
		Retourne FALSE sinon
*/
function is_CVE_match_vulnerable($cpeMatch,$cpe_services,$cpe_hosts=NULL)
{
	if(empty($cpe_services))
	{
		return TRUE;
	}
    if( $cpe_hosts == NULL )
    {
        $cpe_hosts = array();
    }
    if($cpeMatch['vulnerable'])
    {
        foreach($cpe_services as $service_cpe)
        {
            if(is_match_cpe($service_cpe,$cpeMatch))
            {
                return TRUE;
            }
        }
    }
    else
    {
        foreach($cpe_hosts as $host_cpe)
        {
            if(is_match_cpe($host_cpe,$cpeMatch))
            {
                return TRUE;
            }
        }
    }
}

/**
    @brief Vérifie si une CPE est matchée par un $cpeMatch
	@param string $cpe Une cpe
	@param array $cpeMatch Partie cpeMatch d'une régle CVE
		$cpeMatch = json_decode($cve['vul_cpe'][$i]['nodes'][$j]['cpeMatch']
		$cpeMatch contient au minimum les clés suivante :
		 - criteria : une CPE
		 $cpeMatch contient parfois les clés version[Start|End][Excluding|Including]
	@retval TRUE|FALSE	Si la CPE est matchée ou pas
	@details
		- Utilise la function is_same_cpe pour la clé $cpeMatch['criteria']
		- Utilise la function is_match_version pour les autres clés : version[Start|End][Excluding|Including]
*/
function is_match_cpe($cpe,$cpeMatch)
{
    $parsed_service = parse_nmap_cpe($cpe);
    $parsed_match = parse_nmap_cpe($cpeMatch['criteria']);

    if( !is_same_cpe($parsed_service,$parsed_match) )
    {
        return FALSE;
    }
    if(!empty(trim($parsed_service['version'],':*-')))
    {
        return is_match_version($cpeMatch,$parsed_service['version']);
    }
    return TRUE;
}

/**
    @brief Valide si une CPE correspond à un $cpeMatch['criteria']
	@param string $cpe Une cpe
	@param array $cpeMatch Partie cpeMatch d'une régle CVE
		$cpeMatch = json_decode($cve['vul_cpe'][$i]['nodes'][$j]['cpeMatch']
		$cpeMatch contient au minimum les clés suivante :
		 - criteria : une CPE
	@retval TRUE|FALSE	Si la CPE est matchée ou pas
*/
function is_same_cpe($parsed_service,$parsed_match)
{
    if( $parsed_service['type'] != $parsed_match['type'] )
    {
        return FALSE;
    }
    if( $parsed_service['vendor'] != $parsed_match['vendor'] )
    {
        return FALSE;
    }
    if( $parsed_service['product'] != $parsed_match['product'] )
    {
        return FALSE;
    }
    if( empty($parsed_service['version'])  || in_array($parsed_service['version'],array('*','-')) || $parsed_service['version'] == $parsed_match['version'] )
    {
        return TRUE;
    }
    $parsed_match['version'] = trim($parsed_match['version'],':*');
    if( empty($parsed_match['version']))
    {
        return TRUE;
    }
    $parsed_match['version'] = trim($parsed_match['version'],':-');
    if( empty($parsed_match['version']))
    {
        return FALSE;
    }
    return ( strpos($parsed_match['version'],$parsed_service['version']) === 0 ); //Si en début de chaine, on return TRUE, en fin de chaine ou non trouvé, on retourne FALSE
}

/**
    @brief Valide si le numéro de version d'une CPE à l'aide de clés $cpeMatch version[Start|End][Excluding|Including]
	@param string $cpe Une cpe
	@param array $cpeMatch Partie cpeMatch d'une régle CVE
		$cpeMatch = json_decode($cve['vul_cpe'][$i]['nodes'][$j]['cpeMatch']
		Cette fonction utilise uniquement les clés suivantes si elle sont présentes :
			 version[Start|End][Excluding|Including]
	@retval TRUE|FALSE	Si la CPE est matchée ou pas
*/
function is_match_version($cpeMatch,$version)
{
    if( isset($cpeMatch['versionStartExcluding']) && version_compare($version, $cpeMatch['versionStartExcluding'],"<="))
    {
        return FALSE;
    }
    if( isset($cpeMatch['versionStartIncluding']) && version_compare($version, $cpeMatch['versionStartIncluding'],"<"))
    {
        return FALSE;
    }
    if( isset($cpeMatch['versionEndIncluding']) && version_compare($version, $cpeMatch['versionEndIncluding'],">"))
    {
        return FALSE;
    }
    if( isset($cpeMatch['versionEndExcluding']) && version_compare($version, $cpeMatch['versionEndExcluding'],">="))
    {
        return FALSE;
    }
    return TRUE;
}

// ************************SECTION GET CVEs by word into DESC*******************
function getAppCVEsByProductInfo($dbconnection,$product,$vendor=NULL)
{
    $sql = '
        SELECT vul_id,vul_list_id,vul_cpe
        FROM egide_vulnerability
        WHERE vul_description ILIKE :product
    ';
    $params[':product'] = getServiceProductWordSQLLike($product['service_product']);
	if( $vendor == 'microsoft')
	{
		if( empty($params[':product'] ))
		{
			$params[':product'] = '%%';
			$sql.= ' AND vul_description ILIKE :vendor';
			$params[':vendor'] = "% $vendor %";
			$sql.= ' AND vul_description ILIKE :service';
			$params[':service'] = '% '.$product['service_name'].' %';
		}
		if(strpos($params[':product'], 'netbios') !== false )
		{
			$params[':product'] = '%netbios%';
			$params[':vendor'] = "% $vendor %";
			$sql.= ' AND vul_description ILIKE :vendor';
		}
		elseif(strpos($params[':product'], 'HTTPAPI') !== false )
		{
			$params[':product'] = '%http%api%';
			$params[':vendor'] = "% $vendor %";
			$sql.= ' AND vul_description ILIKE :vendor';
		}
		elseif(isset($params[':service']) && strpos($params[':service'], 'microsoft-ds') !== false )
		{
			$params[':service'] = '%Server%Message%Block%';
			$params[':vendor'] = "% $vendor %";
			$sql.= ' AND vul_description ILIKE :vendor';
		}

	}
	elseif( !empty($vendor))
    {
        $sql.= ' AND vul_description ILIKE :vendor';
        $params[':vendor'] = "% $vendor %";
    }
    else
    {
        $sql.= ' AND vul_description ILIKE :service';
        $params[':service'] = '% '.$product['service_name'].' %';
    }
	$stmt = $dbconnection->prepare($sql);
	$stmt->execute($params);

	$ret = array();
	while($row=$stmt->fetch(PDO::FETCH_ASSOC))
	{
		$ret[$row['vul_list_id']]=$row;
	}
    return array_values($ret);
}

function getServiceProductWordSQLLike($product_name)
{
    $ignore_words = array('microsoft','windows','cisco');
	$match_words = array('cisco:telnet','cisco:http');

	foreach($match_words as $word)
	{
		$words = explode(':',$word);
		if(stripos($product_name,$words[0]) !==FALSE)
		{
			if(stripos($product_name,$words[1]) !==FALSE)
			{
				return "%$words[1]%";
			}
		}
	}

    $product_name = trim($product_name);
    $list_words = explode(' ',$product_name);

    //1 mot doit avoir au moins 2 lettres successives et être plus grand que 2
    $tmp = array();
    foreach($list_words as $word)
    {
        if( strlen($word) > 2 && preg_match('/[a-zA-Z]{2,}/',$word))
        {
            $tmp[] = $word;
        }
    }
    $list_words = $tmp;

    $product_name = implode(' ',$list_words);
    if( count($list_words) >= 3 )
    {
        //Le résultat intéressant est toujours dans les 3 premiers mots
        $list_words = array_slice($list_words, 0, 4);

        //Si on a 3 mots, on tente de diminuer car il nous en faut 1 ou 2
        //1. on enleve les mots indésirables
        $tmp = array();
        foreach($list_words as $word)
        {
            if(!in_array(strtolower($word),$ignore_words))
            {
                $tmp[] = $word;
            }
        }
        $list_words = $tmp;

        //On passe a 3 mots
        $list_words = array_slice($list_words, 0, 3);

        //2. en enlevant ceux-sans majuscules
        $tmp = array();
        foreach($list_words as $word)
        {
            if(preg_match('/[A-Z]/', $word))
            {
                $tmp[] = $word;
            }
        }
        if( count($tmp) > 0)
        {
            $list_words = $tmp;
        }

        if( count($list_words) == 3 )
        {
            //2. en enlevant ceux-sans majuscules
            $tmp = array();
            foreach($list_words as $word)
            {
                if(preg_match('/[A-Z]/', $word))
                {
                    $tmp[] = $word;
                }
            }
            $list_words = $tmp;

            //2. On selectionne les gros mots qui ne sont qu'en majuscule si il y en a.
            $tmp = array();
            foreach($list_words as $word)
            {
                if(strlen($word) > 2 && strtoupper($word) == $word)
                {
                    $tmp[] = $word;
                }
            }
            if( count($tmp) > 0 )
            {
                $list_words = $tmp;
            }

            //2. Si non, ou si il y en a plusieurs, on elimine le plus petit
            $smallest_size = 1000;
            foreach($list_words as $key => $word)
            {
                $size = strlen($word);
                if( $size <= $smallest_size)
                {
                    $smallest_word = $key;
                    $smallest_size = $size;
                }
            }
            unset($list_words[$smallest_word]);
        }
        $product_name = implode(' ',$list_words);
    }

    $ret = '';
    if( count($list_words) == 1 ) //1 word, on retourne tel quel
    {
        $ret = $list_words[0];
    }
    if( count($list_words) == 2 ) //2 words, on retire la lettre d a la fin si il y en a une (pour le d de Apache httpd par exemple).
    {
        $ret = rtrim($product_name,'dD');
    }
    $ret = preg_replace('/[^a-zA-Z0-9\.]+/','%',$ret);
    if( empty(trim($ret)))
    {
        return FALSE;
    }
    return "% $ret %";
}

// *********** Essai de fonctions pour microsoft ****************************
function microsoft_os_identitify($build,$version)
{
	$ret = array();
	$ret['build'] = $build;
	if( $build > 22621 )
	{
		$ret['start_date'] = '2022-09-20';
	}
	elseif( $build == 22621 )
	{
		$ret['start_date'] = '2022-09-20';
		$ret['product_num'] = '11';
		$ret['product_version'] = '22H2';
	}
	elseif( $build >= 22000 )
	{
		$ret['start_date'] = '2021-10-05';
		$ret['product_num'] = '11';
		if( $build == 18363 )
		{
			$ret['product_version'] = '21H2';
		}
	}
	elseif( $build >= 19045 )
	{
		$ret['start_date'] = '2022-10-18';
		$ret['product_num'] = '10';
		if( $build == 19045 )
		{
			$ret['product_version'] = '22H2';
		}
		elseif( $build == 20348 )
		{
			$ret['start_date'] = '2021-08-18 ';
			unset($ret['product_num']);
			$ret['server_num'] = '2022';
			$ret['product_version'] = '21H2';
		}
	}
	elseif( $build == 19044 )
	{
		$ret['start_date'] = '2021-11-16';
		$ret['product_num'] = '10';
		$ret['product_version'] = '21H2';
	}
	elseif( $build == 19043 )
	{
		$ret['start_date'] = '2021-11-16';
		$ret['product_num'] = '10';
		$ret['product_version'] = '21H1';
	}
	elseif( $build == 19042 )
	{
		$ret['start_date'] = '2020-10-20';	#date_server
		$ret['product_num'] = '10';
		$ret['server_num'] = '2019';
		$ret['product_version'] = '20H2';
	}
	elseif( $build == 19041 )
	{
		$ret['start_date'] = '2020-06-26';	#date_server
		$ret['product_num'] = '10';
		$ret['server_num'] = '2019';
		$ret['product_version'] = '2004';
	}
	elseif( $build >= 18363 )
	{
		$ret['start_date'] = '2019-11-12';	#common_date
		$ret['product_num'] = '10';
		$ret['server_num'] = '2019';
		if( $build == 18363 )
		{
			$ret['product_version'] = '1909';
		}
	}
	elseif( $build == 18362 )
	{
		$ret['start_date'] = '2019-05-21';	#common_date
		$ret['product_num'] = '10';
		$ret['server_num'] = '2019';
		$ret['product_version'] = '1903';
	}
	elseif( $build >= 17763 )
	{
		$ret['start_date'] = '2018-11-13';	#common_date
		$ret['product_num'] = '10';
		$ret['server_num'] = '2019';
		if( $build == 17763 )
		{
			$ret['product_version'] = '1809';
		}
	}
	elseif( $build >= 17134 )
	{
		$ret['start_date'] = '2018-04-30';	#common_date
		$ret['product_num'] = '10';
		$ret['server_num'] = '2016';
		if( $build == 17134 )
		{
			$ret['product_version'] = '1803';
		}
	}
	elseif( $build >= 16299 )
	{
		$ret['start_date'] = '2017-10-17';	#common_date
		$ret['product_num'] = '10';
		$ret['server_num'] = '2016';
		if( $build == 16299 )
		{
			$ret['product_version'] = '1709';
		}
	}
	elseif( $build >= 15063 )
	{
		$ret['start_date'] = '2017-04-05';	#common_date
		$ret['product_num'] = '10';
		if( $build == 15063 )
		{
			$ret['product_version'] = '1703';
		}
	}
	elseif( $build >= 14393 )
	{
		$ret['start_date'] = '2016-08-02';	#date_windos
		$ret['product_num'] = '10';
		$ret['server_num'] = '2016';
		if( $build == 14393 )
		{
			$ret['product_version'] = '1607';
		}
	}
	elseif( $build >= 10586 )
	{
		$ret['start_date'] = '2015-11-10';
		$ret['product_num'] = '10';
		if( $build == 10586 )
		{
			$ret['product_version'] = '1511';
		}
	}
	elseif( $build >= 10240 )
	{
		$ret['start_date'] = '2015-07-29';
		$ret['product_num'] = '10';
		$ret['product_version'] = '10.0';
	}
	elseif( $build >= 9600 )
	{
		$ret['start_date'] = '2013-10-17';
		$ret['product_num'] = '8.1';
		$ret['server_num'] = '2012';
		$ret['product_version'] = '6.3';
	}
	elseif( $build >= 9200 || $version == '6.2' )
	{
		$ret['start_date'] = '2012-09-04';
		$ret['product_num'] = '8';
		$ret['server_num'] = '2012';
		$ret['product_version'] = $version;
	}
	elseif( $build >= 7601 || $version == '6.1' )
	{
		$ret['start_date'] = '2009-10-22';
		$ret['product_num'] = '7';
		$ret['server_num'] = '2008';
		$ret['product_version'] = $version;
	}
	elseif( $build >= 6002 || $version == '6' || $version == '6.0')
	{
		$ret['start_date'] = '2007-01-30';
		$ret['product_num'] = '6';
		$ret['server_num'] = '2008';
		$ret['product_version'] = 6;
	}
	elseif( $build >= 3790 || $version == '5.2')
	{
		$ret['start_date'] = '2003-04-24';
		$ret['product_num'] = 5;
		$ret['server_num'] = '2003';
		$ret['product_version'] = $version;
	}
	elseif( $build >= 2600 || $version == '5.1')
	{
		if( $build >= 2710 )
		{
			$ret['start_date'] = '2005-10-14';
		}
		elseif( $build >= 2700 )
		{
			$ret['start_date'] = '2004-10-12';
		}
		else
		{
			$ret['start_date'] = '2003-09-30';
		}
		$ret['product_num'] = 5;
		$ret['product_version'] = $version;
	}
	if( $ret['product_version'] == '5.1' || $ret['product_version'] == '5.2' )
	{
		$ret['product_name'] = 'xp';
	}
	elseif( isset($ret['product_num']))
	{
		if($ret['product_num'] == '6' )
		{
			$ret['product_name'] = 'vista';
		}
		else
		{
			$ret['product_name'] = $ret['product_num'];
		}
	}
	return $ret;
}

function search_microsoft_cve($dbconnection,$product_identity)
{
	$params = array();
	$sql = "
		SELECT vul_list_id,vul_date_publishing,vul_date_last_modify,vul_cpe,vul_access_type,vul_description
		FROM egide_cpe_vulnerability
		JOIN egide_vulnerability ON(cpe_vul_id = vul_id)
		WHERE cpe_product_id IN
		(
			SELECT product_id
			FROM egide_cpe_product
			WHERE product_vendor_id = (SELECT vendor_id FROM egide_cpe_vendor WHERE vendor_name = 'microsoft')
			AND product_type = 'operating system'
			AND product_name LIKE '%windows%'
			AND product_name NOT LIKE '%mobile%'
			AND product_name NOT LIKE '%phone%'
			AND (
				product_name = 'all_windows'
				OR product_name = 'windows'
				OR product_name like 'windows_server'
	";

	//Windows XX
	if( isset($product_identity['product_name']) )
	{
		$params[':all_product_name'] = 'windows_'.$product_identity['product_name'];
		$sql .= "
			OR product_name like :all_product_name
		";
		if( isset($product_identity['product_version']))
		{
			$params[':name_and_version'] = 'windows_'.$product_identity['product_name'].'_'.$product_identity['product_version'];
			$sql .= "
				OR product_name like :name_and_version
			";
		}
	}


	//Windows server XX
	if( isset($product_identity['product_version']) )
	{
		$params[':product_server_version'] = 'windows_server_'.$product_identity['product_version'];
		$sql .= "
			OR product_name like :product_server_version
		";
	}
	if( isset($product_identity['server_num']) )
	{
		$params[':product_server_num'] = 'windows_server_'.$product_identity['server_num'];
		$sql .= "
			OR product_name like :product_server_num
		";
	}

	$sql .= ")) AND cpe_equal_version IN('-','*'";

	if( isset($product_identity['product_version']) )
	{
		$params[':product_version'] = $product_identity['product_version'];
		$sql .= ",:product_version";
	}
	if( isset($product_identity['server_num']) )
	{
		$params[':server_num'] = $product_identity['server_num'];
		$sql .= ",:server_num";
	}
	$sql .= ') ORDER BY vul_date_publishing ASC';

	$stmt = $dbconnection->prepare($sql);

	$stmt->execute($params);
	$ret = array();
	while($row=$stmt->fetch(PDO::FETCH_ASSOC))
	{
		$ret[$row['vul_list_id']]=$row;
	}
	return array_values($ret);
}

// ************************SECTION DEVICE IDENTIFICATION*******************
function get_microsoft_cpe_from_product_identity($product_identity,$version=NULL)
{
	if(empty($version)&&isset($product_identity['product_name']))
	{
		$version = $product_identity['product_name'].'.0.'.$product_identity['build'];
	}

	$type = 'unknown';
    $ret = array('windows','all_windows');
	if( isset($product_identity['type']))
	{
		if($product_identity['type'] == 'server' )
		{
			$type = 'server';
			$ret[] = 'windows_server';
		}
		else
		{
			$type = 'workstation';
		}
	}

	if( $type != 'server' )
	{
	    //Windows XX
		if( isset($product_identity['product_name']) )
		{
			$ret[] = 'windows_'.$product_identity['product_name'];
		//	$ret[] = 'windows_'.$product_identity['product_name'].':'.$version;
			if( isset($product_identity['product_version']))
			{
				$ret[] = 'windows_'.$product_identity['product_name'].'_'.$product_identity['product_version'];
		//		$ret[] = 'windows_'.$product_identity['product_name'].'_'.$product_identity['product_version'].':'.$version;
				$ret[] = 'windows_'.$product_identity['product_name'].':'.$product_identity['product_version'];
			}
		}
	}

	if( $type != 'workstation' )
	{
		if( isset($product_identity['product_version']) )
		{
			if( isset($product_identity['server_num']) )
			{
				$ret[] = 'windows_server_'.$product_identity['product_version'].':'.$product_identity['server_num'];
			}
			$ret[] = 'windows_server_'.$product_identity['product_version'];
			$ret[] = 'windows_server:'.$product_identity['product_version'];
		}
		if( isset($product_identity['server_num']) )
		{
			if( isset($product_identity['product_version']) )
			{
				$ret[] = 'windows_server_'.$product_identity['server_num'].':'.$product_identity['product_version'];
			}
			else
			{
				$ret[] = 'windows_server_'.$product_identity['server_num'];
				$ret[] = 'windows_server:'.$product_identity['server_num'];
				$ret[] = 'windows:'.$product_identity['server_num'];
				$ret[] = 'windows:server_'.$product_identity['server_num'];
			}
		}
	}

	if( !empty($version))
	{
		$version = ":$version";
	}
	else
	{
		$version = '';
	}
    foreach($ret as $idx => $value)
    {
		$exp = explode(':',$value);

		$ret[$idx] = "o:microsoft:$value";
		if( !isset($exp[1]))
		{
			$ret[$idx].=$version;
		}

    }
    return $ret;
}

function identify_product_by_services($cpe_list_services)
{
	$parsed_list = array('h'=>array(),'o'=> array());
	foreach($cpe_list_services as $service)
	{
		foreach($service['cpe'] as $cpe)
		{
			$parsed = parse_nmap_cpe($cpe);
			if( isset($parsed_list[$parsed['type']]))
			{
				$product = $parsed['vendor'].':'.$parsed['product'];
				if( !isset($parsed_list[$parsed['type']][$product]))
				{
					$parsed_list[$parsed['type']][$product] = 1;
				}
				else
				{
					$parsed_list[$parsed['type']][$product]++;
				}
			}
		}
	}

	$ret = array();
	foreach(maxValueKeys($parsed_list['o']) as $row )
        {
		$nb = $parsed_list['o'][$row];
        	$ret[] = "$nb:o:".$row;
        }
        foreach(maxValueKeys($parsed_list['h']) as $row )
        {
		$nb = $parsed_list['h'][$row];
		if( $nb > 1 )
		{
                	$ret[] = "$nb:h:".$row;
		}
        }
	return $ret;
}

function identify_product_by_devices($cpe_list_devices)
{
	$parsed_list = array('h'=>array(),'o'=> array());
	foreach($cpe_list_devices as $cpe)
	{
		$parsed = parse_nmap_cpe($cpe);
		if( isset($parsed_list[$parsed['type']]))
		{
			$product = $parsed['vendor'].':'.$parsed['product'];
			if( !isset($parsed_list[$parsed['type']][$product]))
			{
				$parsed_list[$parsed['type']][$product] = 1;
			}
			else
			{
				$parsed_list[$parsed['type']][$product]++;
			}
		}
	}

	$ret = array();
	foreach(maxValueKeys($parsed_list['o']) as $row )
        {
		$nb = $parsed_list['o'][$row];
                if( $nb > 1)
		{
        		$ret[] = "$nb:o:".$row;
		}
        }
        foreach(maxValueKeys($parsed_list['h']) as $row )
        {
		$nb = $parsed_list['h'][$row];
		if( $nb > 1)
		{
                	$ret[] = "$nb:h:".$row;
		}
        }
	return $ret;
}

function identify_partial_product_by_devices($cpe_list_devices)
{
      $ret = array();
      $str = findLongestRecurringSubstring($cpe_list_devices);
      $exp = explode(':',$str);
      if( count($exp) > 3 && strlen($exp[3]) > 3)
      {
	  $word = preg_replace('/[^a-z]+$/','',$exp[3]);
	  if(!empty($word))
	  {
		$exp[0] = 1;
		$exp[1] = trim($exp[1],'/');
		$exp[3] = $word;
	  	$ret[] = implode(':',$exp);
	  }
      }
      return $ret;
}

function findLongestRecurringSubstring($array) {
    $maxLength = 0;
    $longestSubstring = '';

    foreach ($array as $str1)
    {
        foreach ($array as $str2)
        {
            if ($str1 !== $str2)
            {
                $length = 0;
                $substring = '';

                // Compare characters of two strings
                for ($i = 0; $i < strlen($str1) && $i < strlen($str2); $i++)
                {
                    if ($str1[$i] === $str2[$i])
                    {
                        $substring .= $str1[$i];
                        $length++;
                    }
                    else
                    {
                        break; // Stop when characters don't match
                    }
                }

                // Update the longest substring if needed
                if ($length > $maxLength)
                {
                    $maxLength = $length;
                    $longestSubstring = $substring;
                }
            }
        }
    }

    return $longestSubstring;
}

function maxValueKeys($arr) {
    $keyMax = [];
    $valueMax = null;

    foreach ($arr as $key => $value) {
        if ($valueMax === null || $value > $valueMax) {
            // Nouvelle valeur maximale trouvée, réinitialiser le tableau des clés maximales
            $keyMax = array($key);
            $valueMax = $value;
        } elseif ($value === $valueMax) {
            // Plusieurs clés avec la même valeur maximale
            $keyMax[] = $key;
        }
    }
    if( count($keyMax) > 1 ) return array();
    return $keyMax;
}
?>
