<?php

/**
 * @name Model_TableSort
 * @description Classe modèle d'Esia
 *		Toutes les méthodes sont déclarée en 'static' ainsi on garde les Objets PDO pour les programmeurs extérieurs et pour faire des modifications, ils ne sont pas /
 *		obligé de connaitre la hierachie de classe d'ESIA
 * @author Nicolas Biersart <n.biersart@esia-network.com>
 * @contributor
 * @copyright Nicolas Biersart 2012
 * @license
 * @esia 1.00
 * @version 1.0.0
 * @date 16/11/2012
 */
$path = $_SERVER['DOCUMENT_ROOT'];
include_once $path.'/model_tablesort.php';

$arr = Model_esia::getWebPlugin($dbconnection,false,'WEBP_EGIDESCAN');
$plugin_url = $arr[0]['apache_path'];
include_once $path.$plugin_url.'/model_egidescan.php';

class Model_EgideScan_TableSort extends Model_TableSort
{
	/**
	 *  retourne le tableau de correspondance entre le paramètre "cols" du tablesorter et les champs de la base de donnée et le type de tri. Il retourne uniquemejnt les colonnes pour les erreurs active
	 * @param array $lang_error_type tableau associatif faisant la correspondance entre le nom technique des erreurs (error_type) et les variables de langue.
	 * 		format FR: Array ( [warning] => Alerte,    [critical] => Critique,    [unknown] => Inconnu,    [maintenance] => Maintenance)
	 * @param array $lang_plugin tableau associatif faisant la correspondance entre le nom technique des plugins et le nom traduit
	 * 		format FR: Array ( [CHECK_SNMP_LOAD] => Processeur)
	 * @retval array  Tableau de correspondance entre le SQL et le tableSorter
	 */
	private static function ts_getListCVE_columns()
	{
		//l'index de la column doit être égale à l'index dans le table sorter
		$columns[]=array('col'=>'vul_list_id','type'=>'string'); //Fake row pour checkbox (le nombre de colonne doit être identique)
		$columns[]=array('col'=>'vul_list_id','type'=>'string');
		$columns[]=array('col'=>'happen_time','type'=>'date','data'=>'vul_date_publishing');
		$columns[]=array('col'=>'vul_description','type'=>'string');
		$columns[]=array('col'=>'vul_scoring','type'=>'integer');
		$columns[]=array('col'=>'vul_severity','type'=>'string');
		$columns[]=array('col'=>'vul_attack_complexity','type'=>'string');
		$columns[]=array('col'=>'matchnum','type'=>'integer');
		return $columns;
	}

	private static function ts_getListACQ_columns()
	{
		//l'index de la column doit être égale à l'index dans le table sorter
		$columns[]=array('col'=>'vul_list_id','type'=>''); //Fake row pour checkbox (le nombre de colonne doit être identique)
		$columns[]=array('col'=>'vul_list_id','type'=>'string');
		$columns[]=array('col'=>'device_name','type'=>'string');
		$columns[]=array('col'=>'eg_service_name','type'=>'string');
		$columns[]=array('col'=>'happen_time','type'=>'date','data'=>'detect_date_update_state');
		$columns[]=array('col'=>'vul_description','type'=>'string');
		$columns[]=array('col'=>'vul_scoring','type'=>'integer');
		$columns[]=array('col'=>'vul_severity','type'=>'string');
		$columns[]=array('col'=>'vul_attack_complexity','type'=>'string');
		$columns[]=array('col'=>'state_name','type'=>'string');
		return $columns;
	}

	private static function ts_getListSRV_columns()
	{
		//l'index de la column doit être égale à l'index dans le table sorter
		$columns[]=array('col'=>'eg_service_id','type'=>''); //Fake row pour checkbox (le nombre de colonne doit être identique)
		$columns[]=array('col'=>'eg_service_name','type'=>'string');
		$columns[]=array('col'=>'device_name','type'=>'string');
		$columns[]=array('col'=>'device_type','type'=>'string');
		$columns[]=array('col'=>'device_ip_address','type'=>'ipaddress');
		$columns[]=array('col'=>'avg_scoring','type'=>'integer');
		$columns[]=array('col'=>'max_scoring','type'=>'integer');
		$columns[]=array('col'=>'matchnum','type'=>'integer');
		$columns[]=array('col'=>'eg_service_product','type'=>'string');
		$columns[]=array('col'=>'eg_service_version','type'=>'string');
		$columns[]=array('col'=>'eg_service_port','type'=>'integer');
		$columns[]=array('col'=>'eg_service_port_type','type'=>'string');
		return $columns;
	}
	/**
	 * Récupère les erreurs en cours dans la base de donnée
	 * @param integer $user identifiant utilisateur
	 * @param integer $typeInventory type d'inventaire
	 * @param array $TSParams tableau associatif envoyé par le tablesorter (format: Array(
	 *		    [filters] => Array // numéros de colonne et le filtre appliqué
	 *		        (
	 *					[4] => critique
	 *			    ),
	 *			[order] => Array // numéros de colonne qui trie en ascendant ou descendant
	 *			        (
	 *			            [8] => 1
	 *			        )
	 *			[size] => 25, //Nombre d'élement afficher dans la page
	 *			[paging] => 0) //Numéros de page
	 * @param array $lang_error_type tableau associatif faisant la correspondance entre le nom technique des erreurs (error_type) et les variables de langue.
	 * 		format FR: Array ( [warning] => Alerte,    [critical] => Critique,    [unknown] => Inconnu,    [maintenance] => Maintenance)
	 * @param array $lang_plugin tableau associatif faisant la correspondance entre le nom technique des plugins et le nom traduit
	 * 		format FR: Array ( [CHECK_SNMP_LOAD] => Processeur)
	 * @throws EsiaDbException
	 * @retval array array('total'=>$total,'rows'=>$arr); Total est le nombre d'erreur présente et le rows sont les données en tenant compte de la pagination
	 */
	public static function ts_getListCVE($dbconnection,$mode_id_list,$treeview_type,$treeview_nodeid,$user=NULL,$TSParams=NULL)
	{
		$columns = Model_EgideScan_TableSort::ts_getListCVE_columns();
		if( $treeview_type == 'root' )
		{
			$where = '';
		}
		elseif($treeview_type == 'group')
		{
			$box_list = Model_egidescan::getAgentsOfGroup($dbconnection,$treeview_nodeid,$user);
			$in_request = generateInParams('AND',$treeview_nodelist,'vul_iface_agent_id');
			$param = $in_request['param'];
			$where = $in_request['sql'];
		}
		elseif($treeview_type == 'box')
		{
			$where = "AND vul_iface_agent_id=$treeview_nodeid";
		}
		elseif($treeview_type == 'nodeonly')
		{
			$param[':nodeonly'] = $treeview_nodeid;
			$where = "AND eg_device_id=:nodeonly";
		}
		$sql = "
		WITH vulNumber
		    AS (
		        SELECT vul_id, count(*) as nbr
		        FROM egide_vulnerability
		        JOIN egide_vulnerability_detect  ON vul_id = detect_vul_id
		        JOIN egide_vulnerability_detect_state ON state_id=detect_state_id
		        JOIN egide_services ON eg_service_id=detect_eg_service_id
		        JOIN egide_service_access ON eg_access_service_id=eg_service_id
		        JOIN vulnerability_iface ON eg_access_iface_id=vul_iface_id
		        WHERE state_name = 'ACTIVE'
		        AND eg_device_id IN ( SELECT * FROM getUserAccessNode(:user))
				$where
		        GROUP BY vul_id
		    )
			SELECT vul_list_id,vul_description, vul_scoring,
			        vul_severity, vul_attack_complexity,
			        vul_date_publishing,to_char(vul_date_publishing,'DD/MM/YYYY HH24:MI:SS') AS happen_time, vulNumber.nbr as matchnum
			FROM egide_vulnerability
			JOIN vulNumber ON vulNumber.vul_id = egide_vulnerability.vul_id
		";
		$param[':user'] = $user;

		if( $mode_id_list )
		{
			$sql = "SELECT vul_list_id AS vul_id FROM ($sql) as tabletmp";
			$sql_order = '';
			$TSParams['order'] = NULL;

			$flag = Model_TableSort::ts_getActiveErrors_filter($columns,$TSParams,$sql,$param,$sql_order);

			$query = $dbconnection->prepare($sql);
			$query = Model_esia::executeStatement($query,$param);
			return $query->fetchAll(PDO::FETCH_ASSOC);
		}

 		$result = self::ts_executeRequest($dbconnection,$sql,$param,$columns,$TSParams,'matchNum DESC,vul_scoring DESC');

 		$query = $result['response'];
 		$ret['total'] = $result['total'];
 		$ret['rows'] = array();

		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$ret['rows'][] = array(
				'id' => $row->vul_list_id,
				'vul_id'=> $row->vul_list_id,
				'publishing' => explode(' ',$row->vul_date_publishing)[0],
				'description'=> $row->vul_description,
				'scoring'=>$row->vul_scoring,
				'severity' => $row->vul_severity,
				'complexity' => $row->vul_attack_complexity,
				'number' => $row->matchnum,
				'action' => ''
			);
		}
		return $ret;
	}

	public static function ts_getListACQ($dbconnection,$mode_id_list,$treeview_type,$treeview_nodeid,$user=NULL,$TSParams=NULL)
	{
		if( $treeview_type == 'root' )
		{
			$where = '';
		}
		elseif($treeview_type == 'group')
		{
			$box_list = Model_egidescan::getAgentsOfGroup($dbconnection,$treeview_nodeid,$user);
			$in_request = generateInParams('AND',$treeview_nodelist,'vul_iface_agent_id');
			$param = $in_request['param'];
			$where = $in_request['sql'];
		}
		elseif($treeview_type == 'box')
		{
			$param[':box'] = $treeview_nodeid;
			$where = "AND vul_iface_agent_id=:box";
		}
		elseif($treeview_type == 'nodeonly')
		{
			$param[':nodeonly'] = $treeview_nodeid;
			$where = "AND eg_device_id=:nodeonly";
		}

		$columns = Model_EgideScan_TableSort::ts_getListACQ_columns();

		$sql = "SELECT detect_id,state_name,vul_list_id,vul_description, vul_scoring, vul_severity, vul_attack_complexity,device_name,detect_date_update_state,to_char(detect_date_update_state,'DD/MM/YYYY HH24:MI:SS') AS happen_time,detect_username,eg_service_name";
		$sql .= " FROM egide_vulnerability
				 JOIN egide_vulnerability_detect ON vul_id = detect_vul_id
				 JOIN egide_vulnerability_detect_state ON state_id=detect_state_id
				 JOIN egide_services ON eg_service_id=detect_eg_service_id
				 JOIN egide_service_access ON eg_access_service_id=eg_service_id
				 JOIN vulnerability_iface ON eg_access_iface_id=vul_iface_id
				 JOIN esia_device ON (eg_device_id = device_id)
				 WHERE eg_device_id IN ( SELECT * FROM getUserAccessNode(:user)) AND state_name != 'ACTIVE' $where";

		$param[':user'] = $user;

		if( $mode_id_list )
		{
			$sql = "SELECT detect_id AS vul_id FROM ($sql) as tabletmp";
			$sql_order = '';
			$TSParams['order'] = NULL;
			$flag = Model_TableSort::ts_getActiveErrors_filter($columns,$TSParams,$sql,$param,$sql_order);
			$query = $dbconnection->prepare($sql);
			$query = Model_esia::executeStatement($query,$param);
			return $query->fetchAll(PDO::FETCH_ASSOC);
		}

		$result = self::ts_executeRequest($dbconnection,$sql,$param,$columns,$TSParams,'detect_date_update_state DESC,vul_list_id DESC');

		$query = $result['response'];
		$ret['total'] = $result['total'];
		$ret['rows'] = array();

		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$ret['rows'][] = array(
				'id' => $row->detect_id,
				'vul_id'=> $row->vul_list_id,
				'acq_date' => explode('.',$row->detect_date_update_state)[0],
				'description'=> $row->vul_description,
				'scoring'=>$row->vul_scoring,
				'severity' => $row->vul_severity,
				'complexity' => $row->vul_attack_complexity,
				'device_name' => $row->device_name,
				'service_name' => $row->eg_service_name,
				'detect_state' => $row->state_name,
				'action' => ''
			);
		}
		return $ret;
	}

	public static function ts_getListSRV($dbconnection,$mode_id_list,$treeview_type,$treeview_nodeid,$user=NULL,$TSParams=NULL)
	{
		if( $treeview_type == 'root' )
		{
			$where = '';
		}
		elseif($treeview_type == 'group')
		{
			$box_list = Model_egidescan::getAgentsOfGroup($dbconnection,$treeview_nodeid,$user);
			$in_request = generateInParams('AND',$treeview_nodelist,'vul_iface_agent_id');
			$param = $in_request['param'];
			$where = $in_request['sql'];
		}
		elseif($treeview_type == 'box')
		{
			$param[':box'] = $treeview_nodeid;
			$where = "AND vul_iface_agent_id=:box";
		}
		elseif($treeview_type == 'nodeonly')
		{
			$param[':nodeonly'] = $treeview_nodeid;
			$where = "AND eg_device_id=:nodeonly";
		}

		$columns = Model_EgideScan_TableSort::ts_getListSRV_columns();

		$sql = "SELECT eg_service_id, eg_service_name, eg_service_product, eg_service_version, eg_service_port, eg_service_cpe,
					eg_service_extrainfo,eg_service_port_type, host(device_ip_address) AS device_ip_address, device_name, device_type,
					count(detect_vul_id) as matchnum,AVG(vul_scoring) AS avg_scoring,MAX(vul_scoring) AS max_scoring
				FROM public.egide_services
				JOIN egide_vulnerability_detect ON eg_service_id = detect_eg_service_id
				JOIN egide_vulnerability_detect_state ON state_id=detect_state_id
				JOIN esia_device ON device_id = eg_device_id
				JOIN egide_vulnerability ON vul_id = detect_vul_id
				JOIN egide_service_access ON eg_access_service_id=eg_service_id
				JOIN vulnerability_iface ON eg_access_iface_id=vul_iface_id

				WHERE device_id IN ( SELECT * FROM getUserAccessNode(:user))
					AND state_name = 'ACTIVE' $where
				GROUP BY eg_service_id, eg_service_name, eg_service_product, eg_service_version, eg_service_port, eg_service_cpe,
					eg_service_extrainfo,eg_service_port_type, device_ip_address, device_name,device_type";

		$param[':user'] = $user;

		if( $mode_id_list )
		{
			$sql = "SELECT eg_service_id AS srv_id FROM ($sql) as tabletmp";
			$sql_order = '';
			$TSParams['order'] = NULL;
			$flag = Model_TableSort::ts_getActiveErrors_filter($columns,$TSParams,$sql,$param,$sql_order);
			$query = $dbconnection->prepare($sql);
			$query = Model_esia::executeStatement($query,$param);
			return $query->fetchAll(PDO::FETCH_ASSOC);
		}

		$result = self::ts_executeRequest($dbconnection,$sql,$param,$columns,$TSParams,'matchNum DESC,avg_scoring DESC,max_scoring');

		$query = $result['response'];
		$ret['total'] = $result['total'];
		$ret['rows'] = array();

		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$ret['rows'][] = array(
				'srv_id' => $row->eg_service_id,
				'srv_name' => $row->eg_service_name,
				'dev_name' => $row->device_name,
				'dev_type' => $row->device_type,
				'dev_addr' => $row->device_ip_address,
				'avg_score' => $row->avg_scoring,
				'max_score' => $row->max_scoring,
				'vul_total' => $row->matchnum,
				'srv_product' => $row->eg_service_product,
				'srv_version' => $row->eg_service_version,
				'port_num' => $row->eg_service_port,
				'port_type' => $row->eg_service_port_type,
				'action' => ''
			);
		}
		return $ret;
	}

	public static function ts_getListCVEByNode($dbconnection,$user=NULL,$TSParams=NULL,$node=NULL)
	{
		return self::ts_getListCVE($dbconnection,FALSE,$treeview_type,$treeview_nodeid,$user,$TSParams);
	}

	/**
	*** Get data for a vulnerability found by vulnerability DB ID. (vulnerability, node, egide box) box interface)
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $userId l'id de l'utilisateur (pour l'acccess aux noeuds)
	*** @param integer $vulId vulnerability DB ID
	*** @retval array['rows'] = list of vulnerabilities detected, array['total'] = number of rows
	*** @throws EsiaDbException
	**/
	public static function ts_getVulDetectByVulId($dbconnection, $userId, $vulId, $TSParams=NULL)
	{
		$sql = "SELECT *, count(*) OVER() AS total_count FROM (
			SELECT node.device_id, node.device_name AS node_name, host(node.device_ip_address) AS device_ip_address,
				eg_service_name, eg_service_product, eg_service_port, eg_service_port_type,
				detect_id, detect_date, to_char(detect_date, 'DD-MM-YYYY HH24:MI:SS') AS detect_date_format,
				vul_iface_label, vul_type_name, vul_type_factor, vul_type_color,
				box.device_name AS box_name,
				node_group.device_groups_list
			FROM esia_device AS node
			JOIN egide_services ON device_id = eg_device_id
			JOIN egide_vulnerability_detect ON detect_eg_service_id = eg_service_id
			JOIN egide_vulnerability_detect_state ON state_id = detect_state_id
			JOIN egide_service_access ON eg_access_service_id = eg_service_id
			JOIN vulnerability_iface ON eg_access_iface_id = vul_iface_id
			JOIN vulnerability_type ON vul_type_id = vul_iface_type_id
			JOIN esia_device AS box ON vul_iface_agent_id = box.device_id
			JOIN (SELECT group_in_device_id, STRING_AGG(device_group_path, ',' ORDER BY device_group_path ASC) AS device_groups_list
				FROM esia_device_group
				LEFT JOIN esia_device_group_in ON group_in_device_group_id = device_group_id
				GROUP BY group_in_device_id
			) AS node_group ON node_group.group_in_device_id = node.device_id
			WHERE node.device_id IN (SELECT * FROM getUserAccessNode(:user_id))
			AND node.device_delete = FALSE
			AND state_name = 'ACTIVE'
			AND detect_vul_id = :vul_id
		) AS tmp";

		$params = array(
			':user_id' => $userId,
			':vul_id' => $vulId,
		);

		$columns = array(
			array('col' => 'detect_date_format', 'type' => 'date', 'data' => 'detect_date'),
			array('col' => 'device_groups_list', 'type' => 'string'),
			array('col' => 'node_name', 'type' => 'string'),
			array('col' => 'device_ip_address', 'type' => 'ipaddress'),
			array('col' => 'eg_service_name', 'type' => 'string'),
			array('col' => 'eg_service_product', 'type' => 'string'),
			array('col' => 'eg_service_port', 'type' => 'integer'),
			array('col' => 'eg_service_port_type', 'type' => 'string'),
			array('col' => 'box_name', 'type' => 'string'),
			array('col' => 'vul_iface_label', 'type' => 'string'),
			array('col' => 'vul_type_name', 'type' => 'string'),
		);

		$sql_order = " ORDER BY detect_date DESC";
		Model_TableSort::ts_getActiveErrors_filter($columns, $TSParams, $sql, $params, $sql_order);

		$query = $dbconnection->prepare($sql . $sql_order);
		$query = Model_esia::executeStatement($query, $params);
		$total = 0;

		$vulList = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			if($total == 0)
			{
				// Fetch total rows only the first time
				$total = $row->total_count;
			}

			$vulList[] = array(
				'device_id' => $row->device_id,
				'device_name' => $row->node_name,
				'device_ip' => $row->device_ip_address,
				'device_groups_list' => $row->device_groups_list,
				'eg_srv_name' => $row->eg_service_name,
				'eg_srv_product' => $row->eg_service_product,
				'eg_srv_port' => $row->eg_service_port,
				'eg_srv_port_type' => $row->eg_service_port_type,
				'detect_date' => $row->detect_date_format,
				'if_label' => $row->vul_iface_label,
				'if_type_name' => $row->vul_type_name,
				'if_type_factor' => $row->vul_type_factor,
				'if_type_color' => $row->vul_type_color,
				'box_name' => $row->box_name,
			);
        }

		return array('total' => $total, 'rows' => $vulList);
	}

	private static function ts_executeRequest($dbconnection,$sql,$param,$columns,$TSParams,$default_order)
	{
		$sql = "FROM ($sql) as tabletmp";
		$sql_order = "";
		$flag = Model_TableSort::ts_getActiveErrors_filter($columns,$TSParams,$sql,$param,$sql_order);
		if($flag === FALSE)
		{
			//il n'y a pas de modification effectuée et donc, on met la valeur par défaut;
			$sql_order = " ORDER BY $default_order";
		}
		$total = 0;

		$queryCount = $dbconnection->prepare('SELECT count(*) as nbr '.$sql);
		$queryCount = Model_esia::executeStatement($queryCount,$param);
		$row = $queryCount->fetch(PDO::FETCH_OBJ);
		$total = $row->nbr;

		$query = $dbconnection->prepare('SELECT * '.$sql.' '.$sql_order);
		$query = Model_esia::executeStatement($query,$param);

		return array('total'=>$total,'response'=>$query);
	}
}
?>
