INSERT INTO esia_web_plugin(web_id, web_name, web_dir_lang, web_php_menu , web_index_page, web_activate, web_tpl_method, web_plugin_apache_path,web_show_main_menu,web_icon) VALUES (DEFAULT,'WEBP_EGIDESCAN','lang/','menu_egidescan.php','control_egidescan.php',true, 'text', '/plugin/egidescan/',true,'/plugin/egidescan/images/menu_egide.png');

CREATE TABLE IF NOT EXISTS public.egide_provider
(
    provider_id serial NOT NULL,
    provider_name character varying(50) COLLATE pg_catalog."default",
    CONSTRAINT "provide_primaryKey" PRIMARY KEY (provider_id)
);
ALTER TABLE IF EXISTS public.egide_provider OWNER to esia;


CREATE TABLE IF NOT EXISTS public.egide_services
(
    eg_service_id bigserial,
    eg_service_name character varying(100) NOT NULL,
    eg_service_product character varying(100),
    eg_service_version character varying(30),
    eg_service_port integer,
    eg_service_cpe text COLLATE pg_catalog."default",
    eg_service_extrainfo text COLLATE pg_catalog."default",
    eg_device_id bigint NOT NULL,
    eg_service_port_type character varying(5) COLLATE pg_catalog."default" DEFAULT 'tcp'::character varying,
    CONSTRAINT "eg_service_primaryKey" PRIMARY KEY (eg_service_id),
    CONSTRAINT "eg_service_ForeingKey_device_id" FOREIGN KEY (eg_device_id)
        REFERENCES public.esia_device (device_id) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
        NOT VALID
);
ALTER TABLE IF EXISTS public.egide_services OWNER to esia;

CREATE TABLE IF NOT EXISTS public.egide_vulnerability
(
    vul_id bigserial NOT NULL,
    vul_provider_id bigint,
    vul_list_id character varying(30),
    vul_description text,
    vul_url json,
    vul_date_publishing timestamp without time zone,
    vul_scoring_version character varying(20),
    vul_scoring real,
    vul_severity character varying(20),
    vul_attack_complexity character varying(20) ,
    vul_user_state character varying,
    vul_user_message text,
    vul_status character varying(20),
    vul_date_last_modify timestamp without time zone,
    CONSTRAINT "vul_primaryKey" PRIMARY KEY (vul_id),
    CONSTRAINT "vul_foreignKey_provider" FOREIGN KEY (vul_provider_id)
        REFERENCES public.egide_provider (provider_id) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION,
    CONSTRAINT vul_id_vul_list_unique UNIQUE (vul_id,vul_list_id)
);
ALTER TABLE IF EXISTS public.egide_vulnerability OWNER to esia;
CREATE TABLE IF NOT EXISTS public.egide_vulnerability_detect
(
    detect_id serial PRIMARY KEY,
    detect_eg_service_id integer NOT NULL,
    detect_vul_id bigint NOT NULL,
    detect_date timestamp without time zone DEFAULT now(),
    detect_date_update_state timestamp without time zone,
    detect_state character varying(10) COLLATE pg_catalog."default" DEFAULT 'ACTIVE'::character varying,
    detect_user_message text COLLATE pg_catalog."default",
    detect_username character varying(50) COLLATE pg_catalog."default",
    CONSTRAINT "detect_fk_uniq" UNIQUE(detect_eg_service_id, detect_vul_id),
    CONSTRAINT "detect_foreignKey_eg_service" FOREIGN KEY (detect_eg_service_id)
        REFERENCES public.egide_services (eg_service_id) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
        NOT VALID,
    CONSTRAINT "detect_foreignKey_vulnerability" FOREIGN KEY (detect_vul_id)
        REFERENCES public.egide_vulnerability (vul_id) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
);

ALTER TABLE IF EXISTS public.egide_vulnerability_detect OWNER to esia;

--INSERT INTO public.egide_provider(provider_id, provider_name) VALUES (DEFAULT, 'NIST');

-----By Julien
CREATE TABLE IF NOT EXISTS vulnerability_type(
    vul_type_id serial,
    vul_type_name character varying(20) NOT NULL,
    vul_type_factor real NOT NULL,
    vul_type_color character varying(10) NOT NULL,
    CONSTRAINT vul_type_id_primary_key PRIMARY KEY (vul_type_id),
    CONSTRAINT vul_type_name_unique UNIQUE (vul_type_name)
);
ALTER TABLE vulnerability_type OWNER TO esia;


CREATE TABLE IF NOT EXISTS vulnerability_iface(
   vul_iface_id bigserial,
   vul_iface_agent_id bigint NOT NULL ,
   vul_iface_local_name character varying(30) NOT NULL,
   vul_iface_label character varying(30) NOT NULL,
   vul_iface_type_id integer NOT NULL,
   CONSTRAINT vul_iface_id_primaryKey PRIMARY KEY (vul_iface_id),
   CONSTRAINT vul_iface_agent_id_ForeignKey_type_id FOREIGN KEY (vul_iface_agent_id) REFERENCES esia_device(device_id),
   CONSTRAINT vul_iface_type_id_ForeignKey_type_id FOREIGN KEY (vul_iface_type_id) REFERENCES vulnerability_type(vul_type_id)
);
ALTER TABLE vulnerability_iface OWNER TO esia;

INSERT INTO vulnerability_type(vul_type_name,vul_type_factor,vul_type_color)
VALUES ('DMZ',0.9,'#FF0000');
INSERT INTO vulnerability_type(vul_type_name,vul_type_factor,vul_type_color)
VALUES ('Guest',0.7,'#FF9C20');
INSERT INTO vulnerability_type(vul_type_name,vul_type_factor,vul_type_color)
VALUES ('User',0.5,'#FFCC00');
INSERT INTO vulnerability_type(vul_type_name,vul_type_factor,vul_type_color)
VALUES ('Server',0.3,'#D1FF00');
INSERT INTO vulnerability_type(vul_type_name,vul_type_factor,vul_type_color)
VALUES ('MGMT',0.1,'#00E020');

CREATE TABLE IF NOT EXISTS egide_service_access
(
    eg_access_iface_id bigserial,
    eg_access_service_id bigserial,
    CONSTRAINT "eg_access_iface_id_ForeignKey_iface_id" FOREIGN KEY (eg_access_iface_id) REFERENCES vulnerability_iface(vul_iface_id),
    CONSTRAINT "eg_access_service_id_ForeignKey_service_id" FOREIGN KEY (eg_access_service_id) REFERENCES egide_services(eg_service_id),
    CONSTRAINT "eg_access_primaryKey" PRIMARY KEY (eg_access_iface_id,eg_access_service_id)
);
ALTER TABLE IF EXISTS egide_service_access OWNER to esia;

CREATE TABLE IF NOT EXISTS public.vulnerability_scanner (
	scan_id bigserial NOT NULL,
	scan_state character varying(20),
	scan_agent_id bigint,
    	scan_iface_id bigint,
	scan_option json,
	scan_cron_exec bigint,
	CONSTRAINT "vulscan_primaryKey" PRIMARY KEY (scan_id),
    	CONSTRAINT scan_iface_id_ForeignKey_iface_id FOREIGN KEY (scan_iface_id) REFERENCES vulnerability_iface(vul_iface_id),
	CONSTRAINT "scan_cron_exec_ForeignKey_cron_id" FOREIGN KEY (scan_cron_exec) REFERENCES esia_cron_plugin(cron_id)
);
ALTER TABLE IF EXISTS vulnerability_scanner OWNER to esia;
--END BY Julien

-- object: public.vulnerability_scanner_history | type: TABLE --
-- DROP TABLE IF EXISTS public.vulnerability_scanner_history CASCADE;
CREATE TABLE IF NOT EXISTS public.vulnerability_scanner_history (
	scan_history_id bigserial NOT NULL,
	scan_date_start timestamp,
	scan_date_end timestamp,
	scan_state character varying(20),
	scan_id bigint,
	scan_option json,
    scan_result json,
	CONSTRAINT "vulscan_history_primaryKey" PRIMARY KEY (scan_history_id)
);
-- ddl-end --
ALTER TABLE public.vulnerability_scanner_history OWNER TO esia;
-- ddl-end --

-- ddl-end --

-- object: "history_foreignKey_scan_id" | type: CONSTRAINT --
-- ALTER TABLE public.vulnerability_scanner_history DROP CONSTRAINT IF EXISTS "history_foreignKey_scan_id" CASCADE;
ALTER TABLE public.vulnerability_scanner_history ADD CONSTRAINT "history_foreignKey_scan_id" FOREIGN KEY (scan_id)
REFERENCES public.vulnerability_scanner (scan_id) MATCH SIMPLE
ON DELETE NO ACTION ON UPDATE NO ACTION;
-- ddl-end --


INSERT INTO device_tab(
tab_id, tab_name, tab_default_options, tab_url, tab_order, tab_web_id)
VALUES (DEFAULT,'TAB_EGIDESCAN',NULL,'adv_tab_egidescan.php?nodeid=:id', 100,
      (SELECT web_id FROM esia_web_plugin WHERE web_name='WEBP_EGIDESCAN'));


INSERT INTO esia_plugins (plugin_type,plugin_name,plugin_path,plugin_default_parameter,plugin_default_interval_time,plugin_cron_type,plugin_cron_only_existing)
VALUES ('cron','CRON_EGIDE_SCAN_GET','/usr/local/esia/public_html/plugin/egidescan/cron/egide_scanner_plugin.php','-m get',60000,'minute',TRUE);

INSERT INTO esia_plugins (plugin_type,plugin_name,plugin_path,plugin_default_parameter,plugin_default_interval_time,plugin_cron_type,plugin_cron_only_existing)
VALUES ('cron','CRON_EGIDE_SCAN_START','/usr/local/esia/public_html/plugin/egidescan/cron/egide_scanner_plugin.php','-m start -i <scan_id>',900000,'hour',FALSE);

INSERT INTO esia_cron_plugin (
    cron_plugin_id,
    cron_interval,
    cron_status,
    cron_next_exec,
    cron_log_file,
    cron_retries,
    cron_parameter
)
VALUES (
    (SELECT plugin_id FROM esia_plugins WHERE plugin_name = 'CRON_EGIDE_SCAN_GET'),
    '1 minute'::interval,
    'PENDING',
    NOW() + interval '1 minute',
    'cron_egide_scan_get_$SERVICEID_$DATE.log.log',
    0,
    '-m get'
);

ALTER TABLE egide_vulnerability ADD COLUMN vul_access_type character varying(17);
ALTER TABLE egide_vulnerability ADD COLUMN vul_vendor_comments json;
ALTER TABLE egide_vulnerability ADD COLUMN vul_cpe json;
ALTER TABLE egide_vulnerability ADD COLUMN vul_last_update timestamp without time zone;

CREATE TABLE IF NOT EXISTS public.egide_cpe_vendor
(
    vendor_id bigserial,
    vendor_name character varying(100),
    CONSTRAINT "egide_cpe_vendor_primaryKey" PRIMARY KEY (vendor_id),
    CONSTRAINT "vendor_name_NameUNique" UNIQUE (vendor_name)
);
ALTER TABLE IF EXISTS public.egide_cpe_vendor OWNER to esia;

CREATE TABLE IF NOT EXISTS public.egide_cpe_product
(
    product_id bigserial,
    product_name character varying(100),
    product_type character varying(17),
    product_vendor_id bigint,
    CONSTRAINT "egide_cpe_product_primaryKey" PRIMARY KEY (product_id),
    CONSTRAINT "product_vendor_id_ForeignKey_vendor_id" FOREIGN KEY (product_vendor_id) REFERENCES egide_cpe_vendor(vendor_id),
    CONSTRAINT "product_name_NameVendor_Unique" UNIQUE (product_name,product_type,product_vendor_id)
);
ALTER TABLE IF EXISTS public.egide_cpe_product OWNER to esia;

CREATE TABLE IF NOT EXISTS public.egide_cpe_vulnerability
(
    cpe_id bigserial,
    cpe_vul_id bigint,
    cpe_product_id bigint,
    cpe_start_version_exclude character varying(100),
    cpe_start_version_include character varying(100),
    cpe_equal_version character varying(100),
    cpe_end_version_include character varying(100),
    cpe_end_version_exclude character varying(100),
    cpe_origin  character varying(200),
    CONSTRAINT "egide_cpe_vulnerability_primaryKey" PRIMARY KEY (cpe_id),
    CONSTRAINT "cpe_vul_id_ForeignKey_vul_id" FOREIGN KEY (cpe_vul_id) REFERENCES egide_vulnerability(vul_id),
    CONSTRAINT "cpe_product_id_ForeignKey_cpe_product_id" FOREIGN KEY (cpe_product_id) REFERENCES egide_cpe_product(product_id),
    CONSTRAINT "egide_cpe_vulnerability_uniq" UNIQUE(cpe_vul_id,cpe_origin,cpe_start_version_exclude,cpe_start_version_include,cpe_end_version_include,cpe_end_version_exclude)
);
ALTER TABLE IF EXISTS public.egide_cpe_vulnerability OWNER to esia;

CREATE OR REPLACE FUNCTION compare_versions(version1 text, version2 text)
RETURNS integer AS
$$
DECLARE
    parts1 text[];
    parts2 text[];
    counter integer := 1;
BEGIN
    -- Divisez les numéros de version en parties majeures et mineures.
    parts1 := string_to_array(version1, '.');
    parts2 := string_to_array(version2, '.');

    LOOP
        IF parts1[counter] IS NULL AND parts2[counter] IS NULL THEN
            RETURN 0;
        ELSEIF parts1[counter] IS NULL THEN
            RETURN -1;
        ELSEIF parts2[counter] IS NULL THEN
            RETURN 1;
        ELSEIF parts1[counter]  ~ '^[0-9]+$' AND parts2[counter] !~ '^[0-9]+$' THEN
            RETURN -1;
        ELSIF parts1[counter] !~ '^[0-9]+$' AND parts2[counter] ~ '^[0-9]+$' THEN
            RETURN 1;
        ELSIF parts1[counter] !~ '^[0-9]+$' AND parts2[counter] !~ '^[0-9]+$' THEN     
            IF parts1[counter] < parts2[counter] THEN
                RETURN -1;
            ELSIF parts1[counter] > parts2[counter] THEN
                RETURN 1;
            END IF;
        ELSIF parts1[counter]::int < parts2[counter]::int THEN
            RETURN -1;
        ELSIF parts1[counter]::int > parts2[counter]::int THEN
            RETURN 1;
        END IF;
        counter := counter + 1;
    END LOOP;
END;
$$
LANGUAGE plpgsql; 

INSERT INTO public.egide_provider(provider_id, provider_name) VALUES (1, 'NIST');
